@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #include <sys/param.h>
 #include <ctype.h>
 #include <string.h>
@@ -247,6 +224,29 @@ int parse_ACC_pragma()
 	}
     }
 
+    if(PG_IS_IDENT("routine")){
+	pg_ACC_pragma = ACC_ROUTINE;
+	pg_get_token();
+	if(pg_tok == '('){
+	    CExpr *x;
+	    if((x = parse_ACC_namelist()) == NULL) 
+		goto syntax_err;
+	    if((pg_ACC_list = parse_ACC_clauses()) == NULL)  goto syntax_err;
+	    pg_ACC_list = exprListCons(ACC_PG_LIST(ACC_ROUTINE_ARG,x), pg_ACC_list);
+	} else {
+	  if((pg_ACC_list = parse_ACC_clauses()) == NULL)  goto syntax_err;
+	}
+	ret= PRAGMA_EXEC;
+	goto chk_end;
+    }
+    
+    if(PG_IS_IDENT("atomic")){
+      pg_ACC_pragma = ACC_ATOMIC;
+      pg_get_token();
+      if((pg_ACC_list = parse_ACC_clauses()) == NULL) goto syntax_err;
+      goto chk_end;
+    }
+
     addError(NULL,"ACC: unknown ACC directive, '%s'",pg_tok_buf);
   syntax_err:
     return 0;
@@ -336,6 +336,10 @@ static CExpr* parse_ACC_clauses()
 	  pg_get_token();
 	  if((v = parse_ACC_namelist()) == NULL) goto syntax_err;
 	  c = ACC_PG_LIST(ACC_DEV_RESIDENT,v);
+      } else if(PG_IS_IDENT("bind")){
+	  pg_get_token();
+	  if((v = parse_ACC_namelist()) == NULL) goto syntax_err;
+	  c = ACC_PG_LIST(ACC_BIND,v);
       } else if(PG_IS_IDENT("reduction")){
 	  pg_get_token();
 	  if((v = parse_ACC_reduction_namelist(&r)) == NULL) goto syntax_err;
@@ -386,6 +390,21 @@ static CExpr* parse_ACC_clauses()
 	} else if(PG_IS_IDENT("independent")){
 	    pg_get_token();
 	    c = ACC_PG_LIST(ACC_INDEPENDENT,NULL);
+	} else if(PG_IS_IDENT("read")){
+	    pg_get_token();
+	    c = ACC_PG_LIST(ACC_READ,NULL);
+	} else if(PG_IS_IDENT("write")){
+	    pg_get_token();
+	    c = ACC_PG_LIST(ACC_WRITE,NULL);
+	} else if(PG_IS_IDENT("update")){
+	    pg_get_token();
+	    c = ACC_PG_LIST(ACC_UPDATE_CLAUSE,NULL);
+	} else if(PG_IS_IDENT("capture")){
+	    pg_get_token();
+	    c = ACC_PG_LIST(ACC_CAPTURE,NULL);
+	} else if(PG_IS_IDENT("nohost")){
+	    pg_get_token();
+	    c = ACC_PG_LIST(ACC_NOHOST,NULL);
 	} else {
 	  addError(NULL,"unknown ACC directive clause '%s'",pg_tok_buf);
 	    goto syntax_err;
