@@ -1,24 +1,7 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 #ifndef _C_ACC_H
@@ -40,6 +23,8 @@ enum ACC_pragma {
 
     ACC_ENTER_DATA  = 211,
     ACC_EXIT_DATA   = 212,
+    ACC_ATOMIC          = 213, 
+    ACC_ROUTINE         = 214,
 
     ACC_DIR_END
 };
@@ -91,6 +76,15 @@ enum ACC_pragma_clause {
 
     ACC_HOST,
     ACC_DEVICE,
+
+    ACC_READ,
+    ACC_WRITE,
+    ACC_UPDATE_CLAUSE,
+    ACC_CAPTURE,
+
+    ACC_BIND,
+    ACC_NOHOST,
+    ACC_ROUTINE_ARG,
 };
 
 void out_ACC_PRAGMA(FILE *fp, int indent, int pragma_code, CExpr* expr);
