@@ -597,11 +597,11 @@ addSymbolAt(CExprOfSymbol *sym, CExpr *parent, CSymbolTable *symTab,
     assertExpr((CExpr*)sym, symTab != NULL);
 
     switch(symType) {
-    case ST_TYPE:
+    case ST_TYPE: 
     case ST_FUNC:
-    case ST_VAR:
+    case ST_VAR: 
     case ST_PARAM:
-    case ST_ENUM:
+    case ST_ENUM: 
         ht = &symTab->stb_identGroup;
         group = STB_IDENT;
         break;
@@ -672,8 +672,15 @@ addSymbolAt(CExprOfSymbol *sym, CExpr *parent, CSymbolTable *symTab,
                         td->e_preDeclType = htd;
                     }
                     break;
+		case ST_TYPE:
+		    // -----------------
+		    // typedef int hoge;
+		    // typedef hoge hoge;
+		    // -----------------
+		    // Above pattern is ignore for FX100.
+		  break;
                 default:
-                    addRedeclError(sym);
+		    addRedeclError(sym);
                     return;
                 }
                 checkAndMergeIdent(hsym, sym, ht);
@@ -1318,16 +1325,19 @@ allocExprOfNumberConst(CExprCodeEnum exprCode, CBasicTypeEnum bt,
 	      case BT_CHAR:
 		if(n > UCHAR_MAX) overRange1 = 1;
 		else if(n > SCHAR_MAX) bt = BT_UNSIGNED_CHAR;
-		overRange1 = 1;
+		break;
 	      case BT_SHORT:
 		if(n > USHRT_MAX) overRange1 = 1;
 		else if(n > SHRT_MAX) bt = BT_UNSIGNED_SHORT;
+		break;
 	      case BT_INT:
 		if(n > UINT_MAX) overRange1 = 1;
 		else if(n > INT_MAX) bt = BT_UNSIGNED_INT;
+		break;
 	      case BT_LONG:
-		if(n > ULONG_MAX) overRange1 = 1;
+		if(n > (unsigned long long)ULONG_MAX) overRange1 = 1;  //cast avoids a bug when using PGI compiler
 		else if(n > LONG_MAX) bt = BT_UNSIGNED_LONG;
+		break;
 	      default:
 		break;
 	      }
@@ -1864,6 +1874,37 @@ allocExprOfList5(CExprCodeEnum exprCode, CExpr *expr1, CExpr *expr2,
 }
 
 
+/**
+ * \brief
+ * alloc CExprOfList and 6 children
+ *
+ * @param exprCode
+ *      expression code
+ * @param expr1
+ *      child node
+ * @param expr2
+ *      child node
+ * @param expr3
+ *      child node
+ * @param expr4
+ *      child node
+ * @param expr5
+ *      child node
+ * @param expr6
+ *      child node
+ * @return
+ *      allocated node
+ */
+CExprOfList*
+allocExprOfList6(CExprCodeEnum exprCode, CExpr *expr1, CExpr *expr2,
+		 CExpr *expr3, CExpr *expr4, CExpr *expr5, CExpr *expr6)
+{
+  CExprOfList *expr = allocExprOfList5(exprCode, expr1, expr2, expr3, expr4, expr5);
+  exprListAdd((CExpr*)expr, expr6);
+  return expr;
+}
+
+
 /**
  * \brief
  * free CExprOfList
