@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file c-expr.h
  * - defines AST element types.
@@ -1166,6 +1143,8 @@ extern CExprOfList*           allocExprOfList4(CExprCodeEnum exprCode,
                                     CExpr *e1, CExpr *e2, CExpr *e3, CExpr *e4);
 extern CExprOfList*           allocExprOfList5(CExprCodeEnum exprCode,
 					       CExpr *e1, CExpr *e2, CExpr *e3, CExpr *e4, CExpr *e5);
+extern CExprOfList*           allocExprOfList6(CExprCodeEnum exprCode,
+					       CExpr *e1, CExpr *e2, CExpr *e3, CExpr *e4, CExpr *e5, CExpr *e6);
 extern CExprOfNumberConst*    allocExprOfNumberConst(CExprCodeEnum exprCode,
                                     CBasicTypeEnum btype, CCardinalEnum c, char *orgToken);
 extern CExprOfNumberConst*    allocExprOfNumberConst1(CNumValueWithType *nvt);
