@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #include "c-expr.h"
 #include "c-option.h"
 #include "c-comp.h"
@@ -432,10 +409,12 @@ endCheckGccAttr(CExpr *expr)
     }
 
     CCOL_SListNode *ite1, *ite2;
-    CCOL_SL_FOREACH(ite1, &gac->gac_attrArgs) {
+    if(&gac->gac_attrArgs != NULL)
+      CCOL_SL_FOREACH(ite1, &gac->gac_attrArgs) {
         int exists = 0;
         CExpr *attr1 = (CExpr*)CCOL_SL_DATA(ite1);
-        CCOL_SL_FOREACH(ite2, &ngac.gac_attrArgs) {
+	if(&ngac.gac_attrArgs != NULL)
+	  CCOL_SL_FOREACH(ite2, &ngac.gac_attrArgs) {
             CExpr *attr2 = (CExpr*)CCOL_SL_DATA(ite2);
             if(attr1 == attr2) {
                 exists = 1;
@@ -490,7 +469,8 @@ void
 checkGccAttrOutput()
 {
     CCOL_DListNode *ite;
-
+    
+    if(&s_typeDescList != NULL)
     CCOL_DL_FOREACH(ite, &s_typeDescList) {
         CExprOfTypeDesc *td = EXPR_T(CCOL_DL_DATA(ite));
         if(td->e_isGccAttrDuplicated == 0) {
