@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #include <stdlib.h>
 #include <stdarg.h> 
 #include <wchar.h>
@@ -96,6 +73,8 @@ void outx_XMP_Clause(FILE *fp, int indent, CExprOfList* clauseList)
     case XMP_DATA_REDUCE_EQV:
     case XMP_DATA_REDUCE_NEQV:
     case XMP_DATA_REDUCE_MINUS:
+    case XMP_DATA_REDUCE_MAXLOC:
+    case XMP_DATA_REDUCE_MINLOC:
       outxPrint(fp, indent1+1, "<intConstant type=\"int\">%d</intConstant>\n", clauseList->e_aux);
       break;
 
@@ -128,6 +107,7 @@ char *xmpDirectiveName(int c)
   case XMP_DISTRIBUTE:   return "DISTRIBUTE";
   case XMP_ALIGN:        return "ALIGN";
   case XMP_SHADOW:       return "SHADOW";
+  case XMP_STATIC_DESC:  return "STATIC_DESC";
   case XMP_TASK:         return "TASK";
   case XMP_TASKS:        return "TASKS";
   case XMP_LOOP:         return "LOOP";
@@ -140,9 +120,11 @@ char *xmpDirectiveName(int c)
   case XMP_ARRAY:        return "ARRAY";
   case XMP_POST:         return "POST";
   case XMP_WAIT:         return "WAIT";
+  case XMP_LOCK:         return "LOCK";
+  case XMP_UNLOCK:       return "UNLOCK";
   case XMP_TEMPLATE_FIX: return "TEMPLATE_FIX";
   case XMP_REFLECT_INIT: return "REFLECT_INIT";
-  case XMP_REFLECT_DO: return "REFLECT_DO";
+  case XMP_REFLECT_DO:   return "REFLECT_DO";
   case XMP_WAIT_ASYNC:   return "WAIT_ASYNC";
   default:               return "OMP???";
   }
