@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #include "F-front.h"
 
 expv XMP_check_TASK(expr x);
@@ -61,6 +38,7 @@ typedef enum _xmp_list_context {
 expv XMP_compile_subscript_list(expr list,xmp_list_context context);
 expv XMP_compile_ON_ref(expr x);
 expv XMP_compile_clause_opt(expr x);
+expv XMP_compile_list(expr l);
 
 int XMP_reduction_op(expr v)
 {
@@ -201,6 +179,11 @@ void compile_XMP_directive(expr x)
       output_statement(XMP_pragma_list(XMP_LOCAL_ALIAS,c,NULL));
       break;
 
+    case XMP_SAVE_DESC:
+      check_INDCL();
+      output_statement(XMP_pragma_list(XMP_SAVE_DESC,c,NULL));
+      break;
+
     case XMP_TASK:
       check_INEXEC();
       /* check arg: node_ref opt */
@@ -228,7 +211,8 @@ void compile_XMP_directive(expr x)
       check_INEXEC();
       /* check arg: no arg */
       push_ctl(CTL_XMP);
-      CTL_XMP_ARG(ctl_top) = x;
+      //CTL_XMP_ARG(ctl_top) = x;
+      CTL_XMP_ARG(ctl_top) = XMP_pragma_list(XMP_TASKS, EMPTY_LIST, NULL);
       EXPR_LINE(CTL_XMP_ARG(ctl_top)) = current_line;
       break;
 
@@ -238,7 +222,7 @@ void compile_XMP_directive(expr x)
 	 CTL_XMP_ARG_DIR(ctl_top) == XMP_TASKS){
 	CURRENT_STATEMENTS = XMP_check_TASK(CURRENT_STATEMENTS);
 	CTL_BLOCK(ctl_top) = 
-	  XMP_pragma_list(XMP_TASK,CTL_XMP_ARG_CLAUSE(ctl_top),
+	  XMP_pragma_list(XMP_TASKS, CTL_XMP_ARG_CLAUSE(ctl_top),
 			  CURRENT_STATEMENTS);
 	EXPR_LINE(CTL_BLOCK(ctl_top)) = EXPR_LINE(CTL_XMP_ARG(ctl_top));
 	pop_ctl();
@@ -291,30 +275,34 @@ void compile_XMP_directive(expr x)
       output_statement(x);
       break;
 
-    case XMP_REDUCTION:
+    case XMP_REDUCTION: {
       check_INEXEC();
-      output_statement(x);
+      expr o = EXPR_ARG1(EXPR_ARG1(c)); // operator
+      expr l = EXPR_ARG2(EXPR_ARG1(c)); // variable/loc, .../, ...
+      x1 = list2(LIST, o, l); // (operator variables...)
+      x2 = XMP_compile_ON_ref(EXPR_ARG2(c)); // on
+      x3 = compile_expression(EXPR_ARG3(c)); // async
+      c = list3(LIST, x1, x2, x3);
+      output_statement(XMP_pragma_list(XMP_REDUCTION, c, NULL));
       break;
+    }
 
     case XMP_BCAST:
       check_INEXEC();
-      output_statement(x);
+      x1 = XMP_compile_list(EXPR_ARG1(c)); // variables
+      x2 = XMP_compile_ON_ref(EXPR_ARG2(c)); // on
+      x3 = XMP_compile_ON_ref(EXPR_ARG3(c)); // from
+      x4 = compile_expression(EXPR_ARG4(c)); // async
+      c = list4(LIST, x1, x2, x3, x4);
+      output_statement(XMP_pragma_list(XMP_BCAST, c, NULL));
       break;
 
     case XMP_WAIT_ASYNC:
       check_INEXEC();
-      output_statement(x);
-      /* x2 = XMP_compile_ON_ref(EXPR_ARG2(c)); */
-      /* expr c0 = list0(LIST); */
-      /* list lp; */
-      /* FOR_ITEMS_IN_LIST(lp, EXPR_ARG1(c)){ */
-      /*   x1 = LIST_ITEM(lp); */
-      /* 	x1 = compile_expression(x1); */
-      /* 	c0 = list_put_last(c0, x1); */
-      /* } */
-      /* c = list1(LIST,c0); */
-      /* //output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c,NULL)); */
-      /* output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c, x2)); */
+      x1 = XMP_compile_list(EXPR_ARG1(c)); // tags
+      x2 = XMP_compile_ON_ref(EXPR_ARG2(c)); // on
+      c = list2(LIST, x1, x2);
+      output_statement(XMP_pragma_list(XMP_WAIT_ASYNC,c,NULL));
       break;
 
     case XMP_TEMPLATE_FIX:
@@ -328,7 +316,11 @@ void compile_XMP_directive(expr x)
 
     case XMP_COARRAY:
       check_INDCL();
-      /* not yet */
+      output_statement(x);
+      break;
+
+    case XMP_IMAGE:
+      check_INEXEC();
       output_statement(x);
       break;
 
@@ -548,37 +540,65 @@ close_XMP_IO_closure(int st_no, expr x) {
     return ret;
 }
 
+
 expv XMP_check_TASK(expr x)
 {
-    expr xx;
-    expv task_list,current_task;
-    list lp;
+  expr xx;
+  expv task_list;
+  list lp;
     
-    if(x == NULL) return NULL;
-    if(EXPR_CODE(x) != LIST) fatal("XMP_check_SECION: not LIST");
+  if (x == NULL) return NULL;
+  if (EXPR_CODE(x) != LIST) fatal("XMP_check_TASK: not LIST");
 
-    task_list = EMPTY_LIST;
-    current_task = NULL;
-    FOR_ITEMS_IN_LIST(lp,x){
-	xx = LIST_ITEM(lp);
-	if(EXPR_CODE(xx) == XMP_PRAGMA &&
-	   EXPR_INT(EXPR_ARG1(xx)) == XMP_TASK){
-	    if(current_task != NULL)
-		task_list = list_put_last(task_list,current_task);
-	    current_task = EMPTY_LIST;
-	    continue;
-	}
-	if(current_task == NULL){
-	    error_at_node(xx,"statement is not in any TASK");
-	    return NULL;
-	}
-	current_task = list_put_last(current_task,xx);
+  task_list = EMPTY_LIST;
+
+  FOR_ITEMS_IN_LIST(lp,x){
+    xx = LIST_ITEM(lp);
+    if (EXPR_CODE(xx) == XMP_PRAGMA &&
+	EXPR_INT(EXPR_ARG1(xx)) == XMP_TASK){
+      task_list = list_put_last(task_list, xx);
     }
-    task_list = list_put_last(task_list,current_task);
-    return task_list;
+    else {
+      error_at_node(xx,"statement is not in any TASK");
+      return NULL;
+    }
+  }
+
+  return task_list;
 }
 
 
+/* expv XMP_check_TASK(expr x) */
+/* { */
+/*     expr xx; */
+/*     expv task_list,current_task; */
+/*     list lp; */
+    
+/*     if(x == NULL) return NULL; */
+/*     if(EXPR_CODE(x) != LIST) fatal("XMP_check_TASK: not LIST"); */
+
+/*     task_list = EMPTY_LIST; */
+/*     current_task = NULL; */
+/*     FOR_ITEMS_IN_LIST(lp,x){ */
+/* 	xx = LIST_ITEM(lp); */
+/* 	if(EXPR_CODE(xx) == XMP_PRAGMA && */
+/* 	   EXPR_INT(EXPR_ARG1(xx)) == XMP_TASK){ */
+/* 	    if(current_task != NULL) */
+/* 		task_list = list_put_last(task_list,current_task); */
+/* 	    current_task = EMPTY_LIST; */
+/* 	    continue; */
+/* 	} */
+/* 	if(current_task == NULL){ */
+/* 	    error_at_node(xx,"statement is not in any TASK"); */
+/* 	    return NULL; */
+/* 	} */
+/* 	current_task = list_put_last(current_task,xx); */
+/*     } */
+/*     task_list = list_put_last(task_list,current_task); */
+/*     return task_list; */
+/* } */
+
+
 expv XMP_compile_subscript_list(expr l,xmp_list_context context)
 {
     expr x;
@@ -850,3 +870,19 @@ expv XMP_compile_clause_opt(expr x)
     return x; /* nothing at this moment */
 }
 
+expv XMP_compile_list(expr l)
+{
+  expr x, v;
+  list lp;
+
+  expv ret_list = EMPTY_LIST;
+
+  FOR_ITEMS_IN_LIST(lp, l){
+    x = LIST_ITEM(lp);
+    v = compile_expression(x);
+    ret_list = list_put_last(ret_list, v);
+  }
+
+  return ret_list;
+
+}
