@@ -1,31 +1,9 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file F-compile-expr.c
  */
 
 #include "F-front.h"
+#include "F-second-pass.h"
 #include <math.h>
 
 static expv compile_args _ANSI_ARGS_((expr args));
@@ -456,7 +434,7 @@ compile_expression(expr x)
                     return compile_substr_ref(x);
                 } else {
                     if(id)
-                        error("%s is not array nor character", ID_NAME(id));
+                        error("%s is not an array nor a character", ID_NAME(id));
                     else
                         error("not array nor character", ID_NAME(id));
                     goto err;
@@ -587,9 +565,16 @@ compile_expression(expr x)
             bLType = bottom_type(lt);
             bRType = bottom_type(rt);
 
-            if (TYPE_IS_NOT_FIXED(bLType) || TYPE_IS_NOT_FIXED(bRType)) {
+/* FEAST change start */
+            /* if (TYPE_IS_NOT_FIXED(bLType) || TYPE_IS_NOT_FIXED(bRType)) { */
+            /*     type_is_not_fixed = TRUE; */
+            /* } */
+            if(bLType == NULL || bRType == NULL){
+                type_is_not_fixed = TRUE;
+            } else if (TYPE_IS_NOT_FIXED(bLType) || TYPE_IS_NOT_FIXED(bRType)) {
                 type_is_not_fixed = TRUE;
             }
+/* FEAST change end */
 
             switch (biop) {
             case ARITB:
@@ -684,6 +669,12 @@ compile_expression(expr x)
                 /*
                  * First of all, check if which side is reshaped type.
                  */
+/* FEAST add start */
+                if(!lt || !rt){
+                  type_is_not_fixed = TRUE;
+                  goto doEmit;
+                }
+/* FEAST add end */                
                 if (TYPE_IS_RESHAPED(lt) || TYPE_IS_RESHAPED(rt)) {
                     if (bType == NULL) {
                         tp = TYPE_IS_RESHAPED(rt) ? lt : rt;
@@ -782,7 +773,10 @@ compile_expression(expr x)
             }
 
             doEmit:
-            if (type_is_not_fixed)
+/* FEAST CHANGE start */
+            /* if (type_is_not_fixed) */
+            if(type_is_not_fixed && tp)
+/* FEAST CHANGE end */
                 TYPE_SET_NOT_FIXED(bottom_type(tp));
             return expv_cons(op, tp, left, right);
         }
@@ -953,9 +947,15 @@ compile_expression(expr x)
                 }
                 EXPV_TYPE(v) = ID_TYPE(id);
             }
-            if(!expv_is_specification(v))
-                error_at_node(EXPR_ARG1(x),
-                    "character string length must be integer.");
+/* FEAST change start */
+            /* if(!expv_is_specification(v)) */
+            /*     error_at_node(EXPR_ARG1(x), */
+            /*         "character string length must be integer."); */
+            if(!expv_is_specification(v)){
+              EXPV_TYPE(v) = NULL;
+              sp_link_expr((expr)v, 2, current_line);
+            }
+/* FEAST change  end  */
             return v;
         }
         case PLUS_EXPR:
@@ -1121,7 +1121,12 @@ compile_ident_expression(expr x)
 
     done:
     if (ret == NULL) {
-        fatal("%s: invalid code", __func__);
+/* FEAST change start */
+        /* fatal("%s: invalid code", __func__); */
+      EXPV_TYPE(x) = NULL;
+      ret = (expv)x;
+      sp_link_expr((expr)x, 3, current_line);
+/* FEAST change  end  */
     }
 
     return ret;
@@ -1446,6 +1451,12 @@ expv_assignment(expv v1, expv v2)
     TYPE_DESC tp1 = EXPV_TYPE(v1);
     TYPE_DESC tp2 = EXPV_TYPE(v2);
 
+/* FEAST add start */
+    if(!tp1 || !tp2){
+      return NULL;
+    }
+/* FEAST add end */
+
     if (EXPV_CODE(v2) == F95_ARRAY_CONSTRUCTOR) {
         if (!IS_ARRAY_TYPE(tp1)) {
             error("lhs expr is not an array.");
@@ -1630,7 +1641,10 @@ compile_array_ref(ID id, expv vary, expr args, int isLeft) {
 
     nDims = TYPE_N_DIM(tp);
 
-    if (!IS_ARRAY_TYPE(tp)) fatal("%s: not ARRAY_TYPE", __func__);
+    if (!IS_ARRAY_TYPE(tp)){ //fatal("%s: not ARRAY_TYPE", __func__);
+      error_at_id(id, "%s: not ARRAY_TYPE", ID_NAME(id));
+      return NULL;
+    }
     if (!TYPE_DIM_FIXED(tp)) fix_array_dimensions(tp);
 
     /*
@@ -2005,7 +2019,7 @@ compile_coarray_ref(expr coarrayRef){
   tp = get_rightmost_id_type(obj);
   if (!tp) return NULL;
   if (!tp->codims){
-    error_at_node(coarrayRef, "Only coarrays can be coindexed.");
+    error_at_node(coarrayRef, "The variable is not declared as a coarray.");
     return NULL;
   }
 
@@ -2076,7 +2090,7 @@ compile_coarray_ref(expr coarrayRef){
   }
 
   if (tp->codims->corank != n){
-    error_at_node(image_selector, "wrong number of cosubscript on '%s'");
+    error_at_node(image_selector, "wrong number of cosubscript.");
     return NULL;
   }
 
@@ -2371,6 +2385,12 @@ compile_function_call(ID f_id, expr args) {
             /* f_id is not defined yet. */
             tp = (ID_TYPE(f_id) != NULL) ? ID_TYPE(f_id) : new_type_desc();
             TYPE_SET_USED_EXPLICIT(tp);
+/* FEAST add start */
+            if(tp->basic_type == TYPE_UNKNOWN){
+              /* ID_TYPE(f_id) = NULL; */
+              sp_link_id(f_id, 4, current_line);
+            }
+/* FEAST add  end  */
 
             a = compile_args(args);
 #if 0
