@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file F-front.h
  */
@@ -32,7 +9,6 @@
 
 #define OMNI_FORTRAN_ENTRY_POINT        "__Omni_FortranEntryPoint"
 
-
 #include "config.h"
 #include <assert.h>
 #if SIZEOF_UNSIGNED_INT == 4
@@ -375,6 +351,8 @@ extern int leave_comment_flag;
 
 #define EMPTY_LIST list0(LIST)
 
+#define NAME_FOR_NONAME_PROGRAM "xmpf_main"
+
 /*
  * I/O information specifiers
  */
@@ -791,6 +769,7 @@ extern expr     create_implicit_decl_expv(TYPE_DESC tp, char * first, char * sec
 
 extern void compile_OMP_directive(expr v);
 int OMP_reduction_op(expr v);
+int OMP_depend_op(expr v);
 
 extern void compile_XMP_directive(expr v);
 int XMP_reduction_op(expr v);
