@@ -1,32 +1,8 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file F-intrinsic.c
  */
 
 #include "F-front.h"
-
 #include "F-intrinsics-types.h"
 
 #define isValidType(tp)         \
@@ -100,7 +76,37 @@ compile_intrinsic_call(ID id, expv args) {
     EXT_ID extid;
 
     if (SYM_TYPE(ID_SYM(id)) != S_INTR) {
-        fatal("%s: not intrinsic symbol", __func__);
+      //fatal("%s: not intrinsic symbol", __func__);
+
+      // declarea as intrinsic but not defined in the intrinc table
+
+      SYM_TYPE(ID_SYM(id)) = S_INTR;
+
+      if (args == NULL) {
+        args = list0(LIST);
+      }
+
+      if (ID_TYPE(id) == NULL) implicit_declaration(id);
+      tp = ID_TYPE(id);
+      //tp = BASIC_TYPE_DESC(TYPE_SUBR);
+
+      expv symV = expv_sym_term(F_FUNC, NULL, ID_SYM(id));
+
+      ftp = function_type(tp);
+      TYPE_SET_INTRINSIC(ftp);
+
+      extid = new_external_id_for_external_decl(ID_SYM(id), ftp);
+      ID_TYPE(id) = ftp;
+      PROC_EXT_ID(id) = extid;
+      if (TYPE_IS_EXTERNAL(tp)){
+	ID_STORAGE(id) = STG_EXT;
+      }
+      else{
+	EXT_PROC_CLASS(extid) = EP_INTRINSIC;
+      }
+
+      ret = expv_cons(FUNCTION_CALL, tp, symV, args);
+      return ret;
     }
 
     ep = &(intrinsic_table[SYM_VAL(ID_SYM(id))]);
@@ -171,7 +177,8 @@ compile_intrinsic_call(ID id, expv args) {
             }
             tp = EXPV_TYPE(a);
             if (!(isValidType(tp))) {
-                return NULL;    /* error recovery */
+	      //return NULL;    /* error recovery */
+	      continue;
             }
             if (compare_intrinsic_arg_type(a, tp,
                                            ((isVarArgs == 0) ?
@@ -199,8 +206,9 @@ compile_intrinsic_call(ID id, expv args) {
         if (INTR_RETURN_TYPE(ep) != INTR_TYPE_NONE) {
             tp = get_intrinsic_return_type(ep, args, kindV);
             if (!(isValidType(tp))) {
-                fatal("%s: can't determine return type.", __func__);
-                return NULL;
+	      //fatal("%s: can't determine return type.", __func__);
+	      //return NULL;
+	      tp = BASIC_TYPE_DESC(TYPE_GNUMERIC_ALL);
             }
         } else {
             tp = BASIC_TYPE_DESC(TYPE_SUBR);
