@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 /**
  * \file F-intrinsics-table.c
  */
@@ -335,7 +312,7 @@ intrinsic_entry intrinsic_table[] = {
     { INTR_SELECTED_INT_KIND,   INTR_NAME_GENERIC,      "selected_int_kind",    0,      {INTR_TYPE_INT},        INTR_TYPE_INT,  1, -6, LANGSPEC_F90 },
 
     // SELECTED_REAL_KIND ([P, R])
-    { INTR_SELECTED_REAL_KIND,  INTR_NAME_GENERIC,      "selected_real_kind",   0,      {},                             INTR_TYPE_INT,  0, -6, LANGSPEC_F90 },
+    { INTR_SELECTED_REAL_KIND,  INTR_NAME_GENERIC,      "selected_real_kind",   0,      {INTR_TYPE_NONE},               INTR_TYPE_INT,  0, -6, LANGSPEC_F90 },
     { INTR_SELECTED_REAL_KIND,  INTR_NAME_GENERIC,      "",                     0,      {INTR_TYPE_INT},                INTR_TYPE_INT,  1, 0, LANGSPEC_F90 },
     { INTR_SELECTED_REAL_KIND,  INTR_NAME_GENERIC,      "",                     0,      {INTR_TYPE_INT, INTR_TYPE_INT}, INTR_TYPE_INT,  2, 0, LANGSPEC_F90 },
 
@@ -666,7 +643,7 @@ intrinsic_entry intrinsic_table[] = {
     /* 21. Intrinsic subroutines */
 
     // DATE_AND_TIME ([DATE, TIME, ZONE, VALUES])
-    { INTR_DATE_AND_TIME,       INTR_NAME_GENERIC,      "date_and_time",        0,      {},                                                     INTR_TYPE_NONE, 0, -1, LANGSPEC_F90 },
+    { INTR_DATE_AND_TIME,       INTR_NAME_GENERIC,      "date_and_time",        0,      { INTR_TYPE_NONE },                                     INTR_TYPE_NONE, 0, -1, LANGSPEC_F90 },
     { INTR_DATE_AND_TIME,       INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_CHAR },                                     INTR_TYPE_NONE, 1, -1, LANGSPEC_F90 },
     { INTR_DATE_AND_TIME,       INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_CHAR, INTR_TYPE_CHAR },                     INTR_TYPE_NONE, 2, -1, LANGSPEC_F90 },
     { INTR_DATE_AND_TIME,       INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_CHAR, INTR_TYPE_CHAR, INTR_TYPE_CHAR },     INTR_TYPE_NONE, 3, -1, LANGSPEC_F90 },
@@ -680,7 +657,7 @@ intrinsic_entry intrinsic_table[] = {
     { INTR_RANDOM_NUMBER,      INTR_NAME_GENERIC,      "random_number",        0,      { INTR_TYPE_ALL_REAL },                                 INTR_TYPE_NONE, 1, -1, LANGSPEC_F90 },
 
     // RANDOM_SEED ([SIZE, PUT, GET])
-    { INTR_RANDOM_SEED,         INTR_NAME_GENERIC,      "random_seed",          0,      {},                                                     INTR_TYPE_NONE, 0, -1, LANGSPEC_F90 },
+    { INTR_RANDOM_SEED,         INTR_NAME_GENERIC,      "random_seed",          0,      { INTR_TYPE_NONE },                                     INTR_TYPE_NONE, 0, -1, LANGSPEC_F90 },
     { INTR_RANDOM_SEED,         INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT },                                      INTR_TYPE_NONE, 1, -1, LANGSPEC_F90 },
     { INTR_RANDOM_SEED,         INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT, INTR_TYPE_INT_ARRAY },                  INTR_TYPE_NONE, 2, -1, LANGSPEC_F90 },
     { INTR_RANDOM_SEED,         INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT, INTR_TYPE_INT_ARRAY, INTR_TYPE_INT_ARRAY },     INTR_TYPE_NONE, 3, -1, LANGSPEC_F90 },
@@ -688,7 +665,7 @@ intrinsic_entry intrinsic_table[] = {
 
 
     // SYSTEM_CLOCK ([COUNT, COUNT_RATE, COUNT_MAX])
-    { INTR_SYSTEM_CLOCK,        INTR_NAME_GENERIC,      "system_clock",         0,      {},                                             INTR_TYPE_NONE, 0, -6, LANGSPEC_F90 },
+    { INTR_SYSTEM_CLOCK,        INTR_NAME_GENERIC,      "system_clock",         0,      { INTR_TYPE_NONE },                             INTR_TYPE_NONE, 0, -6, LANGSPEC_F90 },
     { INTR_SYSTEM_CLOCK,        INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT },                              INTR_TYPE_NONE, 1, -6, LANGSPEC_F90 },
     { INTR_SYSTEM_CLOCK,        INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT, INTR_TYPE_INT },               INTR_TYPE_NONE, 2, -6, LANGSPEC_F90 },
     { INTR_SYSTEM_CLOCK,        INTR_NAME_GENERIC,      "",                     0,      { INTR_TYPE_INT, INTR_TYPE_INT, INTR_TYPE_INT },        INTR_TYPE_NONE, 3, -6, LANGSPEC_F90 },
@@ -725,32 +702,49 @@ intrinsic_entry intrinsic_table[] = {
     /* 20. Pointer association status functions */
 
     // NULL ([MOLD])
-    { INTR_NULL,       INTR_NAME_GENERIC,      "null",          0,      { },                                    INTR_TYPE_LHS, 0, -7, LANGSPEC_F95 },
+    { INTR_NULL,       INTR_NAME_GENERIC,      "null",          0,      { INTR_TYPE_NONE },                     INTR_TYPE_LHS, 0, -7, LANGSPEC_F95 },
     { INTR_NULL,       INTR_NAME_GENERIC,      "",              0,      { INTR_TYPE_ANY_ARRAY_ALLOCATABLE },    INTR_TYPE_TARGET, 1, 0,  LANGSPEC_F95 },
     { INTR_NULL,       INTR_NAME_GENERIC,      "",              0,      { INTR_TYPE_POINTER },                  INTR_TYPE_TARGET, 1, 0,  LANGSPEC_F95 },
 
-
     /* 21. Intrinsic subroutines */
 
     // CPU_TIME (TIME)
     { INTR_CPU_TIME,    INTR_NAME_GENERIC,      "cpu_time",     0,      { INTR_TYPE_REAL },                                    INTR_TYPE_NONE, 1, -6, LANGSPEC_F95 },
     { INTR_CPU_TIME,    INTR_NAME_GENERIC,      "",             0,      { INTR_TYPE_DREAL },                                   INTR_TYPE_NONE, 1, -6, LANGSPEC_F95 },
 
+    /*
+     * CAF1.0 (subset of Fortran2008) intrinic functions
+     * This list should match with XcodeML-Exc-Tools/src/exc/xmpF/XMPtransCoarrayRun.java
+     */
+
+    { INTR_NUM_IMAGES,    INTR_NAME_GENERIC,   "num_images",        0,      {},                               INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_THIS_IMAGE,    INTR_NAME_GENERIC,   "this_image",        0,      {INTR_TYPE_ANY},                  INTR_TYPE_INT, -1, -8, LANGSPEC_NONSTD },
+    { INTR_IMAGE_INDEX,   INTR_NAME_GENERIC,   "image_index",       0,      {INTR_TYPE_ANY},                  INTR_TYPE_INT, -1, -8, LANGSPEC_NONSTD },
+    { INTR_LCOBOUND,      INTR_NAME_GENERIC,   "lcobound",          0,      {INTR_TYPE_ANY},                  INTR_TYPE_INT, -1, -8, LANGSPEC_NONSTD },
+    { INTR_UCOBOUND,      INTR_NAME_GENERIC,   "ucobound",          0,      {INTR_TYPE_ANY},                  INTR_TYPE_INT, -1, -8, LANGSPEC_NONSTD },
+
+    /* hidden interfaces for debugging */
+    { INTR_COARRAY_ALLOCATED_BYTES,   INTR_NAME_GENERIC,   "xmpf_coarray_allocated_bytes",     0,   {},   INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_COARRAY_GARBAGE_BYTES,     INTR_NAME_GENERIC,   "xmpf_coarray_garbage_bytes",       0,   {},   INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+
+
+    /* XMP/F */
+
     { INTR_DESC_OF,        INTR_NAME_GENERIC,      "xmp_desc_of",         0,      {INTR_TYPE_ANY},       INTR_TYPE_DREAL,  1, -8, LANGSPEC_NONSTD },
 
-    { INTR_GET_MPI_COMM,        INTR_NAME_GENERIC,      "xmp_get_mpi_comm",         0,      {},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_GET_MPI_COMM,        INTR_NAME_GENERIC,      "xmp_get_mpi_comm",         0,      {INTR_TYPE_NONE},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_NUM_NODES,        INTR_NAME_GENERIC,      "xmp_num_nodes",         0,      {},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_NUM_NODES,        INTR_NAME_GENERIC,      "xmp_num_nodes",         0,      {INTR_TYPE_NONE},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_NODE_NUM,         INTR_NAME_GENERIC,      "xmp_node_num",          0,      {},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_NODE_NUM,         INTR_NAME_GENERIC,      "xmp_node_num",          0,      {INTR_TYPE_NONE},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_ALL_NUM_NODES,        INTR_NAME_GENERIC,      "xmp_all_num_nodes",         0,      {},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_ALL_NUM_NODES,        INTR_NAME_GENERIC,      "xmp_all_num_nodes",         0,      {INTR_TYPE_NONE},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_ALL_NODE_NUM,         INTR_NAME_GENERIC,      "xmp_all_node_num",          0,      {},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
+    { INTR_ALL_NODE_NUM,         INTR_NAME_GENERIC,      "xmp_all_node_num",          0,      {INTR_TYPE_NONE},       INTR_TYPE_INT,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_WTIME,         INTR_NAME_GENERIC,      "xmp_wtime",          0,      {},       INTR_TYPE_DREAL,  0, -8, LANGSPEC_NONSTD },
+    { INTR_WTIME,         INTR_NAME_GENERIC,      "xmp_wtime",          0,      {INTR_TYPE_NONE},       INTR_TYPE_DREAL,  0, -8, LANGSPEC_NONSTD },
 
-    { INTR_WTICK,         INTR_NAME_GENERIC,      "xmp_wtick",          0,      {},       INTR_TYPE_DREAL,  0, -8, LANGSPEC_NONSTD },
+    { INTR_WTICK,         INTR_NAME_GENERIC,      "xmp_wtick",          0,      {INTR_TYPE_NONE},       INTR_TYPE_DREAL,  0, -8, LANGSPEC_NONSTD },
 
     { INTR_ARRAY_NDIMS,         INTR_NAME_GENERIC,      "xmp_array_ndims",          0,      {INTR_TYPE_ANY, INTR_TYPE_INT},       INTR_TYPE_INT, 2, -8, LANGSPEC_NONSTD },
 
