@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 /**
@@ -275,6 +258,16 @@ typedef enum {
     /* F95 intrinsic subroutines. */
     INTR_CPU_TIME,
 
+    /* CAF1.0 (F2008) intrinsic functions */
+    INTR_NUM_IMAGES,
+    INTR_THIS_IMAGE,
+    INTR_IMAGE_INDEX,
+    INTR_LCOBOUND,
+    INTR_UCOBOUND,
+    INTR_COARRAY_ALLOCATED_BYTES,    // hidden interface
+    INTR_COARRAY_GARBAGE_BYTES,      // hidden interface
+
+    /* XMP/F */
     INTR_DESC_OF,
     INTR_GET_MPI_COMM,
     INTR_NUM_NODES,
