@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package xcodeml.util;
@@ -62,6 +45,11 @@ public class XmOption
     /** backend compiler vendor */
     private static int _compilerVendor = COMP_VENDOR_GNU;
     
+    /** if supressing generation of USE statement for coarray runtime
+     *  (TEMPORARY)
+     */
+    private static boolean _coarrayNoUseStatement = false;  
+
     private XmOption()
     {
     }
@@ -289,4 +277,18 @@ public class XmOption
     {
         _isAtomicIO = atomicIO;
     }
+
+    /**
+     * Set/get suboption -fcoarray-no-use-statement (boolean)
+     */
+    public static void setCoarrayNoUseStatement(boolean coarrayNoUseStatement)
+    {
+        _coarrayNoUseStatement = coarrayNoUseStatement;
+    }
+
+    public static boolean coarrayNoUseStatement()
+    {
+        return _coarrayNoUseStatement;
+    }
+
 }
