@@ -1,31 +1,19 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.block;
 
 import java.io.*;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.Vector;
 
 import exc.object.*;
+import exc.openmp.OMPanalyzeDecl;
+import exc.openmp.OMPpragma;
 
 /**
  * A object to represent a function definition. Note that it is not a kind of
@@ -42,12 +30,11 @@ public class FuncDefBlock
     public FuncDefBlock(XobjectDef def)
     {
         this.def = def;
-        fblock = (FunctionBlock)def.getProp(funcBlockProp); // check cache
-        if(fblock == null) {
-            def.setDef(canonicalizeExpr(def.getDef()));
-            fblock = Bcons.buildFunctionBlock(def);
-            def.setProp(funcBlockProp, fblock);
-        }
+	// System.out.println("getDef="+def.getDef());
+	def.setDef(canonicalizeExpr(def.getDef()));
+	fblock = Bcons.buildFunctionBlock(def);
+	// System.out.println("fblock="+fblock);
+	def.setProp(funcBlockProp, fblock);
     }
 
     /** make clone */
@@ -59,7 +46,7 @@ public class FuncDefBlock
 
     /** constructor with fucntion name, id_list, decls, body and env. */
     public FuncDefBlock(Xobject name, Xobject id_list, Xobject decls, BlockList body,
-        Xobject gcc_attrs, XobjectFile env)
+			Xobject gcc_attrs, XobjectFile env)
     {
         // make dummy XobjectDef
         this.def = XobjectDef.Func(name, id_list, decls, body.toXobject());
@@ -89,8 +76,11 @@ public class FuncDefBlock
 
     public void Finalize()
     {
-        // System.out.println("-- Finalize:"); print();
-        def.setDef(fblock.toXobject());
+	if(fblock != null)
+	    def.setDef(fblock.toXobject());
+	else 
+	    def.setDef(null);
+        // System.out.println("-- Finalize:="+def.getDef());
         def.remProp(funcBlockProp); // flush cache
     }
 
@@ -143,4 +133,67 @@ public class FuncDefBlock
         debug_out.print(fblock);
         debug_out.flush();
     }
+
+  @Override
+  public String toString()
+  {
+    return "[FuncDefBlock:" + fblock + "]";
+  }
+
+    public void searchCommonMember(String name,OMPanalyzeDecl env,XobjectDef d)
+    {
+	Map <String,XobjList> common_db= new HashMap<String,XobjList>();
+	Vector <Xobject> xobjVector= new Vector<Xobject>();
+	XobjList nameList = new XobjList();
+
+	topdownBlockIterator bi = new topdownBlockIterator(this.fblock);
+	for(bi.init();!bi.end();bi.next())
+	    {
+		XobjectIterator xi = new topdownXobjectIterator(bi.getBlock().toXobject());
+		for(xi.init();!xi.end();xi.next())
+		    {
+			Xobject xobj = xi.getXobject();
+			Xobject x=null;
+			if(xobj!=null)
+			    switch (xobj.Opcode())
+				{
+				case F_COMMON_DECL:
+				    {
+					String keyIdent =xobj.getArgOrNull(0).getArgOrNull(0).getName().toString();
+					for(XobjArgs args = xobj.getArgOrNull(0).getArgOrNull(1).getArgs(); args!=null ;args= args.nextArgs())
+					    {
+						x = args.getArg().getArgOrNull(0);
+						nameList.add(args.getArg().getArgOrNull(0));
+						xobjVector.add(x);
+					    }
+					common_db.put(keyIdent, nameList);
+					nameList = new XobjList();
+				    }
+				default:
+				}
+		    }
+
+	    }
+	topdownBlockIterator bi2 = new topdownBlockIterator(this.fblock);
+	for(bi2.init();!bi2.end();bi2.next())
+	    {
+		if(bi2.getBlock().toXobject().Opcode()==Xcode.OMP_PRAGMA ){
+		    {
+			XobjectIterator xi2 = new topdownXobjectIterator(bi2.getBlock().toXobject());
+			for(xi2.init();!xi2.end();xi2.next())
+			    {
+				if(xi2.getXobject()!=null  && xi2.getXobject().Opcode()==Xcode.LIST && xi2.getXobject().getArg(0).Opcode()==Xcode.STRING && OMPpragma.valueOf(xi2.getXobject().getArg(0)) == OMPpragma.DATA_COPYIN)
+				    {
+					for(int i=0;i<xobjVector.size();i++)
+					    {
+						for(XobjArgs args2 = xi2.getXobject().getArg(1).getArgs();args2!=null ;args2=args2.nextArgs())
+						    if(xobjVector.get(i).getName().equals(args2.getArg().getName()))
+							env.addThdprvVars(d.findIdent(args2.getArg().getName()));
+					    }
+				    }
+			    }
+		    }
+		}
+	    }
+    }
 }
