@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.block;
@@ -46,12 +29,13 @@ public class bottomupBlockIterator extends BlockIterator
     public void init()
     {
         index = 0;
-        currentBlock = blocks.elementAt(0);
+        if(blocks != null) currentBlock = blocks.elementAt(0);
     }
 
     @Override
     public void init(Block b)
     {
+	if(b == null) return;
         blocks = new Vector<Block>();
         bottomupTraverse(b);
         index = 0;
@@ -60,6 +44,7 @@ public class bottomupBlockIterator extends BlockIterator
 
     public void init(BlockList body)
     {
+	if(body == null) return;
         blocks = new Vector<Block>();
         bottomupTraverse(body);
         index = 0;
@@ -91,6 +76,7 @@ public class bottomupBlockIterator extends BlockIterator
     @Override
     public void next()
     {
+	if(currentBlock == null) return;
         if(++index >= blocks.size())
             currentBlock = null;
         else
@@ -100,12 +86,14 @@ public class bottomupBlockIterator extends BlockIterator
     @Override
     public boolean end()
     {
+	if(currentBlock == null) return true;
         return index >= blocks.size();
     }
 
     @Override
     public int size()
     {
+	if(blocks == null) return 0;
         return blocks.size();
     }
 }
