@@ -1,28 +1,5 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 package exc.object;
-import exc.block.Block;
+import exc.block.*;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -171,7 +148,6 @@ public class Ident extends Xobject
             return name;
     }
 
-    @Override
     public int getFrank()
     {
         return type.getNumDimensions();
@@ -241,6 +217,7 @@ public class Ident extends Xobject
         return (Type() == null) ? false : Type().wasCoarray();
     }
 
+
     public boolean isDeclared()
     {
         return declared;
@@ -300,10 +277,33 @@ public class Ident extends Xobject
     @Override
     public Xobject cfold(Block block)
     {
-      if (fparam_value != null) {
-        Xobject that = ((XobjList)fparam_value).args.arg;
-        return that.cfold(block);
+      if (Type().isFparameter() && fparam_value != null) {
+        // I don't know why but fparam_value is always in this form.
+        if (fparam_value.Nargs() == 2 && fparam_value.getArg(1) == null) {
+          Xobject value = fparam_value.getArg(0);
+          return value.cfold(block);
+        } else {
+          XmLog.fatal("Ident.cfold: unknown form of fparam_value");
+        }
       }
+
+      if (declared_module != null) {
+        XobjectDefEnv xobjDefEnv = ((FunctionBlock)block).getEnv();
+        XobjectFile xobjFile = (XobjectFile)xobjDefEnv;
+
+        if (xobjFile.findVarIdent(declared_module) == null)
+          XmLog.fatal("Ident.cfold: not found module name in globalSymbols: " + 
+                      declared_module);
+
+        for (XobjectDef punit: xobjFile.getDefs()) {
+          if (declared_module.equals(punit.getName())) {
+            // found the module that declares this ident
+            Ident ident2 = punit.getDef().findVarIdent(name);
+            return ident2.cfold(block);
+          }
+        }
+      }
+
       return this.copy();
     }
 
