@@ -1,32 +1,7 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 package exc.object;
 
 import java.math.BigInteger;
-
 import exc.block.Block;
-
 import xcodeml.util.XmOption;
 import static xcodeml.util.XmLog.fatal;
 
@@ -223,6 +198,7 @@ public class Xcons
             type = type.getRef();
         switch(x.Opcode()) {
         case ARRAY_ADDR:
+            return Xcons.Symbol(Xcode.ARRAY_ADDR, type, x.getSym());
         case VAR_ADDR:
             return Xcons.Symbol(Xcode.VAR, type, x.getSym());
         case FUNC_ADDR:
@@ -900,6 +876,11 @@ public class Xcons
         return FindexRange(lb, ub, null);
     }
     
+    public static Xobject FindexRange(Xobject ub)
+    {
+        return FindexRange(Xcons.IntConstant(1), ub, null);
+    }
+    
     public static Xobject FindexRangeOfAssumedShape()
     {
         return Xcons.List(Xcode.F_INDEX_RANGE, (Xobject)null, null, null, IntConstant(1));
