@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.object;
@@ -76,20 +59,86 @@ public class XobjString extends XobjConst
     @Override
     public Xobject cfold(Block block)
     {
-      if (value == null) 
-        return this.copy();
-
-      Xobject id_list = block.getBody().getIdentList();
-      if (id_list == null)
-        return this.copy();
-
-      Ident ident = id_list.findVarIdent(value);
+      Ident ident = findIdent(block);
       if (ident == null)
         return this.copy();
 
       return ident.cfold(block);
     }
 
+    @Override
+    public Xobject getSubscripts()
+    {
+      switch (Opcode()) {
+      case VAR:
+        return null;
+      default:
+        break;
+      }
+      throw new UnsupportedOperationException(toString());
+    }
+
+    @Override
+    public int getFrank(Block block)
+    {
+      switch (Opcode()) {
+      case VAR:
+        Ident ident = findIdent(block);
+        if (ident == null)
+          throw new UnsupportedOperationException(toString());
+        return ident.getFrank();
+      default:
+        return 0;       // scalar
+      }
+    }
+
+    public Ident findIdent(Block block)
+    {
+      if (value == null) 
+        return null;
+
+      Xobject id_list = block.getBody().getIdentList();
+      if (id_list == null)
+        return null;
+
+      Ident ident = id_list.findVarIdent(value);
+      return ident;
+    }
+
+
+    public Xobject lbound(int dim, Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.lbound(dim);
+    }
+    public Xobject ubound(int dim, Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.ubound(dim);
+    }
+    public Xobject extent(int dim, Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.extent(dim);
+    }
+
+    public Xobject[] lbounds(Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.lbounds();
+    }
+    public Xobject[] ubounds(Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.ubounds();
+    }
+    public Xobject[] extents(Block block)
+    {
+      Fshape shape = new Fshape(this, block);
+      return shape.extents();
+    }
+
+
     @Override
     public Xobject copy()
     {
