@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 package exc.object;
 
 import java.util.LinkedList;
@@ -51,6 +28,10 @@ public class XobjectDefEnv extends PropObject
     
     public void setIdentList(Xobject identList)
     {
+      ///////////////////////////////////////////////
+      //System.out.println("+++ XobjectDefEnv.setIdentList(identList): identList=");
+      //System.out.println(identList);
+      ///////////////////////////////////////////////
         this.identList = identList;
     }
 
@@ -151,7 +132,10 @@ public class XobjectDefEnv extends PropObject
                         if(id.getStorageClass() == StorageClass.FTYPE_NAME ||
                             id.getStorageClass() == StorageClass.TAGNAME)
                             return id;
-                        break;
+                    case IXobject.FINDKIND_TYPEDEF_NAME:
+                      if(id.getStorageClass() == StorageClass.TYPEDEF_NAME)
+                        return id;
+                      break;
                     }
                 }
             }
