@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 package exc.object;
 
 import exc.block.Block;
@@ -238,8 +215,8 @@ public class Xtype
         new TypeInfo(doubleType,            "double",               null,            "Freal"),
         new TypeInfo(longdoubleType,        "long_double",          null,               "Freal"),
         new TypeInfo(boolType,              "bool",                 "Flogical",         "Flogical"),
-        new TypeInfo(floatComplexType,      "float_complex",        null,               "Fcomplex"),
-        new TypeInfo(doubleComplexType,     "double_complex",       "Fcomplex",         "Fcomplex"),
+        new TypeInfo(floatComplexType,      "float_complex",        "Fcomplex",         "Fcomplex"),
+        new TypeInfo(doubleComplexType,     "double_complex",       null,               "Fcomplex"),
         new TypeInfo(longDoubleComplexType, "long_double_complex",  null,               "Fcomplex"),
         new TypeInfo(gccBuiltinVaListType,  "__builtin_va_list",    null,               null),
         new TypeInfo(FcharacterType,        null,                   "Fcharacter",       "Fcharacter"),
@@ -683,7 +660,7 @@ public class Xtype
         throw new UnsupportedOperationException();
     }
 
-    /** Fortran: get function result name */
+    /** Fortran: set function result name */
     public void setFuncResultName(String fresult_name)
     {
         throw new UnsupportedOperationException();
