@@ -9,11 +9,8 @@ import xcodeml.util.XmLog;
 public class ACC {
   public final static String prop = "_ACC_PROP_";
   
-  public static boolean debugFlag = true;
   private static boolean errorFlag      = false;
   
-  public static boolean useReadOnlyDataCache = true;
-
   public static final int ACC_ASYNC_SYNC = -1;
   public static final int ACC_ASYNC_NOVAL = -2;
   public static final int HOST_TO_DEVICE = 400;
@@ -27,10 +24,9 @@ public class ACC {
   public static final String PRESENT_OR_COPY_DATA_FUNC_NAME = "_ACC_pcopy_data";
   public static final String FIND_DATA_FUNC_NAME = "_ACC_find_data";
 
-
-  public static Xobject createBasicTypeConstantObj(Xtype type) {
-    return Xcons.IntConstant(type.getBasicType() + 500);
-  }
+  public static int version = 20;
+  public static Platform platform = Platform.CUDA;
+  public static AccDevice device = AccDevice.NONE;
 
   public static void exitByError() {
     if (errorFlag) System.exit(1);
@@ -45,6 +41,10 @@ public class ACC {
     XmLog.warning("[OpenACC] " + msg);
   }
 
+  public static void warning(LineNo l, String msg){
+    XmLog.warning(l, "[OpenACC] " + msg);
+  }
+
   public static void fatal(String msg) {
     XmLog.fatal("[OpenACC] " + msg);
   }
@@ -53,13 +53,8 @@ public class ACC {
     XmLog.debug("[OpenACC] " + msg);
   }
 
-  public static Ident getMacroId(String name) {
-    return new Ident(name, StorageClass.EXTERN, Xtype.Function(Xtype.voidType),
-                     Xcons.Symbol(Xcode.FUNC_ADDR, Xtype.Pointer(Xtype.Function(Xtype.voidType)), name), VarScope.GLOBAL);
-  }
-
-  public static Ident getMacroId(String name, Xtype type) {
-    return new Ident(name, StorageClass.EXTERN, Xtype.Function(type),
-                     Xcons.Symbol(Xcode.FUNC_ADDR, Xtype.Pointer(Xtype.Function(type)), name), VarScope.GLOBAL);
+  public enum Platform{
+    CUDA,
+    OpenCL,
   }
 }
