@@ -1,25 +1,7 @@
 package exc.openacc;
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -29,7 +11,7 @@ import exc.object.*;
 import java.io.*;
 import java.util.*;
 
-public class ACCgpuDecompileWriter extends PrintWriter {
+class ACCgpuDecompileWriter extends PrintWriter {
   private XobjectFile _env = null;
 
   public ACCgpuDecompileWriter(Writer out, XobjectFile env) {
@@ -58,9 +40,9 @@ public class ACCgpuDecompileWriter extends PrintWriter {
     printWithIdentList(def.getDef(), _env.getGlobalIdentList(), false, null);
   }
   
-  public void printFunc(XobjectDef def){
+  void printFunc(XobjectDef def){
     String funcName = def.getName();
-    boolean isDeviceFunc = funcName.endsWith(ACCgpuKernel.ACC_GPU_DEVICE_FUNC_SUFFIX);
+    boolean isDeviceFunc = funcName.endsWith(AccKernel.ACC_GPU_DEVICE_FUNC_SUFFIX);
     printWithIdentList(def.getDef(), _env.getGlobalIdentList(), isDeviceFunc, (Ident)def.getNameObj());
   }
   
@@ -136,12 +118,20 @@ public class ACCgpuDecompileWriter extends PrintWriter {
               func_args += ")";
             }
             if (isDeviceFunc) {
-              println("__global__ static");
+              switch (ACC.platform){
+                case CUDA:
+                  println("__global__ static");
+                  break;
+                case OpenCL:
+                  println("__kernel");
+                  break;
+              }
             } else {
               println("extern \"C\"");
             }
             printDeclType(id.Type().getRef(), funcName + func_args);
           } else {
+            //FIXME what is this part?
             func_args = "(";
             if (v.getArg(1) != null) {
               for (XobjArgs a = v.getArg(1).getArgs(); a != null; a = a.nextArgs()) {
@@ -441,8 +431,20 @@ public class ACCgpuDecompileWriter extends PrintWriter {
       if (prop != null) {
         println();
         println("{");
-        println("dim3 _ACC_GPU_DIM3_block(" + prop.getArg(0).getName() + ", " + prop.getArg(1).getName() + ", " + prop.getArg(2).getName() + ");");
-        println("dim3 _ACC_GPU_DIM3_thread(" + prop.getArg(3).getName() + ", " + prop.getArg(4).getName() + ", " + prop.getArg(5).getName() + ");");
+        print("dim3 _ACC_GPU_DIM3_block(");
+        print(prop.getArg(0));
+        print(",");
+        print(prop.getArg(1));
+        print(",");
+        print(prop.getArg(2));
+        println(");");
+        print("dim3 _ACC_GPU_DIM3_thread(");
+        print(prop.getArg(3));
+        print(",");
+        print(prop.getArg(4));
+        print(",");
+        print(prop.getArg(5));
+        println(");");
       }
 
       print(funcName);
@@ -960,7 +962,14 @@ public class ACCgpuDecompileWriter extends PrintWriter {
   private void printIdentDecl(Ident id) {
     Object isShared = id.getProp(ACCgpuDecompiler.GPU_STRAGE_SHARED);
     if(isShared != null){
-      print("__shared__ ");
+      switch(ACC.platform){
+        case CUDA:
+          print("__shared__ ");
+          break;
+        case OpenCL:
+          print("__local ");
+          break;
+      }
     }
     switch (id.getStorageClass()) {
       case AUTO:
@@ -1128,6 +1137,7 @@ public class ACCgpuDecompileWriter extends PrintWriter {
   private void printDeclList(Xobject v, Xobject id_list) {
     Ident id;
     if (v == null) {
+      printDeclList(Xcons.List(), id_list);
       return;
     }
 
@@ -1137,6 +1147,10 @@ public class ACCgpuDecompileWriter extends PrintWriter {
           for(XobjArgs a = v.getArgs(); a != null; a = a.nextArgs()) {
             printDeclList(a.getArg(),id_list);
           }
+          XobjList addDeclList = getDeclForNotDeclared((XobjList)id_list);
+          for(XobjArgs a = addDeclList.getArgs(); a != null; a = a.nextArgs()) {
+            printDeclList(a.getArg(),id_list);
+          }
         } break;
       case VAR_DECL:
         {
@@ -1167,4 +1181,26 @@ public class ACCgpuDecompileWriter extends PrintWriter {
         break;
     }
   }
+
+  /* copied from XmcXobjectToXcodeTranslator.java */
+  private XobjList getDeclForNotDeclared(XobjList identList) {
+    if (identList == null) {
+      return null;
+    }
+
+    XobjList declList = Xcons.List();
+    for (Xobject a : identList) {
+      Ident id = (Ident)a;
+      if (id.isDeclared() || !id.getStorageClass().isVarOrFunc()) {
+        continue;
+      }
+      Xtype t = id.Type();
+      Xcode declCode = t.isFunction() ? Xcode.FUNCTION_DECL
+              : Xcode.VAR_DECL;
+      declList.add(Xcons.List(declCode, Xcons.Symbol(Xcode.IDENT,
+                      id.getName()),
+              null));
+    }
+    return declList;
+  }
 }
