@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.openmp;
@@ -39,6 +22,9 @@ public enum OMPpragma
      */
     PARALLEL,           /* parallel <clause_list> */
     FOR,                /* (for|do) <clause_list> */
+    TASK,               /* task <clause list>*/
+    SIMD,               /* simd <clause list>*/
+    DECLARE,            /* declare <clause_list> */
     SECTIONS,           /* sections <clause_list> */
     SECTION,            /* section */
     SINGLE,             /* single <clause list> */
@@ -85,6 +71,14 @@ public enum OMPpragma
     DATA_REDUCTION_IOR(14),     // Fortran
     DATA_REDUCTION_IEOR(15),    // Fortran
 
+    DATA_DEPEND_IN,
+    DATA_DEPEND_OUT,
+    DATA_DEPEND_INOUT,
+
+    DATA_FINAL,
+    DIR_UNTIED,
+    DIR_MERGEABLE,
+
     DIR_ORDERED,
     DIR_IF,
     DIR_NOWAIT,
