@@ -1,30 +1,16 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.openmp;
 
+import java.util.List;
+
 import xcodeml.util.XmOption;
 import exc.object.*;
+import exc.xcodeml.XcodeMLtools;
 import exc.block.*;
 
 /**
@@ -111,19 +97,29 @@ public class OMPtranslate implements XobjectDefVisitor
         }
         
         FuncDefBlock fd = new FuncDefBlock(d);
+
+	List list = anaDecl.getCommonName();
+        for(int i=0;i<list.size();i++)
+              fd.searchCommonMember(list.get(i).toString(),anaDecl,d);
+
         if(XmOption.isLanguageC())
             fd.removeDeclInit();
 
         if(OMP.hasError())
             return;
         
+        OMP.debug("3");
+
         anaPragma.run(fd, anaDecl);
         if(OMP.hasError())
             return;
 
+        OMP.debug("4");
+
         rewriteExpr.run(fd, anaDecl);
         if(OMP.hasError())
             return;
+        OMP.debug("5");
 
         transPragma.run(fd);
         if(OMP.hasError())
