@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.util;
@@ -30,12 +13,8 @@ import java.util.List;
 import exc.object.XobjectFile;
 
 import exc.openacc.ACC;
-import exc.openacc.ACCanalyzePragma;
-import exc.openacc.ACCglobalDecl;
-import exc.openacc.ACCgpuDecompiler;
-import exc.openacc.ACCrewritePragma;
-import exc.openacc.ACCtranslatePragma;
-
+import exc.openacc.AccDevice;
+import exc.openacc.AccTranslator;
 import exc.openmp.OMP;
 import exc.openmp.OMPtranslate;
 
@@ -48,8 +27,8 @@ import exc.xcodeml.XcodeMLtools;
 import exc.xcodeml.XcodeMLtools_F;
 import exc.xcodeml.XcodeMLtools_Fmod;
 import exc.xcodeml.XcodeMLtools_C;
+import xcodeml.IXobject;
 import xcodeml.XmLanguage;
-import xcodeml.XmObj;
 import xcodeml.binding.XmXcodeProgram;
 import xcodeml.util.*;
 
@@ -88,8 +67,11 @@ public class omompx
       "  -xf          process XcodeML/Fortran document.",
       "  -l           suppress line directive in decompiled code.",
       "  -fopenmp     enable OpenMP translation.",
-      "  -fcoarry     enable coarray translation.",
-      "  -fnocoarry   pass without coarray translation (default).",
+      "  -fcoarry[=suboption]",
+      "               enable coarray translation optionally with a suboption.",
+      "  -fnocoarry   pass without coarray translation (default for C).",
+      "  -fcoarry-no-use-statement",
+      "               supress generation of the USE statement for coarray runtime libraries.",
       "  -fatomicio   enable transforming Fortran IO statements to atomic operations.",
       "  -w N         set max columns to N for Fortran source.",
       "  -gnu         decompile for GNU Fortran (default).",
@@ -104,10 +86,10 @@ public class omompx
       "  -dump        output Xcode file and decompiled file to standard output.",
       "  -domp        enable output OpenMP translation debug message.",
       " Profiling Options:",
-      "  -scalasca-all      : output results in scalasca format for all directives.",
-      "  -scalasca          : output results in scalasca format for selected directives.",
-      "  -tlog-all          : output results in tlog format for all directives.",
-      "  -tlog              : output results in tlog format for selected directives.",
+      "  -scalasca-all       : output results in scalasca format for all directives.",
+      "  -scalasca-selective : output results in scalasca format for selected directives.",
+      "  -tlog-all           : output results in tlog format for all directives.",
+      "  -tlog-selective     : output results in tlog format for selected directives.",
       "",
       "  -enable-threads  enable 'threads' clause",
       "  -enable-gpu      enable xmp-dev directive/clauses"
@@ -140,7 +122,15 @@ public class omompx
     boolean doScalasca = false;
     boolean doTlog = false;
     int maxColumns = 0;
+    String coarray_suboption = "";            // HIDDEN
+    boolean coarray_noUseStmt = false;     // TEMPORARY
+    int accDefaultVectorLength = 0;
+    boolean accDisableReadOnlyDataCache = false;
         
+    // environment variable analysis
+    Boolean xmpf_onlyCafMode = "1".equals(System.getenv("XMP_ONLYCAF"));
+    Boolean xmpf_skipCafMode = "1".equals(System.getenv("XMP_SKIPCAF"));
+
     for(int i = 0; i < args.length; ++i) {
       String arg = args[i];
       String narg = (i < args.length - 1) ? args[i + 1] : null;
@@ -159,6 +149,10 @@ public class omompx
         coarray = true;
       } else if(arg.equals("-fnocoarray")) {
         coarray = false;
+      } else if(arg.startsWith("-fcoarray=")) {                  // HIDDEN
+        coarray_suboption += arg.substring(arg.indexOf("=")+1);
+      } else if(arg.equals("-fcoarray-no-use-statement")) {       // TEMPORARY
+        coarray_noUseStmt = true;
       } else if(arg.equals("-facc")) {
         openACC = true; 
       } else if(arg.equals("-fxmp")) {
@@ -202,13 +196,13 @@ public class omompx
         XmOption.setCompilerVendor(XmOption.COMP_VENDOR_GNU);
       } else if(arg.equals("-intel")) {
         XmOption.setCompilerVendor(XmOption.COMP_VENDOR_INTEL);
-      } else if (arg.equals("-scalasca")) {
+      } else if (arg.equals("-scalasca-selective")) {
         selective_profile = true;
         doScalasca = true;
       } else if (arg.equals("-scalasca-all")) {
         all_profile = true;
         doScalasca = true;
-      } else if (arg.equals("-tlog")) {
+      } else if (arg.equals("-tlog-selective")) {
         selective_profile = true;
         doTlog = true;
       } else if (arg.equals("-tlog-all")) {
@@ -227,8 +221,17 @@ public class omompx
       } else if (arg.startsWith("-max_assumed_shape=")) {
 	  String n = arg.substring(19);
 	  exc.xmpF.XMP.MAX_ASSUMED_SHAPE = Integer.parseInt(n);
-      } else if (arg.equals("-no-ldg")){
-        exc.openacc.ACC.useReadOnlyDataCache = false;
+      } else if (arg.equals("-no-ldg")) {
+        accDisableReadOnlyDataCache = true;
+      } else if (arg.startsWith("-default-veclen=")) {
+        String n = arg.substring("-default-veclen=".length());
+        accDefaultVectorLength = Integer.parseInt(n);
+      } else if (arg.startsWith("-platform=")){
+        String n = arg.substring("-platform=".length());
+        ACC.platform = ACC.Platform.valueOf(n);
+      } else if (arg.startsWith("-device=")){
+        String n = arg.substring("-device=".length());
+        ACC.device = AccDevice.valueOf(n);
       } else if(arg.startsWith("-")){
         error("unknown option " + arg);
       } else if(inXmlFile == null) {
@@ -280,6 +283,7 @@ public class omompx
     XmOption.setIsXcalableMPthreads(xcalableMPthreads);
     XmOption.setIsXcalableMPGPU(xcalableMPGPU);
     XmOption.setTlogMPIisEnable(doTlog);
+    XmOption.setCoarrayNoUseStatement(coarray_noUseStmt);   // TEMPORARY
 
     XobjectFile xobjFile;
     String srcPath = inXmlFile;
@@ -353,6 +357,11 @@ public class omompx
           xobjFile.addHeaderLine("# include \"xmp_tlog.h\"");
         }
       }
+      if(openACC){
+        if(xobjFile.findIdent("acc_init", IXobject.FINDKIND_ANY) == null){
+          xobjFile.addHeaderLine("# include \"openacc.h\"");
+        }
+      }
       xmpTranslator.finalize();
 
       if(xcodeWriter != null) {
@@ -361,45 +370,50 @@ public class omompx
       }
     }
 
-    if(xmpf) {  // XcalableMP xmpF translation
+    if (xmpf && xmpf_skipCafMode) {
+      System.out.println("<SKIP-CAF MODE> XMP/F Coarray translator is " +
+                         "bypassed for " + xobjFile.getSourceFileName() + ".");
+    }
 
-      // Error check and light analysis
-      exc.xmpF.XMPtransCoarray
-        caf_translator0 = new exc.xmpF.XMPtransCoarray(xobjFile, 0);
-      xobjFile.iterateDef(caf_translator0);
+    if (xmpf && !xmpf_skipCafMode) {
+
+      // Coarray Fortran pass#1
+      exc.xmpF.XMPtransCoarray caf_translator1 =
+        new exc.xmpF.XMPtransCoarray(xobjFile, 1, coarray_suboption,
+                                     xmpf_onlyCafMode);
+      xobjFile.iterateDef(caf_translator1);
       if(exc.xmpF.XMP.hasErrors())
         System.exit(1);
+      caf_translator1.finish();
 
-      if (caf_translator0.containsCoarray()) {
+      // Coarray Fortran pass#2
+      exc.xmpF.XMPtransCoarray caf_translator2 =
+        new exc.xmpF.XMPtransCoarray(xobjFile, 2, coarray_suboption,
+                                     xmpf_onlyCafMode);
+      xobjFile.iterateDef(caf_translator2);
+      if(exc.xmpF.XMP.hasErrors())
+        System.exit(1);
+      caf_translator2.finish();
 
-        //XMP.warning("translating coarray features");
-
-        // Coarray Fortran pass#1
-        exc.xmpF.XMPtransCoarray
-          caf_translator1 = new exc.xmpF.XMPtransCoarray(xobjFile, 1);
-        xobjFile.iterateDef(caf_translator1);
-        if(exc.xmpF.XMP.hasErrors())
-          System.exit(1);
-        caf_translator1.finish();
-
-        // Coarray Fortran pass#2
-        exc.xmpF.XMPtransCoarray
-          caf_translator2 = new exc.xmpF.XMPtransCoarray(xobjFile, 2);
-        xobjFile.iterateDef(caf_translator2);
-        if(exc.xmpF.XMP.hasErrors())
-          System.exit(1);
-        caf_translator2.finish();
-
-      } else {    // without coarray features
-
-        // XMP Fortran
-        exc.xmpF.XMPtranslate
-          xmp_translator = new exc.xmpF.XMPtranslate(xobjFile);
-        xobjFile.iterateDef(xmp_translator);
-        if(exc.xmpF.XMP.hasErrors())
-          System.exit(1);
-        xmp_translator.finish();
+      if(xcodeWriter != null) {
+        xobjFile.Output(xcodeWriter);
+        xcodeWriter.flush();
       }
+    }
+
+    if (xmpf && xmpf_onlyCafMode) {
+      System.out.println("<ONLY-CAF MODE> XMP/F gloval-view translator is " +
+                         "bypassed for " + xobjFile.getSourceFileName() + ".");
+    }
+
+    if (xmpf && !xmpf_onlyCafMode) {
+      // XMP Fortran
+      exc.xmpF.XMPtranslate
+        xmp_translator = new exc.xmpF.XMPtranslate(xobjFile);
+      xobjFile.iterateDef(xmp_translator);
+      if(exc.xmpF.XMP.hasErrors())
+        System.exit(1);
+      xmp_translator.finish();
 
       if(xcodeWriter != null) {
         xobjFile.Output(xcodeWriter);
@@ -408,6 +422,7 @@ public class omompx
     }
 
     // OpenMP translation
+//    OMP.debugFlag = true;
     if(openMP) {
       OMPtranslate omp_translator = new OMPtranslate(xobjFile);
       xobjFile.iterateDef(omp_translator);
@@ -424,33 +439,28 @@ public class omompx
     }
     
     if(openACC){
-      ACCglobalDecl accGlobalDecl = new ACCglobalDecl(xobjFile);
-      ACCanalyzePragma accAnalyzer = new ACCanalyzePragma(accGlobalDecl);
-      xobjFile.iterateDef(accAnalyzer);
-      accAnalyzer.finalize();
-      ACC.exitByError();
-      
-      ACCtranslatePragma accTranslator = new ACCtranslatePragma(accGlobalDecl);
-      xobjFile.iterateDef(accTranslator);
-      accTranslator.finalize();
-      ACC.exitByError();
-      
-      ACCrewritePragma accRewriter = new ACCrewritePragma(accGlobalDecl);
-      xobjFile.iterateDef(accRewriter);
-      accRewriter.finalize();
-      ACC.exitByError();
-      
-      ACCgpuDecompiler gpuDecompiler = new ACCgpuDecompiler();
-      gpuDecompiler.decompile(accGlobalDecl);
+      if(ACC.device == AccDevice.NONE){
+        switch(ACC.platform){
+          case CUDA:
+          case OpenCL:
+            ACC.device = AccDevice.Fermi;
+            break;
+        }
+      }
 
-      accGlobalDecl.setupGlobalConstructor();
-      accGlobalDecl.setupGlobalDestructor();
-      accGlobalDecl.setupMain();
-      ACC.exitByError();
-      
-      xobjFile.addHeaderLine("# include \"acc.h\"");
-      xobjFile.addHeaderLine("# include \"acc_gpu.h\"");
-      accGlobalDecl.finalize();
+      if(accDefaultVectorLength > 0) {
+        ACC.device.setDefaultVectorLength(accDefaultVectorLength);
+      }
+
+      if(accDisableReadOnlyDataCache == true){
+        ACC.device.setUseReadOnlyDataCache(false);
+      }
+
+      //XmOption.setDebugOutput(true);
+      AccTranslator accTranslator = new AccTranslator(xobjFile, false);
+      xobjFile.iterateDef(accTranslator);
+
+      accTranslator.finish();
       
       if(xcodeWriter != null) {
         xobjFile.Output(xcodeWriter);
@@ -536,7 +546,7 @@ public class omompx
       }
       decompiler.decompile(context, xcodeDoc, decompWriter);
       // for collect-init
-      decompWriter.write(xobjFile.getTailText());
+      //decompWriter.write(xobjFile.getTailText());
       decompWriter.flush();
     
       if(!dump && outputDecomp) {
