@@ -1,24 +1,6 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -37,6 +19,7 @@ public class XMP {
   public final static String DESC_PREFIX_		= "_XMP_DESC_";
   public final static String NODES_SIZE_PREFIX_		= "_XMP_NODES_SIZE_";
   public final static String NODES_RANK_PREFIX_		= "_XMP_NODES_RANK_";
+  public final static String STATIC_DESC_PREFIX_        = "_XMP_STATIC_";
   public final static String GPU_HOST_DESC_PREFIX_	= "_XMP_GPU_HOST_DESC_";
   public final static String GPU_DEVICE_DESC_PREFIX_	= "_XMP_GPU_DEVICE_DESC_";
   public final static String GPU_DEVICE_ADDR_PREFIX_	= "_XMP_GPU_DEVICE_ADDR_";
