@@ -1,24 +1,6 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -319,6 +301,28 @@ public class XMPlocalDecl {
     bodyList.add(Xcons.List(Xcode.EXPR_STATEMENT, funcId.Call(funcArgs)));
   }
 
+  public static void addConstructorCall2_staticDesc(String funcName, Xobject funcArgs, XMPglobalDecl globalDecl, Block block,
+						    Ident flagVar, boolean setFlag){
+    XobjList bodyList = (XobjList)block.getProp(CONSTRUCTOR);
+    if (bodyList == null){
+      bodyList = Xcons.List(Xcode.LIST);
+      block.setProp(CONSTRUCTOR, (Object)bodyList);
+    }
+
+    Ident funcId = globalDecl.declExternFunc(funcName);
+    //bodyList.add(Xcons.List(Xcode.EXPR_STATEMENT, funcId.Call(funcArgs)));
+
+    XobjList ifBody = Xcons.List();
+    ifBody.add(Xcons.List(Xcode.EXPR_STATEMENT, funcId.Call(funcArgs)));
+    if (setFlag) ifBody.add(Xcons.List(Xcode.EXPR_STATEMENT, Xcons.Set(flagVar.Ref(), Xcons.IntConstant(1))));
+    Xobject body = Xcons.List(Xcode.IF_STATEMENT,
+			      Xcons.unaryOp(Xcode.LOG_NOT_EXPR, flagVar.Ref()),
+			      ifBody,
+			      null);
+    bodyList.add(body);
+
+  }
+
   public static void addAllocCall2(String funcName, Xobject funcArgs, XMPglobalDecl globalDecl, Block block) {
     XobjList bodyList = (XobjList)block.getProp(ALLOC);
     if (bodyList == null){
