@@ -1,33 +1,8 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
-
 package exc.xcalablemp;
 
 import exc.block.*;
 import exc.object.*;
 import exc.openacc.ACCpragma;
-
 import java.util.*;
 
 public class XMPrewriteExpr {
@@ -66,6 +41,9 @@ public class XMPrewriteExpr {
     XMPlocalDecl.setupConstructor(fb);
     XMPlocalDecl.setupDestructor(fb);
 
+    // add a barrier at the end of the original main
+    if (fb.getName() == "main") addBarrier(fb);
+
     def.Finalize();
   }
 
@@ -160,23 +138,187 @@ public class XMPrewriteExpr {
     else if ((leftExpr.Opcode() == Xcode.CO_ARRAY_REF) || (rightExpr.Opcode() == Xcode.CO_ARRAY_REF)) {
       return rewriteCoarrayAssignExpr(myExpr, exprParentBlock, localXMPsymbolTable, iter);
     }
+    // else if (leftExpr.Opcode() == Xcode.SUB_ARRAY_REF){
+    //   return rewriteSubArrayAssignExpr(myExpr, exprParentBlock);
+    // }
     else {
       return rewriteExpr(myExpr, exprParentBlock);
     }
   }
 
-  //void _XMP_shortcut_put_image1(int target_image, _XMP_coarray_t *dst_desc,
-  //      _XMP_coarray_t *src_desc, int dst_point, int src_point, int transfer_length)
-  //void _XMP_shortcut_put_image2(int target_image1, int target_image2, _XMP_coarray_t *dst_desc, ...)
+
+  // private Xobject rewriteSubArrayAssignExpr(Xobject assignStmt, Block b) throws XMPexception {
+
+  //   Xobject left = assignStmt.left();
+
+  //   assert left.Opcode() == Xcode.SUB_ARRAY_REF;
+
+  //   List<Ident> varList = new ArrayList<Ident>(XMP.MAX_DIM);
+  //   List<Ident> varListTemplate = new ArrayList<Ident>(XMP.MAX_DIM);
+  //   for (int i = 0; i < XMP.MAX_DIM; i++) varListTemplate.add(null);
+  //   List<Xobject> lbList = new ArrayList<Xobject>(XMP.MAX_DIM);
+  //   List<Xobject> lenList = new ArrayList<Xobject>(XMP.MAX_DIM);
+  //   List<Xobject> stList = new ArrayList<Xobject>(XMP.MAX_DIM);
+
+  //   //
+  //   // convert LHS
+  //   //
+
+  //   String arrayName = left.getArg(0).getSym();
+
+  //   Xtype arrayType = null;
+  //   Ident arrayId = b.findVarIdent(arrayName);
+  //   if (arrayId != null){
+  //     arrayType = arrayId.Type();
+  //   }
+	
+  //   if (arrayType == null) throw new XMPexception("array should be declared statically");
+
+  //   Xtype elemType = arrayType.getArrayElementType();
+  //   int n = arrayType.getNumDimensions();
+
+  //   XobjList subscripts = (XobjList)left.getArg(1);
+
+  //   for (int i = 0; i < n; i++, arrayType = arrayType.getRef()){
+
+  //     long dimSize = arrayType.getArraySize();
+  //     Xobject sizeExpr;
+  //     if (dimSize == 0 || arrayType.getKind() == Xtype.POINTER){
+  // 	throw new XMPexception("array should be declared statically");
+  //     }
+  //     else if (dimSize == -1){
+  //       sizeExpr = arrayType.getArraySizeExpr();
+  //     }
+  //     else {
+  // 	sizeExpr = Xcons.LongLongConstant(0, dimSize);
+  //     }
+
+  //     Xobject sub = subscripts.getArg(i);
+
+  //     Ident var;
+  //     Xobject lb, len, st;
+
+  //     if (sub.Opcode() != Xcode.LIST) continue;
+
+  //     var = XMPtranslateLocalPragma.declIdentWithBlock(b, "_XMP_loop_i" + Integer.toString(i), Xtype.intType);
+  //     varList.add(var);
+
+  //     lb = ((XobjList)sub).getArg(0);
+  //     if (lb == null) lb = Xcons.IntConstant(0);
+  //     len = ((XobjList)sub).getArg(1);
+  //     if (len == null) len = sizeExpr;
+  //     st = ((XobjList)sub).getArg(2);
+  //     if (st == null) st = Xcons.IntConstant(1);
+
+  //     lbList.add(lb);
+  //     lenList.add(len);
+  //     stList.add(st);
+
+  //     Xobject expr;
+  //     expr = Xcons.binaryOp(Xcode.MUL_EXPR, var.Ref(), st);
+  //     expr = Xcons.binaryOp(Xcode.PLUS_EXPR, expr, lb);
+
+  //     subscripts.setArg(i, expr);
+
+  //   }
+
+  //   Xobject new_left = Xcons.arrayRef(elemType, left.getArg(0), subscripts);
+
+  //   //
+  //   // convert RHS
+  //   //
+
+  //   // NOTE: Since the top level object cannot be replaced, the following conversion is applied to
+  //   //       the whole assignment.
+  //   XobjectIterator j = new topdownXobjectIterator(assignStmt);
+  //   for (j.init(); !j.end(); j.next()) {
+  //     Xobject x = j.getXobject();
+
+  //     if (x.Opcode() != Xcode.SUB_ARRAY_REF) continue;
+
+  //     int k = 0;
+
+  //     String arrayName1 = x.getArg(0).getSym();
+
+  //     XMPalignedArray array1 = _globalDecl.getXMPalignedArray(arrayName1, b);
+  //     Xtype arrayType1 = null;
+  //     if (array1 != null){
+  // 	arrayType1 = array1.getArrayType();
+  //     }
+  //     else {
+  // 	Ident arrayId1 = b.findVarIdent(arrayName1);
+  // 	if (arrayId1 != null){
+  // 	  arrayType1 = arrayId1.Type();
+  // 	}
+  //     }
+	
+  //     if (arrayType1 == null) throw new XMPexception("array should be declared statically");
+
+  //     Xtype elemType1 = arrayType1.getArrayElementType();
+  //     int m = arrayType1.getNumDimensions();
+
+  //     XobjList subscripts1 = (XobjList)x.getArg(1);
+
+  //     for (int i = 0; i < m; i++, arrayType1 = arrayType1.getRef()){
+
+  // 	Xobject sub = subscripts1.getArg(i);
+
+  // 	Ident var;
+  // 	Xobject lb, st;
+
+  // 	if (sub.Opcode() != Xcode.LIST) continue;
+
+  // 	lb = ((XobjList)sub).getArg(0);
+  // 	if (lb == null) lb = Xcons.IntConstant(0);
+  // 	st = ((XobjList)sub).getArg(2);
+  // 	if (st == null) st = Xcons.IntConstant(1);
+
+  // 	Xobject expr;
+  // 	expr = Xcons.binaryOp(Xcode.MUL_EXPR, varList.get(k).Ref(), st);
+  // 	expr = Xcons.binaryOp(Xcode.PLUS_EXPR, expr, lb);
+
+  // 	subscripts1.setArg(i, expr);
+  // 	k++;
+  //     }
+
+  //     Xobject new_x = Xcons.arrayRef(elemType1, x.getArg(0), subscripts1);
+  //     j.setXobject(new_x);
+
+  //   }
+
+  //   //
+  //   // construct loop
+  //   //
+
+  //   BlockList loop = null;
+
+  //   BlockList body = Bcons.emptyBody();
+  //   body.add(Xcons.Set(new_left, assignStmt.right()));
+
+  //   for (int i = varList.size() - 1; i >= 0; i--){
+  //     loop = Bcons.emptyBody();
+  //     loop.add(Bcons.FORall(varList.get(i).Ref(), Xcons.IntConstant(0), lenList.get(i), Xcons.IntConstant(1),
+  // 			    Xcode.LOG_LT_EXPR, body));
+  //     body = loop;
+  //   }
+
+  //   return Bcons.COMPOUND(loop).toXobject();
+
+  // }
+
+
   private Xobject createShortcutCoarray(int imageDims, XobjList imageList, String commkind, 
                                         XMPcoarray dstCoarray, XMPcoarray srcCoarray,
-                                        Xobject dstCoarrayExpr, Xobject srcCoarrayExpr) throws XMPexception
+                                        Xobject dstCoarrayExpr, Xobject srcCoarrayExpr,
+                                        boolean isDstCoarrayOnAcc, boolean isSrcCoarrayOnAcc) throws XMPexception
   // dstCoarray is left expression. srcCoarray is right expression.
   {
     // Set Function Name
     // If image set is 2 dimension and Put operation,
     // function name is "_XMP_shortcut_put_image2"
+    boolean isAcc = isDstCoarrayOnAcc || isSrcCoarrayOnAcc;
     String funcName = "_XMP_coarray_shortcut_" + commkind;
+    if(isAcc) funcName += "_acc";
     Ident funcId = _globalDecl.declExternFunc(funcName);
     XobjList funcArgs = Xcons.List();
 
@@ -374,31 +516,61 @@ public class XMPrewriteExpr {
     position = Xcons.binaryOp(Xcode.MUL_EXPR, position, Xcons.SizeOf(elmtType));
     funcArgs.add(position);
 
-    // length
-    Xobject length = null;
+    // dst_length
+    Xobject dst_length = null;
     if(dstCoarrayExpr.Opcode() == Xcode.SUB_ARRAY_REF){
       if(dstCoarrayDepthContinuous == 0){
-        length = Xcons.IntConstant((int)dstCoarray.getSizeAt(0) * dstCoarrayDistance[0]);
+        dst_length = Xcons.IntConstant((int)dstCoarray.getSizeAt(0) * dstCoarrayDistance[0]);
       }
       else{
         Xobject tripletList = dstCoarrayExpr.getArg(1).getArg(dstCoarrayDepthContinuous-1);
         if(tripletList.isConstant() || tripletList.isVariable()){
-          length = Xcons.IntConstant(dstCoarrayDistance[dstCoarrayDepthContinuous-1]);
+          dst_length = Xcons.IntConstant(dstCoarrayDistance[dstCoarrayDepthContinuous-1]);
         }
         else{
-          length = Xcons.binaryOp(Xcode.MUL_EXPR, ((XobjList)tripletList).getArg(1),
+          dst_length = Xcons.binaryOp(Xcode.MUL_EXPR, ((XobjList)tripletList).getArg(1),
                                   Xcons.IntConstant(dstCoarrayDistance[dstCoarrayDepthContinuous-1]));
         }
       }
     }
     else if(dstCoarrayExpr.Opcode() == Xcode.ARRAY_REF || dstCoarrayExpr.Opcode() == Xcode.VAR){
-      length = Xcons.IntConstant(1);
+      dst_length = Xcons.IntConstant(1);
     }
     else{
       throw new XMPexception("Not supported this coarray Syntax");
     }
-    length = Xcons.binaryOp(Xcode.MUL_EXPR, length, Xcons.SizeOf(elmtType));
-    funcArgs.add(length);
+    funcArgs.add(dst_length);
+
+    // src_length
+    Xobject src_length = null;
+    if(srcCoarrayExpr.Opcode() == Xcode.SUB_ARRAY_REF){
+      if(srcCoarrayDepthContinuous == 0){
+        src_length = Xcons.IntConstant((int)srcCoarray.getSizeAt(0) * srcCoarrayDistance[0]);
+      }
+      else{
+        Xobject tripletList = srcCoarrayExpr.getArg(1).getArg(srcCoarrayDepthContinuous-1);
+        if(tripletList.isConstant() || tripletList.isVariable()){
+          src_length = Xcons.IntConstant(srcCoarrayDistance[srcCoarrayDepthContinuous-1]);
+        }
+        else{
+          src_length = Xcons.binaryOp(Xcode.MUL_EXPR, ((XobjList)tripletList).getArg(1),
+                                      Xcons.IntConstant(srcCoarrayDistance[srcCoarrayDepthContinuous-1]));
+        }
+      }
+    }
+    else if(srcCoarrayExpr.Opcode() == Xcode.ARRAY_REF || srcCoarrayExpr.Opcode() == Xcode.VAR){
+      src_length = Xcons.IntConstant(1);
+    }
+    else{
+      throw new XMPexception("Not supported this coarray Syntax");
+    }
+    funcArgs.add(src_length);
+
+    if(isAcc){
+      //add location of coarray (host=0, acc=1)
+      funcArgs.add(Xcons.IntConstant(isDstCoarrayOnAcc? 1 : 0));
+      funcArgs.add(Xcons.IntConstant(isSrcCoarrayOnAcc? 1 : 0));
+    }
 
     // Create function
     Xobject newExpr = funcId.Call(funcArgs);
@@ -427,20 +599,10 @@ public class XMPrewriteExpr {
     if(coarray == null){
       throw new XMPexception("cannot find coarray '" + coarrayName + "'");
     }
-    
+
     // Get Coarray Dims
     XobjList funcArgs = Xcons.List();
-    int coarrayDims;
-    if(coarrayExpr.getArg(0).Opcode() == Xcode.SUB_ARRAY_REF || coarrayExpr.getArg(0).Opcode() == Xcode.ARRAY_REF){
-      XobjList tripletList = (XobjList)(coarrayExpr.getArg(0)).getArg(1);
-      coarrayDims = tripletList.Nargs();
-    }
-    else if(coarrayExpr.getArg(0).Opcode() == Xcode.VAR){
-      coarrayDims = 1;
-    }
-    else{
-      throw new XMPexception("Not supported this coarray Syntax");
-    }
+    int coarrayDims  = coarray.getVarDim();
 
     // Get Local Dims
     boolean isArray;
@@ -452,8 +614,12 @@ public class XMPrewriteExpr {
       Ident varId = localExpr.findVarIdent(localName);
       localDims = varId.Type().getNumDimensions();
     }
-    else if(localExpr.Opcode() == Xcode.VAR){
-      localName = localExpr.getName();
+    else if(localExpr.Opcode() == Xcode.VAR || localExpr.Opcode() == Xcode.POINTER_REF){
+      if(localExpr.Opcode() == Xcode.VAR)
+        localName = localExpr.getName();
+      else
+        localName = localExpr.getArg(0).getName();
+      
       isArray = false;
       localDims = 1;
     }
@@ -469,7 +635,7 @@ public class XMPrewriteExpr {
 
     // Get image Dims
     XobjList imageList = (XobjList)coarrayExpr.getArg(1);
-    int imageDims = imageList.Nargs();
+    int imageDims = coarray.getImageDim();
 
     // Shortcut Function
     if(isContinuousArray(coarrayExpr, exprParentBlock) &&
@@ -478,28 +644,19 @@ public class XMPrewriteExpr {
       {
         XMPcoarray remoteCoarray = coarray;
         XMPcoarray localCoarray = _globalDecl.getXMPcoarray(localName, exprParentBlock);
+        boolean isRemoteCoarrayUseDevice = isUseDevice(remoteCoarray.getName(), exprParentBlock);
+        boolean isLocalCoarrayUseDevice = isUseDevice(localCoarray.getName(), exprParentBlock);
         if(leftExpr.Opcode() == Xcode.CO_ARRAY_REF)
           {  // put a[:]:[1] = b[:];
             return createShortcutCoarray(imageDims, imageList, "put", remoteCoarray, localCoarray,
-                                         coarrayExpr.getArg(0), localExpr);
+                                         coarrayExpr.getArg(0), localExpr, isRemoteCoarrayUseDevice, isLocalCoarrayUseDevice);
           }
         else{ // get a[:] = b[:]:[1]
           return createShortcutCoarray(imageDims, imageList, "get", localCoarray, remoteCoarray,
-                                       localExpr, coarrayExpr.getArg(0));
+                                       localExpr, coarrayExpr.getArg(0), isRemoteCoarrayUseDevice, isLocalCoarrayUseDevice);
         }
       }
 
-    // Set function _XMP_coarray_rdma_set()
-    /*
-    Ident funcId = _globalDecl.declExternFunc("_XMP_coarray_rdma_set");
-    funcArgs.add(Xcons.IntConstant(coarrayDims));
-    funcArgs.add(Xcons.IntConstant(localDims));
-    funcArgs.add(Xcons.IntConstant(imageDims));
-    Xobject newExpr = funcId.Call(funcArgs);
-    newExpr.setIsRewrittedByXmp(true);
-    b.add(newExpr);
-    */
-
     // Set function _XMP_coarray_rdma_coarray_set_X()
     Ident funcId;
     funcArgs = Xcons.List();
@@ -612,7 +769,6 @@ public class XMPrewriteExpr {
     iter.insertStatement(newExpr);
 
     // Set function _XMP_coarray_rdma_do()
-    funcId = _globalDecl.declExternFunc("_XMP_coarray_rdma_do");
     funcArgs = Xcons.List();
     if(leftExpr.Opcode() == Xcode.CO_ARRAY_REF){
       funcArgs.add(Xcons.IntConstant(XMPcoarray.PUT));
@@ -621,13 +777,18 @@ public class XMPrewriteExpr {
       funcArgs.add(Xcons.IntConstant(XMPcoarray.GET));
     }
 
+    boolean isLocalOnDevice = false;
+    boolean isRemoteOnDevice = false;
+    
     // Get Coarray Descriptor
     funcArgs.add(Xcons.SymbolRef(coarray.getDescId()));
+    isRemoteOnDevice = isUseDevice(coarray.getName(), exprParentBlock);
 
     // Get Local Pointer Name
     if(localExpr.Opcode() == Xcode.SUB_ARRAY_REF || localExpr.Opcode() == Xcode.ARRAY_REF){
       Xobject varAddr = localExpr.getArg(0);
       String varName = varAddr.getName();
+      isLocalOnDevice = isUseDevice(varName, exprParentBlock);
       XMPcoarray localArray = _globalDecl.getXMPcoarray(varName, exprParentBlock);
       if(localArray == null){
 	  funcArgs.add(varAddr);
@@ -639,8 +800,14 @@ public class XMPrewriteExpr {
 	  funcArgs.add(Xcons.SymbolRef(localArray.getDescId()));
       }
     }
-    else if(localExpr.Opcode() == Xcode.VAR){
-      String varName = localExpr.getName();
+    else if(localExpr.Opcode() == Xcode.VAR || localExpr.Opcode() == Xcode.POINTER_REF){
+      String varName;
+      if(localExpr.Opcode() == Xcode.VAR)
+        varName = localExpr.getName();
+      else
+        varName = localExpr.getArg(0).getName();
+      
+      isLocalOnDevice = isUseDevice(varName, exprParentBlock);
       XMPcoarray localVar = _globalDecl.getXMPcoarray(varName, exprParentBlock);
       if(localVar == null){
 	Xobject varAddr = Xcons.AddrOf(localExpr);
@@ -660,6 +827,14 @@ public class XMPrewriteExpr {
       throw new XMPexception("Not supported this coarray Syntax");
     }
 
+    boolean isAcc = isRemoteOnDevice || isLocalOnDevice;
+    if(isAcc){
+      funcId = _globalDecl.declExternFunc("_XMP_coarray_rdma_do_acc");
+      funcArgs.add(Xcons.IntConstant(isRemoteOnDevice? 1 : 0));
+      funcArgs.add(Xcons.IntConstant(isLocalOnDevice? 1 : 0));
+    }else{
+      funcId = _globalDecl.declExternFunc("_XMP_coarray_rdma_do");
+    }
     newExpr = funcId.Call(funcArgs);
     newExpr.setIsRewrittedByXmp(true);
     iter.insertStatement(newExpr);
@@ -771,7 +946,7 @@ public class XMPrewriteExpr {
     if(myExpr.Opcode() == Xcode.CO_ARRAY_REF)
       myExpr = myExpr.getArg(0);
 
-    if(myExpr.Opcode() == Xcode.VAR || myExpr.Opcode() == Xcode.ARRAY_REF){
+    if(myExpr.Opcode() == Xcode.VAR || myExpr.Opcode() == Xcode.ARRAY_REF || myExpr.Opcode() == Xcode.POINTER_REF){
       return true;
     }
     else if(myExpr.Opcode() == Xcode.SUB_ARRAY_REF){
@@ -944,10 +1119,14 @@ public class XMPrewriteExpr {
   }
 
   private boolean isCoarray(Xobject myExpr, Block block){
-    if(myExpr.Opcode() == Xcode.ARRAY_REF || myExpr.Opcode() == Xcode.SUB_ARRAY_REF){
+    if(myExpr.Opcode() == Xcode.ARRAY_REF || myExpr.Opcode() == Xcode.SUB_ARRAY_REF ||
+       myExpr.Opcode() == Xcode.ADDR_OF){
       myExpr = myExpr.getArg(0);
     }
-    
+
+    if(myExpr.Opcode() == Xcode.POINTER_REF || myExpr.Opcode() == Xcode.ARRAY_REF)
+      myExpr = myExpr.getArg(0);
+
     XMPcoarray coarray = _globalDecl.getXMPcoarray(myExpr.getSym(), block);
     
     if(coarray == null)
@@ -1170,7 +1349,6 @@ public class XMPrewriteExpr {
 	    iter.setXobject(rewriteArrayRef(myExpr, block));
 	    break;
 	  case SUB_ARRAY_REF:
-	    //System.out.println("sub_array_ref="+myExpr.toString());
 	    break;
 	  case XMP_DESC_OF:
 	    iter.setXobject(rewriteXmpDescOf(myExpr, block));
@@ -1193,6 +1371,12 @@ public class XMPrewriteExpr {
 	  	p.setType(Xtype.Pointer(t));
 	      }
 	    }
+            else if (f.Opcode() == Xcode.FUNC_ADDR && f.getString().equals("xmp_atomic_define")){
+              iter.setXobject(rewriteXmpAtomicDefine(myExpr, block));
+            }
+            else if (f.Opcode() == Xcode.FUNC_ADDR && f.getString().equals("xmp_atomic_ref")){
+              iter.setXobject(rewriteXmpAtomicRef(myExpr, block));
+            }
 	  default:
 	  }
 	}
@@ -1201,6 +1385,163 @@ public class XMPrewriteExpr {
     }
   }
 
+  private boolean isOneElement(Xobject myExpr) throws XMPexception {
+    if(myExpr.Opcode() == Xcode.CO_ARRAY_REF)
+      myExpr = myExpr.getArg(0);
+
+    Xcode op = myExpr.Opcode();
+    if(op == Xcode.VAR || op == Xcode.ARRAY_REF || op == Xcode.ADDR_OF || op == Xcode.INT_CONSTANT)
+      return true;
+    else if(op == Xcode.SUB_ARRAY_REF)
+      return false;
+    else{
+      throw new XMPexception("Unexpected value.");
+    }
+  }
+
+  private Xobject rewriteXmpAtomicRef(Xobject myExpr, Block block) throws XMPexception {
+    Xobject localExpr   = myExpr.getArg(1).getArg(0);
+    Xobject coarrayExpr = myExpr.getArg(1).getArg(1);
+
+    String coarrayName;
+    if(coarrayExpr.Opcode() == Xcode.CO_ARRAY_REF){  // xmp_atomic_ref(&value, atom[1]:[2]);
+      coarrayName = XMPutil.getXobjSymbolName(coarrayExpr.getArg(0));
+    }
+    else if(coarrayExpr.Opcode() == Xcode.VAR){      // xmp_atomic_ref(&value, atom);
+      coarrayName = coarrayExpr.getName();
+    }
+    else if(Xcode.ARRAY_REF == Xcode.ARRAY_REF){     // xmp_atomic_ref(&value, atom[1]);
+      coarrayName = coarrayExpr.getArg(0).getName();
+    }
+    else{
+      throw new XMPexception("UNKNOWN DATA TYPE of the 1st argument in xmp_atomic_ref().");
+    }
+    
+    XMPcoarray coarray = _globalDecl.getXMPcoarray(coarrayName, block);
+    XobjList funcArgs  = Xcons.List(coarray.getDescId());
+
+    if(coarray == null)
+      throw new XMPexception("cannot find coarray '" + coarrayName + "'");
+
+    // Two arguments in xmp_atomic_ref() must be int-type
+    if(! localExpr.Type().isPointer())
+      throw new XMPexception("The first argument of atomic_ref() must be an int-type pointer.");
+
+    if(coarray.getElmtType() != Xtype.intType)
+      throw new XMPexception("The second argument of atomic_ref() must be an int-type.");
+    
+    // i.e. Only 1 element can be used.
+    if(!isOneElement(coarrayExpr) || !isOneElement(localExpr))
+      throw new XMPexception("An argument of atomic_ref() must be a scalar coarray or coindexed object.");
+      
+    // Add offset
+    funcArgs.add(getCoarrayOffset(coarrayExpr, coarray));
+
+    // Get Coarray Dims
+    int imageDims = (coarrayExpr.Opcode() == Xcode.CO_ARRAY_REF)? coarray.getImageDim() : 0;
+    String funcName = "_XMP_atomic_ref_" + imageDims;
+    Ident funcId    = _globalDecl.declExternFunc(funcName);
+    for(int i=0;i<imageDims;i++)
+      funcArgs.add(coarrayExpr.getArg(1).getArg(i));
+
+    // When localExpr is coarray, descriptor of the coarray is added to argument
+    if(isCoarray(localExpr, block)){
+      localExpr = localExpr.getArg(0);
+      if(localExpr.Opcode() == Xcode.VAR){
+        funcArgs.add(rewriteVarRef(localExpr, block, false));
+      }
+      else{  //  else if(localExpr.Opcode() == Xcode.ARRAY_REF){
+        funcArgs.add(Xcons.AddrOf(rewriteArrayRef(localExpr, block)));
+      }
+      String localName = (localExpr.Opcode() == Xcode.VAR)? localExpr.getName() : localExpr.getArg(0).getName();
+      funcArgs.add(_globalDecl.getXMPcoarray(localName, block).getDescId());
+      funcArgs.add(getCoarrayOffset(localExpr, _globalDecl.getXMPcoarray(localName, block)));
+    }
+    else{
+      funcArgs.add(localExpr);
+      funcArgs.add(Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0))); // NULL
+      funcArgs.add(Xcons.IntConstant(0));                                // offset (This value is not used)
+    }
+
+    // The variable must be an int-type
+    funcArgs.add(Xcons.SizeOf(Xtype.intType));
+    
+    return funcId.Call(funcArgs);
+  }
+  
+  private Xobject rewriteXmpAtomicDefine(Xobject myExpr, Block block) throws XMPexception {
+    Xobject coarrayExpr = myExpr.getArg(1).getArg(0);
+    Xobject localExpr   = myExpr.getArg(1).getArg(1);
+    
+    String coarrayName;
+    if(coarrayExpr.Opcode() == Xcode.CO_ARRAY_REF){  // xmp_atomic_define(atom[1]:[2]), value);
+      coarrayName = XMPutil.getXobjSymbolName(coarrayExpr.getArg(0));
+    }
+    else if(coarrayExpr.Opcode() == Xcode.VAR){      // xmp_atomic_define(atom, value);
+      coarrayName = coarrayExpr.getName();
+    }
+    else if(Xcode.ARRAY_REF == Xcode.ARRAY_REF){     // xmp_atomic_define(atom[1], value);
+      coarrayName = coarrayExpr.getArg(0).getName();
+    }
+    else{
+      throw new XMPexception("UNKNOWN DATA TYPE of the 1st argument in xmp_atomic_define().");
+    }
+
+    XMPcoarray coarray = _globalDecl.getXMPcoarray(coarrayName, block);
+    XobjList funcArgs  = Xcons.List(coarray.getDescId());
+
+    if(coarray == null)
+      throw new XMPexception("cannot find coarray '" + coarrayName + "'");
+
+    // Two arguments in xmp_atomic_define() must be int-type
+    if(coarray.getElmtType() != Xtype.intType || localExpr.Type() != Xtype.intType)
+      throw new XMPexception("An argument of atomic_define() must be an int-type.");
+
+    // i.e. Only 1 element can be used.
+    if(!isOneElement(coarrayExpr) || !isOneElement(localExpr))
+      throw new XMPexception("An argument of atomic_define() must be a scalar coarray or coindexed object.");
+
+    // Add offset
+    funcArgs.add(getCoarrayOffset(coarrayExpr, coarray));
+
+    // Get Coarray Dims
+    int imageDims = (coarrayExpr.Opcode() == Xcode.CO_ARRAY_REF)? coarray.getImageDim() : 0;
+    String funcName = "_XMP_atomic_define_" + imageDims;
+    Ident funcId    = _globalDecl.declExternFunc(funcName);
+    for(int i=0;i<imageDims;i++)
+      funcArgs.add(coarrayExpr.getArg(1).getArg(i));
+
+    if(localExpr.Opcode() == Xcode.INT_CONSTANT){
+      funcArgs.add(localExpr);
+      funcArgs.add(Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0))); // NULL
+      funcArgs.add(Xcons.IntConstant(0));                                // offset (This value is not used)
+    }
+    else{
+      // When localExpr is coarray, descriptor of the coarray is added to argument
+      if(isCoarray(localExpr, block)){
+        if(localExpr.Opcode() == Xcode.VAR){
+          funcArgs.add(rewriteVarRef(localExpr, block, true));
+        }
+        else{ //  else if(localExpr.Opcode() == Xcode.ARRAY_REF){
+          funcArgs.add(rewriteArrayRef(localExpr, block));
+        }
+        String localName = (localExpr.Opcode() == Xcode.VAR)? localExpr.getName() : localExpr.getArg(0).getName();
+        funcArgs.add(_globalDecl.getXMPcoarray(localName, block).getDescId());
+	funcArgs.add(getCoarrayOffset(localExpr, _globalDecl.getXMPcoarray(localName, block)));
+      }
+      else{
+        funcArgs.add(localExpr);
+        funcArgs.add(Xcons.Cast(Xtype.voidPtrType, Xcons.IntConstant(0))); // NULL
+	funcArgs.add(Xcons.IntConstant(0));                                // offset (This value is not used)
+      }
+    }
+    
+    // The variable must be an int-type
+    funcArgs.add(Xcons.SizeOf(Xtype.intType));
+    
+    return funcId.Call(funcArgs);
+  }
+  
   private Xobject rewriteXmpDescOf(Xobject myExpr, Block block) throws XMPexception {
     String entityName = myExpr.getArg(0).getName();
     XMPobject entity = _globalDecl.getXMPobject(entityName, block);
@@ -1362,15 +1703,25 @@ public class XMPrewriteExpr {
   private Xobject getCoarrayOffset(Xobject myExpr, XMPcoarray coarray){
     // "a[N][M][K]" is defined as a coarray.
     // If a[i][j][k] is referred, this function returns "(i * M * K) + (j * K) + (k)"
+
     if(myExpr.Opcode() == Xcode.VAR){
       return Xcons.Int(Xcode.INT_CONSTANT, 0);
     }
+    else if(myExpr.Opcode() == Xcode.ARRAY_REF){
+      myExpr = myExpr.getArg(1);
+    }
+    else if(myExpr.Opcode() == Xcode.CO_ARRAY_REF){
+      if(myExpr.getArg(0).Opcode() == Xcode.VAR)
+        return Xcons.Int(Xcode.INT_CONSTANT, 0);
+      else  // (myExpr.Opcode() == Xcode.ARRAY_REF)
+        myExpr = myExpr.getArg(0).getArg(1);
+    }
 
     Xobject newExpr = null;
     for(int i=0; i<coarray.getVarDim(); i++){
       Xobject tmp = null;
       for(int j=coarray.getVarDim()-1; j>i; j--){
-        int size = coarray.getSizeAt(j);
+	  int size = (int)coarray.getSizeAt(j);
         if(tmp == null){
           tmp = Xcons.Int(Xcode.INT_CONSTANT, size);
         } else{
@@ -1385,9 +1736,11 @@ public class XMPrewriteExpr {
       if(tmp != null){
         var = Xcons.binaryOp(Xcode.MUL_EXPR, tmp, var);
       }
+
       if(newExpr == null){
         newExpr = var.copy();
-      } else{
+      }
+      else{
         newExpr = Xcons.binaryOp(Xcode.PLUS_EXPR, newExpr, var);
       }
     }
@@ -1411,7 +1764,7 @@ public class XMPrewriteExpr {
     }
     else{
       newExpr = Xcons.binaryOp(Xcode.PLUS_EXPR,
-			       _globalDecl.findVarIdent(XMP.COARRAY_ADDR_PREFIX_ + coarray.getName()),
+			       _globalDecl.findVarIdent(XMP.COARRAY_ADDR_PREFIX_ + coarray.getName()).Ref(),
 			       newExpr);
       newExpr = Xcons.PointerRef(newExpr);
     }
@@ -1668,53 +2021,41 @@ public class XMPrewriteExpr {
     return XMPrewriteExpr.createRewriteAlignedArrayFunc(alignedArray, arrayDimCount, args);
   }
 
-  public static void rewriteLoopIndexInLoop(Xobject expr, String loopIndexName,
-					    XMPtemplate templateObj, int templateIndex,
-                                            XMPglobalDecl globalDecl, Block block) throws XMPexception {
+  public static void rewriteLoopIndexInLoop(Xobject expr, String loopIndexName, XMPtemplate templateObj,
+                                            int templateIndex, XMPglobalDecl globalDecl, Block block) throws XMPexception {
     if (expr == null) return;
     topdownXobjectIterator iter = new topdownXobjectIterator(expr);
     for (iter.init(); !iter.end(); iter.next()) {
       Xobject myExpr = iter.getXobject();
       if (myExpr == null) {
         continue;
-      } else if (myExpr.isRewrittedByXmp()) {
+      }
+      else if (myExpr.isRewrittedByXmp()) {
         continue;
       }
+      
       switch (myExpr.Opcode()) {
       case VAR:
 	{
 	  if (loopIndexName.equals(myExpr.getSym())) {
 	    iter.setXobject(calcLtoG(templateObj, templateIndex, myExpr));
 	  }
-	} break;
+	}
+        break;
       case ARRAY_REF:
 	{
 	  XMPalignedArray alignedArray = globalDecl.getXMPalignedArray(myExpr.getArg(0).getSym(), block);
 	  if (alignedArray == null) {
 	    rewriteLoopIndexVar(templateObj, templateIndex, loopIndexName, myExpr);
-	  } else {
+	  }
+          else {
 	    myExpr.setArg(1, rewriteLoopIndexArrayRefList(templateObj, templateIndex, alignedArray,
 							  loopIndexName, (XobjList)myExpr.getArg(1)));
 	  }
-	} break;
+	}
+        break;
       case POINTER_REF:
 	{
-	  // Xobject addr_expr = myExpr.getArg(0);
-	  // if (addr_expr.Opcode() == Xcode.PLUS_EXPR){
-
-	  //   Xobject pointer = addr_expr.getArg(0);
-	  //   Xobject offset = addr_expr.getArg(1);
-
-	  //   if (pointer.Opcode() == Xcode.VAR){
-	  //     XMPalignedArray alignedArray = globalDecl.getXMPalignedArray(pointer.getSym(), block);
-	  //     if (alignedArray != null){
-	  // 	  addr_expr.setArg(0, alignedArray.getAddrId().Ref());
-	  // 	  addr_expr.setArg(1, rewriteLoopIndexArrayRef(templateObj, templateIndex, alignedArray, 0,
-	  // 						       loopIndexName, offset));
-	  //     }
-	  //   }
-	  // }
-
 	  XMPalignedArray alignedArray = null;
 	  XobjList indexList = Xcons.List();
 
@@ -1738,9 +2079,6 @@ public class XMPrewriteExpr {
 					     rewriteLoopIndexArrayRefList(templateObj, templateIndex, alignedArray,
 									  loopIndexName, indexList));
 	    iter.setXobject(newExpr);
-	    // myExpr.getArg(0).setArg(0, alignedArray.getAddrId().Ref());
-	    // myExpr.getArg(0).setArg(1, rewriteLoopIndexArrayRefList(templateObj, templateIndex, alignedArray,
-	    // 							    loopIndexName, indexList));
 	  }
 
 	  break;
@@ -1783,6 +2121,7 @@ public class XMPrewriteExpr {
 
     XobjList newArrayRefList = Xcons.List();
     int arrayDimIdx = 0;
+
     for (Xobject x : arrayRefList) {
       newArrayRefList.add(rewriteLoopIndexArrayRef(t, ti, a, arrayDimIdx, loopIndexName, x));
       arrayDimIdx++;
@@ -1798,7 +2137,8 @@ public class XMPrewriteExpr {
     if (arrayRef.Opcode() == Xcode.VAR) {
       if (loopIndexName.equals(arrayRef.getString())) {
         return calcShadow(t, ti, a, ai, arrayRef);
-      } else {
+      }
+      else {
         return arrayRef;
       }
     }
@@ -1898,8 +2238,24 @@ public class XMPrewriteExpr {
     for (iter2.init(); !iter2.end(); iter2.next()){
       Block block = iter2.getBlock();
       if (block.Opcode() == Xcode.OMP_PRAGMA){
-	Xobject clauses = ((PragmaBlock)block).getClauses();
+
+	PragmaBlock pragmaBlock = (PragmaBlock)block;
+
+	Xobject clauses = pragmaBlock.getClauses();
 	if (clauses != null) rewriteOmpClauses(clauses, (PragmaBlock)block, fb, localXMPsymbolTable);
+
+	if (pragmaBlock.getPragma().equals("PARALLEL_FOR")){
+	  BlockList body = pragmaBlock.getBody();
+	  if (body.getDecls() != null){
+	    BlockList newBody = Bcons.emptyBody(body.getIdentList().copy(), body.getDecls().copy());
+	    body.setIdentList(null);
+	    body.setDecls(null);
+	    newBody.add(Bcons.PRAGMA(Xcode.OMP_PRAGMA, pragmaBlock.getPragma(),
+				     pragmaBlock.getClauses(), body));
+	    pragmaBlock.replace(Bcons.COMPOUND(newBody));
+	  }
+	}
+
       }
     }
   }
@@ -1936,8 +2292,8 @@ public class XMPrewriteExpr {
           Xobject loop_var = null;
           BasicBlockIterator i = new BasicBlockIterator(pragmaBlock.getBody());
           for (Block b = pragmaBlock.getBody().getHead(); b != null; b = b.getNext()){
-            if (b.Opcode() == Xcode.F_DO_STATEMENT){
-              loop_var = ((FdoBlock)b).getInductionVar();
+            if (b.Opcode() == Xcode.FOR_STATEMENT){
+              loop_var = ((ForBlock)b).getInductionVar();
             }
           }
           if (loop_var == null) continue;
@@ -2105,7 +2461,7 @@ public class XMPrewriteExpr {
 	  String arraySizeName = "_ACC_size_" + arrayAddr.getSym();
           Ident arraySizeId = body.declLocalIdent(arraySizeName, Xtype.unsignedlonglongType);
 	  
-          Block getArraySizeFuncCall = _globalDecl.createFuncCallBlock("_XMP_get_array_total_elmts", Xcons.List(descId.Ref()));
+          Block getArraySizeFuncCall = _globalDecl.createFuncCallBlock("_XMP_coarray_get_total_elmts", Xcons.List(descId.Ref()));
           body.insert(Xcons.Set(arraySizeId.Ref(), getArraySizeFuncCall.toXobject()));
           
           XobjList arrayRef = Xcons.List(coarrayAddrRef, Xcons.List(Xcons.IntConstant(0), arraySizeId.Ref()));
@@ -2133,8 +2489,91 @@ public class XMPrewriteExpr {
       return arrayRef;
   }
 
+  void rewriteACCPragma(XobjectDef def){
+    Xobject x = def.getDef();
+    Xobject pragma = x.getArg(0);
 
-  public  void rewriteVarDecl(Xobject varDecl, boolean isLocal) {
+    if(! pragma.getString().equals("DECLARE")){
+      return;
+    }
+
+    XobjList clauses = (XobjList)x.getArg(1);
+    for(XobjArgs args = clauses.getArgs(); args != null; args = args.nextArgs()){ //for(Xobject clause : clauses){
+      Xobject clause = args.getArg();
+      Xobject clauseKind = clause.getArg(0);
+      if(! clauseKind.getString().equals("CREATE")){
+        XMP.fatal("'" + clauseKind.getString() + "' clause for coarray is not implemented yet");
+      }
+      XobjList clauseArgs = (XobjList)clause.getArg(1);
+      for(XobjArgs args2 = clauseArgs.getArgs(); args2 != null; args2 = args2.nextArgs()){ //for(Xobject var : clauseArgs){
+        Xobject var = args2.getArg();
+        String varName = var.getName();
+        XMPcoarray coarray = _globalDecl.getXMPcoarray(varName);
+        if(coarray == null) continue;
+
+        if(coarray.getVarId().getStorageClass() != StorageClass.EXTERN){
+          //add func call like "_XMP_coarray_malloc_do_acc(&(_XMP_COARRAY_DESC_a), &(_XMP_COARRAY_ADDR_DEV_a));"
+          Xtype elementType = coarray.getElmtType();
+          Ident devPointerId = _globalDecl.declStaticIdent(XMP.COARRAY_ADDR_PREFIX_ + "DEV_" + varName, Xtype.Pointer(elementType));
+          String funcName = "_XMP_coarray_malloc_do_acc";
+          XobjList funcArgs = Xcons.List(coarray.getDescId().getAddr(),
+                  devPointerId.getAddr());
+          _globalDecl.addGlobalInitFuncCall(funcName, funcArgs);
+
+          //add func call like "acc_map_data(_XMP_COARRAY_ADDR_a, _XMP_COARRAY_ADDR_DEV_a, _XMP_coarray_get_total_elmts(_XMP_COARRAY_DESC_a) * sizeof(int));"
+          Ident getTotalElmtFuncId = _globalDecl.declExternFunc("_XMP_coarray_get_total_elmts", Xtype.intType);
+          Xobject getTotalElmtFuncCall = getTotalElmtFuncId.Call(Xcons.List(coarray.getDescId().Ref()));
+          Ident hostPointerId = _globalDecl.findVarIdent(XMP.COARRAY_ADDR_PREFIX_ + varName);
+          funcName = "acc_map_data";
+          funcArgs = Xcons.List(hostPointerId.Ref(),
+                  devPointerId.Ref(),
+                  Xcons.binaryOp(Xcode.MUL_EXPR,
+                          Xcons.SizeOf(elementType),
+                          getTotalElmtFuncCall));
+          _globalDecl.addGlobalInitFuncCall(funcName, funcArgs);
+        }
+
+
+        //remove var from clause
+        clauseArgs.removeArgs(args2);
+      }
+      if(clauseArgs.isEmpty()){
+        //remove clause
+        clauses.removeArgs(args);
+      }
+    }
+
+    if(clauses.isEmpty()){
+      //delete directive
+      def.setDef(Xcons.List(Xcode.TEXT,
+              Xcons.String("/* OpenACC declare directive is removed by XcalableACC */")));
+    }
+  }
+
+  private boolean isUseDevice(String varName, Block block){
+    Ident varId = block.findVarIdent(varName);
+    if(varId == null) return false;
+    
+    for(Block b = block; b != null; b = b.getParentBlock()){
+      if(b.Opcode() != Xcode.ACC_PRAGMA) continue;
+      PragmaBlock pb = (PragmaBlock)b;
+      if(! pb.getPragma().equals("HOST_DATA")) continue;
+
+      for(Xobject clause : (XobjList)pb.getClauses()){
+        if(! clause.getArg(0).getString().equals("USE_DEVICE")) continue;
+
+        for(Xobject var : (XobjList)clause.getArg(1)){
+          if(var.getSym().equals(varName) && b.findVarIdent(varName) == varId){
+            return true;
+          }
+        }
+      }
+    }
+
+    return false;
+  }
+
+  public void rewriteVarDecl(Xobject varDecl, boolean isLocal) {
     assert(varDecl.Opcode() == Xcode.VAR_DECL);
 
     String varName = varDecl.getArg(0).getName();
@@ -2158,4 +2597,27 @@ public class XMPrewriteExpr {
     }
   }
 
+  private void addBarrier(FunctionBlock fb){
+
+    topdownBlockIterator iter = new topdownBlockIterator(fb);
+
+    for (iter.init(); !iter.end(); iter.next()) {
+
+      Block b = iter.getBlock();
+
+      // insert a barrier before each return statement
+      if (b.Opcode() == Xcode.RETURN_STATEMENT){
+    	Ident f = _globalDecl.declExternFunc("_XMP_barrier_EXEC", Xtype.Function(Xtype.voidType));
+    	b.insert(f.Call(Xcons.List()));
+      }
+	
+    }
+
+    // add a barrier at the end of the function
+    Ident f = _globalDecl.declExternFunc("_XMP_barrier_EXEC", Xtype.Function(Xtype.voidType));
+    BlockList bl = fb.getBody().getHead().getBody();
+    bl.add(f.Call(Xcons.List()));
+
+  }
+
 }
