@@ -1,24 +1,6 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -158,8 +140,13 @@ public class XMPshadow {
 
     if (isLocalPragma) {
       //XMPlocalDecl.addConstructorCall("_XMP_init_shadow", shadowFuncArgs, globalDecl, pb);
-      XMPlocalDecl.addConstructorCall2(fname, shadowFuncArgs, globalDecl, parentBlock);
-      
+      if (alignedArray.isStaticDesc()){
+	XMPlocalDecl.addConstructorCall2_staticDesc(fname, shadowFuncArgs, globalDecl, parentBlock,
+						    alignedArray.getFlagId(), false);
+      }
+      else {
+	XMPlocalDecl.addConstructorCall2(fname, shadowFuncArgs, globalDecl, parentBlock);
+      }      
     }
     else {
       globalDecl.addGlobalInitFuncCall(fname, shadowFuncArgs);
@@ -237,9 +224,18 @@ public class XMPshadow {
       }
 
       if (asyncId != null){
-	Ident funcId = globalDecl.declExternFunc("_XMP_reflect_async__");
-	XobjList funcArgs = Xcons.List(alignedArray.getDescId().Ref(), asyncId);
-	reflectFuncBody.add(Bcons.Statement(funcId.Call(funcArgs)));
+        Ident funcId1 = globalDecl.declExternFunc("xmpc_init_async");
+        XobjList funcArgs1 = Xcons.List(asyncId);
+
+	Ident funcId2 = globalDecl.declExternFunc("_XMP_reflect_async__");
+	XobjList funcArgs2 = Xcons.List(alignedArray.getDescId().Ref(), asyncId);
+
+        Ident funcId3 = globalDecl.declExternFunc("xmpc_start_async");
+        XobjList funcArgs3 = Xcons.List();
+
+        reflectFuncBody.add(Bcons.Statement(funcId1.Call(funcArgs1)));
+	reflectFuncBody.add(Bcons.Statement(funcId2.Call(funcArgs2)));
+        reflectFuncBody.add(Bcons.Statement(funcId3.Call(funcArgs3)));
       }
       else {
 	Ident funcId = globalDecl.declExternFunc("_XMP_reflect__");
