@@ -1,24 +1,6 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -31,10 +13,13 @@ public class XMPsymbolTable {
   private HashMap<String, XMPalignedArray> _XMPalignedArrayTable;
   private HashMap<String, XMPcoarray> _XMPcoarrayTable;
 
+  private HashMap<String, Integer> _XMPstaticDescTable;
+
   public XMPsymbolTable() {
     _XMPobjectTable = new HashMap<String, XMPobject>();
     _XMPalignedArrayTable = new HashMap<String, XMPalignedArray>();
     _XMPcoarrayTable = new HashMap<String, XMPcoarray>();
+    _XMPstaticDescTable = new HashMap<String, Integer>();
   }
 
   public void putXMPobject(XMPobject o) {
@@ -76,4 +61,13 @@ public class XMPsymbolTable {
   public XMPcoarray getXMPcoarray(String name) {
     return _XMPcoarrayTable.get(name);
   }
+
+  public void putStaticDesc(String name){
+    _XMPstaticDescTable.put(name, 0);
+  }
+
+  public boolean isStaticDesc(String name){
+    return _XMPstaticDescTable.containsKey(name);
+  }
+
 }
