@@ -1,27 +1,3 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
-
 package exc.xcalablemp;
 
 import exc.block.*;
@@ -59,13 +35,13 @@ public class XMPutil {
 
   public static String getXobjSymbolName(Xobject x) throws XMPexception {
     switch (x.Opcode()) {
-      case VAR:
-        return x.getSym();
-      case ARRAY_REF:
-      case SUB_ARRAY_REF:
-        return x.getArg(0).getSym();
-      default:
-        throw new XMPexception("cannot get the symbol name of " + x.toString());
+    case VAR:
+      return x.getSym();
+    case ARRAY_REF:
+    case SUB_ARRAY_REF:
+      return x.getArg(0).getSym();
+    default:
+      throw new XMPexception("cannot get the symbol name of " + x.toString());
     }
   }
 
@@ -103,7 +79,8 @@ public class XMPutil {
 
     if (loopIterTable == null) {
       return null;
-    } else {
+    }
+    else {
       return loopIterTable.get(indVarName);
     }
   }
@@ -117,11 +94,30 @@ public class XMPutil {
       } else {
         return arraySize * getArrayElmtCount(arrayType.getRef());
       }
-    } else {
+    }
+    else {
       return 1;
     }
   }
 
+  public static Xobject getArrayElmt(Xtype type, int dim) throws XMPexception {
+    if(dim < 0)
+      throw new XMPexception("int dim must be more than 0");
+
+    if(!type.isArray())
+      return Xcons.IntConstant(1);
+    
+    ArrayType arrayType = (ArrayType)type;
+    for(int i=0;i<dim;i++)
+      arrayType = (ArrayType)arrayType.getRef();
+
+    long arraySize = arrayType.getArraySize();
+    if(arraySize == 0 || arraySize == -1)
+      return arrayType.getArraySizeExpr();
+    else
+      return Xcons.IntConstant((int)arrayType.getArraySize());
+  }
+
   public static XMPpair<Ident, Xtype> findTypedVar(String name, PragmaBlock pb) throws XMPexception {
     Ident id = pb.findVarIdent(name);
     if (id == null)
