@@ -1,30 +1,22 @@
 /*
  * $tsukuba_Release: Omni OpenMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 // package exc.xcodeml;
 
 package exc.xcodeml;
 
+
+import static xcodeml.util.XmDomUtil.collectChildNodes;
+import static xcodeml.util.XmDomUtil.collectElementsExclude;
+import static xcodeml.util.XmDomUtil.getAttr;
+import static xcodeml.util.XmDomUtil.getAttrBool;
+import static xcodeml.util.XmDomUtil.getContent;
+import static xcodeml.util.XmDomUtil.getContentText;
+import static xcodeml.util.XmDomUtil.getElement;
+
 import java.io.Reader;
 import java.util.ArrayList;
 import java.util.List;
@@ -232,6 +224,7 @@ public class XcodeMLtools_C extends XcodeMLtools {
       return getChildList(n, xlist);
     }
 
+    case TEXT:
     case PRAGMA_LINE: {
       String contentText = getContentText(n);
       x = Xcons.List(code,
@@ -722,9 +715,17 @@ public class XcodeMLtools_C extends XcodeMLtools {
 
   /** process subArrayRef. */
   private XobjList enterSubArrayRef(Xcode code, Xtype type, Node subArrayRefNode) {
-    ArrayList<Node> childNodes = collectElementsExclude(subArrayRefNode,
-							"arrayAddr");
+
     Node arrayAddrNode = getElement(subArrayRefNode, "arrayAddr");
+    ArrayList<Node> childNodes;
+    if (arrayAddrNode != null){
+      childNodes = collectElementsExclude(subArrayRefNode, "arrayAddr");
+    }
+    else { // pointer
+      arrayAddrNode = getElement(subArrayRefNode, "Var");
+      childNodes = collectElementsExclude(subArrayRefNode, "Var");
+    }
+
     XobjList objList = enterAsXobjList(subArrayRefNode,
 				       code,
 				       type,
