@@ -1,26 +1,3 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 package exc.xcodeml;
 
 import static xcodeml.util.XmLog.fatal;
@@ -977,7 +954,6 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             break;
 
         // directive
-
         case PRAGMA_LINE:
             e = addChildNodes(createElement(name),
                               trans(xobj.getArg(0).getString()));
@@ -986,7 +962,6 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
             e = addChildNodes(createElement(name),
                               trans(xobj.getArg(0).getString()));
             break;
-
         case OMP_PRAGMA: {
             e = createElement(name);
 
@@ -1007,8 +982,10 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
 
                         addChildNode(g, trans(a.getArg(0).getString()));
 
-			Xobject vars = a.getArg(1);
-                        if (vars != null){
+			if (a.Nargs() > 1){
+
+			  Xobject vars = a.getArg(1);
+			  if (vars != null){
 
 			    if (vars instanceof XobjString){
 			      addChildNode(g, trans(vars));
@@ -1021,6 +998,8 @@ public class XmcXobjectToXcodeTranslator extends XmXobjectToXcodeTranslator {
 			      addChildNode(g, g1);
 			    }
 
+			  }
+
 			}
 
                         addChildNode(f1, g);
