@@ -1,24 +1,6 @@
 /*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
  *  $
  */
 
@@ -32,9 +14,19 @@ import java.util.*;
  * Madiator for coindexed object such as a(i,j)[k]
  */
 public class XMPcoindexObj {
+
+  // GET/PUT Interface types
+  // (see also 1libxmpf/include/xmpf_internal_coarray.h)
+  //final static int GetInterfaceType = 6;           // valid last implementation
+  final static int GetInterfaceType = 8;
+  //final static int PutInterfaceType = 7;           // valid last implementation
+  final static int PutInterfaceType = 8;
+
   // constants
-  final static String COARRAYPUT_PREFIX = "xmpf_coarray_put";
-  final static String COARRAYGET_PREFIX = "xmpf_coarray_get";
+  final static String COARRAYPUT_PREFIX = "xmpf_coarray_put";                // for Type 7
+  final static String COARRAYGET_PREFIX = "xmpf_coarray_get";                // for Type 6
+  final static String COARRAYGET_GENERIC_NAME = "xmpf_coarray_get_generic";  // for Type 8
+  final static String COARRAYPUT_GENERIC_NAME = "xmpf_coarray_put_generic";  // for Type 8
 
   // attributes
   String name;
@@ -50,26 +42,22 @@ public class XMPcoindexObj {
   //  CONSTRUCTOR
   //------------------------------
   public XMPcoindexObj(Xobject obj, XMPcoarray coarray) {
-    name = _getName(obj);
-    subscripts = _getSubscripts(obj);
-    cosubscripts = _getCosubscripts(obj);
-    //_setNameAndSubscripts(obj);
     this.obj = obj;
+    name = _getName(obj);
     this.coarray = coarray;
-    exprRank = _getExprRank();
-    if (subscripts == null && exprRank > 0)
-      subscripts = _wholeArraySubscripts(exprRank);
+    _initOthers();
   }
 
-  /* construct and find the instance of XMPcoarray
-   */
   public XMPcoindexObj(Xobject obj, ArrayList<XMPcoarray> coarrays) {
+    this.obj = obj;
     name = _getName(obj);
+    coarray = XMPcoarray.findCoarrayInCoarrays(name, coarrays);
+    _initOthers();
+  }
+
+  private void _initOthers() {
     subscripts = _getSubscripts(obj);
     cosubscripts = _getCosubscripts(obj);
-    //_setNameAndSubscripts(obj);
-    this.obj = obj;
-    coarray = _findCoarrayInCoarrays(name, coarrays);
     exprRank = _getExprRank();
     if (subscripts == null && exprRank > 0)
       subscripts = _wholeArraySubscripts(exprRank);
@@ -87,16 +75,8 @@ public class XMPcoindexObj {
     return name;
   }
 
-
   private Xobject _getSubscripts(Xobject obj) {
-    Xobject varRef = obj.getArg(0).getArg(0);
-    if (varRef.Opcode() == Xcode.F_ARRAY_REF)        // subarray
-      return varRef.getArg(1);
-    else if (varRef.Opcode() == Xcode.VAR)    // scalar or whole array
-      return null;
-    else
-      XMP.fatal("broken Xcode to describe a coindexed object");
-    return null;
+    return obj.getSubscripts();
   }
 
   private Xobject _getCosubscripts(Xobject obj) {
@@ -113,6 +93,8 @@ public class XMPcoindexObj {
     Xobject list = Xcons.List();
     for (int i = 0; i < exprRank; i++)
       list.add(Xcons.FindexRangeOfAssumedShape());
+    if (list.hasNullArg())
+      XMP.fatal("INTERNAL: generated null argument (_wholeArraySubscripts)");
 
     return list;
   }
@@ -140,51 +122,62 @@ public class XMPcoindexObj {
   }    
 
 
-  /* TEMPORARY VERSION
-   *  not conversion but only error checking
+  /*  Implicit type conversion if necessary
+   *  ex.  RHS -> int(RHS)
+   *  ex.  RHS -> real(RHS,8)
    */
-  private Xobject _convRhsType(Xobject rhs) {
-    Xtype lhsType = coarray.getFtype();
+  private Xobject _convRhsType(Xobject rhs)
+  {
+    String lhsTypeStr = coarray.getFtypeString();
     Xobject lhsKind = coarray.getFkind();
 
-    Xtype rhsType;
-    Xobject rhsKind;
+    if (lhsTypeStr == null) {
+      XMP.warning("No cast function for RHS was generated because of " +
+                  "unknown type of coarray LHS: " + coarray);
+      return rhs;
+    }
 
-    rhsType = rhs.Type();
+    Xtype rhsType = rhs.Type();
     if (rhsType.getKind() == Xtype.F_ARRAY)
       rhsType = rhsType.getRef();
-    rhsKind = rhsType.getFkind();
+    String rhsTypeStr = coarray.getFtypeString(rhsType.getBasicType());
+    Xobject rhsKind = rhsType.getFkind();
 
-    int ltype = (lhsType == null) ? 0 : (lhsType.getKind());
-    int rtype = (rhsType == null) ? 0 : (rhsType.getKind());
+    if (lhsKind == null) {      // case default kind
+      if (rhsKind == null && lhsTypeStr.equals(rhsTypeStr)) {
+        // same type and same default kind
+        return rhs;
+      }
 
-    int lkind = (lhsKind == null) ? 0 : (lhsKind.getInt());
-    int rkind = (rhsKind == null) ? 0 : (rhsKind.getInt());
-
-    if (ltype == 0 || rtype == 0) {
-      // doubtful
-      XMP.warning("Automatic type conversion will not be generated " +
-                  "in this coindexed assignment statement.");
-      return rhs;
+      //XMP.warning("Invoked automatic type conversion ("+lhsTypeStr+")", block);
+      return _convTypeByIntrinsicCall(lhsTypeStr, rhs);
     }
 
-    if (ltype != rtype) {
-      // error
-      XMP.error("current restriction: found coindexed assignment statement " +
-                "with implicit type conversion");
-      return rhs;
+    if (lhsTypeStr.equals(rhsTypeStr) && rhsKind != null) {
+      if (lhsKind.canGetInt() && rhsKind.canGetInt() &&
+          lhsKind.getInt() == rhsKind.getInt())
+        // same type and same kind
+        return rhs;
     }
 
-    if (lkind != 0 && rkind != 0 && lkind != rkind) {
-      // error
-      XMP.error("current restriction: found coindexed assignment statement " +
-                "with implicit type-kind conversion");
-      return rhs;
-    }
+    //XMP.warning("Invoked automatic type conversion ("+lhsTypeStr+" with kind)", block);
+    return _convTypeByIntrinsicCall(lhsTypeStr, rhs, lhsKind);
+  }
 
-    // Though it is still doubtful in the case (lkind == 0 || rkind == ).
+  private Xobject _convTypeByIntrinsicCall(String fname, Xobject expr)
+  {
+    //FunctionType ftype = new FunctionType(Xtype.FunspecifiedType, Xtype.TQ_FINTRINSIC);
+    FunctionType ftype = new FunctionType(Xtype.FintType, Xtype.TQ_FINTRINSIC);
+    Ident fident = getEnv().declIntrinsicIdent(fname, ftype);
+    return fident.Call(Xcons.List(expr));
+  }
 
-    return rhs;
+  private Xobject _convTypeByIntrinsicCall(String fname, Xobject expr, Xobject kind)
+  {
+    //FunctionType ftype = new FunctionType(Xtype.FunspecifiedType, Xtype.TQ_FINTRINSIC);
+    FunctionType ftype = new FunctionType(Xtype.FintType, Xtype.TQ_FINTRINSIC);
+    Ident fident = getEnv().declIntrinsicIdent(fname, ftype);
+    return fident.Call(Xcons.List(expr, kind));
   }
 
 
@@ -201,54 +194,145 @@ public class XMPcoindexObj {
     return count;
   }
 
-  private XMPcoarray _findCoarrayInCoarrays(String name,
-                                            ArrayList<XMPcoarray> coarrays) {
-    for (XMPcoarray coarray: coarrays) {
-      if (coarray.getName() == name) {
-        return coarray;
-      }
-    }
 
-    if (coarray == null)
-      XMP.error("INTERNAL: could not find coarray in coarrays. name=" + name);
-    return null;
+  //------------------------------
+  //  run: GET communication
+  //------------------------------
+  public Xobject toFuncRef() {
+    Xtype type = getType();
+    Xobject funcRef;
+    if (type.isStruct())
+      funcRef = toFuncRef_struct();
+    else
+      funcRef = toFuncRef_basic();
+    return funcRef;
+  }
+
+  private Xobject toFuncRef_struct() {
+    // transfer(get_as_string, obj)
+
+    // call runtime as character(len=1), dimension(sizeof(obj))
+    Xobject mold = Xcons.FcharacterConstant(Xtype.FcharacterType, " ", null);
+    Xobject funcRef = toFuncRef_core(mold);  
+
+    // cast operation
+    Ident transferId = declIntIntrinsicIdent("transfer");
+    Xobject castExpr = transferId.Call(Xcons.List(funcRef, getIdent()));
+
+    return castExpr;
+  }
+
+  private Xobject toFuncRef_basic() {
+    Xobject mold = getMoldObj();
+    return toFuncRef_core(mold);
   }
 
 
-  //------------------------------
-  //  run
-  //------------------------------
-  public Xobject toFuncRef() {
-    // type-6 used
-    Xobject mold = getObj().getArg(0).getArg(0);   // object w/o coindex
+  Xobject toFuncRef_core(Xobject mold) {
+    switch (GetInterfaceType) {
+    case 6:
+      return toFuncRef_core_type6(mold, COARRAYGET_PREFIX);
+    case 8:
+      return toFuncRef_core_type8(mold, COARRAYGET_GENERIC_NAME);
+    default:
+      XMP.fatal("INTERNAL: obsoleted Get Interface Type: " +
+                GetInterfaceType);
+      break;
+    }
+    return null;
+  }
+
+  private Xobject toFuncRef_core_type8(Xobject mold, String funcName) {
+    // type8 used
+    Xobject actualArgs = _makeActualArgs_type8(mold);
+
+    Ident funcIdent = getEnv().findVarIdent(funcName, null);
+    if (funcIdent == null) {
+      Xtype baseType = new BasicType(BasicType.F_NUMERIC_ALL);
+                                            // ^^^^^^^^^^^^^ 
+                                            // should be F_ALL or F_GENERIC if possible
+      Xtype funcType = Xtype.Function(baseType);
+      funcIdent = getEnv().declExternIdent(funcName, funcType);
+    }                                           
+    Xobject funcRef = funcIdent.Call(actualArgs);
+    return funcRef;
+  }
+
+  private Xobject toFuncRef_core_type6(Xobject mold, String funcPrefix) {
+    // type6 used
     Xobject actualArgs = _makeActualArgs_type6(mold);
 
     Xtype xtype = getType().copy();
     xtype.removeCodimensions();
 
-    String funcName = COARRAYGET_PREFIX + exprRank + "d";
+    String funcName = funcPrefix + exprRank + "d";
     Ident funcIdent = getEnv().findVarIdent(funcName, null);
     if (funcIdent == null) {
+      // bug460: funcIdent could not find because the module declaring the
+      // name is not defined in the same file.
       Xtype baseType = _getBasicType(xtype);   // regard type as its basic type
                                                //// RETHINK! It might be neutral type?
       Xtype funcType = Xtype.Function(baseType);
       funcIdent = getEnv().declExternIdent(funcName, funcType);
+      // This workaround for bug460 does not work well.
+      //funcIdent = Ident.Fident(funcName, null);
     }                                           
 
     Xobject funcRef = funcIdent.Call(actualArgs);
     return funcRef;
   }
 
+
+  //------------------------------
+  //  run: PUT communication
+  //------------------------------
   public Xobject toCallStmt(Xobject rhs, Xobject condition) {
-    // type-6 used
+    Xtype type = getType();
+    Xobject subrStmt;
+    Xobject mold;
+
+    if (type.isStruct()) {
+      XMP.fatal("Not supported type of coarray: " + getName());
+      return null;
+    } else {
+      mold = getMoldObj();
+    }
+
+    switch (PutInterfaceType) {
+    case 8:
+      return toCallStmt_type8(mold, rhs, COARRAYPUT_GENERIC_NAME);
+    case 7:
+      return toCallStmt_type7(rhs, condition, COARRAYPUT_PREFIX);
+    default:
+      XMP.fatal("INTERNAL: obsoleted Get Interface Type: " +
+                PutInterfaceType);
+      break;
+    }
+    return null;
+  }
+
+  public Xobject toCallStmt_type8(Xobject mold, Xobject rhs, String subrName) {
+    // type8 used
+    Xobject actualArgs = _makeActualArgs_type8(mold, rhs);
+
+    Ident subrIdent = getEnv().findVarIdent(subrName, null);
+    if (subrIdent == null) {
+      subrIdent = getEnv().declExternIdent(subrName,
+                                           Xtype.FexternalSubroutineType);
+    }
+    Xobject subrCall = subrIdent.callSubroutine(actualArgs);
+    return subrCall;
+  }
+
+  public Xobject toCallStmt_type7(Xobject rhs, Xobject condition, String subrPrefix) {
+    // type7 used
     Xobject actualArgs =
-      _makeActualArgs_type6(_convRhsType(rhs),
-                            condition);
+      _makeActualArgs_type7(_convRhsType(rhs), condition);
 
     // "scalar" or "array" or "spread" will be selected.
     String pattern = _selectCoarrayPutPattern(rhs);
 
-    String subrName = COARRAYPUT_PREFIX + "_" + pattern;
+    String subrName = subrPrefix + "_" + pattern;
     //// I'm not clear why this is OK and the case xmpf_coarray_proc_init is 
     //// not OK with the similar interface blocks.
     Ident subrIdent = getEnv().findVarIdent(subrName, null);
@@ -280,6 +364,129 @@ public class XMPcoindexObj {
   }
 
 
+  /* generate actual arguments
+   * cf. libxmpf/src/xmpf_coarray_put.c
+   *
+   * Type-8:
+   *   utilize F90 generic-name interface
+   *     COARRAYGET_GENERIC_NAME(descPtr, coindex, mold) result(dst)
+   *     COARRAYPUT_GENERIC_NAME(descPtr, coindex, mold, src)
+   *       integer(8), intent(in)            :: descPtr
+   *       integer, intent(in)               :: coindex
+   *       anytype&kind_anydim, intent(in)   :: mold
+   *         local data corresponding to the coindexed remote object/variable
+   *       same type&kind, same rank as mold :: src
+   *       same type&kind, same rank as mold :: dst
+   * Type-7:
+   *   add baseAddr to Type-6 as an extra argument 
+   *   in order to tell the optimization compiler the data will be referred.
+   * Type-6:
+   *       (void *descPtr, void* baseAddr, int element, int image,
+   *        [void* rhs, int scheme,] int exprRank,
+   *        void* nextAddr1, int count1,
+   *        ...
+   *        void* nextAddrN, int countN )
+   *   where N is rank of the reference (0<=N<=15 in Fortran 2008).
+   */
+
+  // for subroutine atrimc_define
+  public Xobject makeActualArgs(Xobject src) {
+    return _makeActualArgs_type8(getMoldObj(), src);
+  }
+
+  private Xobject _makeActualArgs_type8(Xobject mold, Xobject src) {
+    Xobject actualArgs = _makeActualArgs_type8();
+    actualArgs.add(mold);
+    actualArgs.add(src);
+    return actualArgs;
+  }
+  private Xobject _makeActualArgs_type8(Xobject mold) {
+    Xobject actualArgs = _makeActualArgs_type8();
+    actualArgs.add(mold);
+    return actualArgs;
+  }
+  private Xobject _makeActualArgs_type8() {
+    XMPcoarray coarray = getCoarray();
+    Xobject baseAddr = getBaseAddr();
+    Xobject descPtr = coarray.getDescPointerIdExpr(baseAddr);
+    Xobject coindex = coarray.getImageIndex(baseAddr, cosubscripts);
+    Xobject actualArgs = Xcons.List(descPtr, coindex);
+
+    if (actualArgs.hasNullArg())
+      XMP.fatal("INTERNAL: generated null argument (_makeActualArgs_type8)");
+
+    return actualArgs;
+  }
+
+  private Xobject _makeActualArgs_type7(Xobject addArg1) {
+    return _makeActualArgs_type7(addArg1, null);
+  }
+  private Xobject _makeActualArgs_type7(Xobject addArg1, Xobject addArg2) {
+    Xobject actualArgs = _makeActualArgs_type6(addArg1, addArg2);
+    Xobject coarrayName =  Xcons.FvarRef(coarray.getIdent());
+    actualArgs.add(coarrayName);
+    return actualArgs;
+  }
+
+  private Xobject _makeActualArgs_type6(Xobject addArg1) {
+    return _makeActualArgs_type6(addArg1, null);
+  }
+  private Xobject _makeActualArgs_type6(Xobject addArg1, Xobject addArg2) {
+    XMPcoarray coarray = getCoarray();
+
+    Xobject baseAddr = getBaseAddr();
+    Xobject descPtr = coarray.getDescPointerIdExpr(baseAddr);
+    Xobject locBaseAddr = getBaseAddr_type6();
+    Xobject element = coarray.getElementLengthExpr();
+    Xobject image = coarray.getImageIndex(baseAddr, cosubscripts);
+    Xobject actualArgs =
+      Xcons.List(descPtr, locBaseAddr, element, image);
+
+    if (addArg1 != null)
+      actualArgs.add(addArg1);
+    if (addArg2 != null)
+      actualArgs.add(addArg2);
+
+    actualArgs.add(Xcons.IntConstant(exprRank));
+
+    int hostRank = coarray.getRank();
+    for (int i = 0; i < hostRank; i++) {
+      if (isTripletIndex(i)) {
+        actualArgs.add(getNeighboringAddr_type6(i));
+        actualArgs.add(getSizeFromTriplet(i));
+      }
+    }
+
+    if (actualArgs.hasNullArg())
+      XMP.fatal("INTERNAL: generated null argument (_makeActualArgs_type6)");
+
+    return actualArgs;
+  }
+
+
+  // TEMPORARY VERSION (for Type 6)
+  private Xtype _getBasicType(Xtype xtype) {
+
+    int baseTypeCode = 0;
+    switch (xtype.getKind()) {
+    case Xtype.F_ARRAY:
+      baseTypeCode = xtype.getRef().getBasicType();
+      break;
+    case Xtype.BASIC:
+      baseTypeCode = xtype.getBasicType();
+      break;
+    case Xtype.STRUCT:
+      XMP.fatal("internal error: STRUCT unsupported in _getTypeSuffix()");
+      break;
+    default:
+      XMP.fatal("internal error: unexpected kind in _getTypeSuffix(): xtype.getKind()");
+      break;
+    }
+
+    return new BasicType(baseTypeCode);
+  }
+
+
   /* generate actual arguments
    * cf. libxmpf/src/xmpf_coarray_put.c
    *
@@ -319,98 +526,38 @@ public class XMPcoindexObj {
       }
     }
 
-    // null check
-    for (Xobject arg: (XobjList)actualArgs) {
-      if (arg == null)
-        XMP.error("internal error: " + 
-                  "null augument generated in _makeActualArgs_type5()");
-    }
+    if (actualArgs.hasNullArg())
+      XMP.fatal("INTERNAL: generated null argument (_makeActualArgs_type5)");
 
     return actualArgs;
   }
 
 
-  /* generate actual arguments
-   * cf. libxmpf/src/xmpf_coarray_put.c
-   *
-   * Type-6:
-   *       (void *descPtr, void* baseAddr, int element, int image,
-   *        [void* rhs, int scheme,] int exprRank,
-   *        void* nextAddr1, int count1,
-   *        ...
-   *        void* nextAddrN, int countN )
-   *   where N is rank of the reference (0<=N<=15 in Fortran 2008).
-   */
-  private Xobject _makeActualArgs_type6(Xobject addArg1) {
-    return _makeActualArgs_type6(addArg1, null);
-  }
-
-  private Xobject _makeActualArgs_type6(Xobject addArg1, Xobject addArg2) {
-    XMPcoarray coarray = getCoarray();
-
-    Xobject baseAddr = getBaseAddr();
-    Xobject descPtr = coarray.getDescPointerIdExpr(baseAddr);
-    Xobject locBaseAddr = getBaseAddr_type6();
-    Xobject element = coarray.getElementLengthExpr();
-    Xobject image = coarray.getImageIndex(baseAddr, cosubscripts);
-    Xobject actualArgs =
-      Xcons.List(descPtr, locBaseAddr, element, image);
-
-    if (addArg1 != null)
-      actualArgs.add(addArg1);
-    if (addArg2 != null)
-      actualArgs.add(addArg2);
-
-    actualArgs.add(Xcons.IntConstant(exprRank));
-
-    int hostRank = coarray.getRank();
-    for (int i = 0; i < hostRank; i++) {
-      if (isTripletIndex(i)) {
-        actualArgs.add(getNeighboringAddr_type6(i));
-        actualArgs.add(getSizeFromTriplet(i));
-      }
-    }
-
-    // null check
-    for (Xobject arg: (XobjList)actualArgs) {
-      if (arg == null)
-        XMP.error("internal error: " + 
-                  "null augument generated in _makeActualArgs_type6()");
-    }
-
-    return actualArgs;
-  }
-
-
-  // TEMPORARY VERSION
-  private Xtype _getBasicType(Xtype xtype) {
-
-    int baseTypeCode = 0;
-    switch (xtype.getKind()) {
-    case Xtype.F_ARRAY:
-      baseTypeCode = xtype.getRef().getBasicType();
-      break;
-    case Xtype.BASIC:
-      baseTypeCode = xtype.getBasicType();
-      break;
-    case Xtype.STRUCT:
-      XMP.error("internal error: STRUCT unsupported in _getTypeSuffix()");
-      break;
-    default:
-      XMP.error("internal error: unexpected kind in _getTypeSuffix(): xtype.getKind()");
-      break;
-    }
-
-    return new BasicType(baseTypeCode);
-  }
-
-
   //------------------------------
   //  inquirement and evaluation
   //------------------------------
+  public Xobject getImageIndex() {
+    return coarray.getImageIndex(getBaseAddr(), cosubscripts);
+  }
+
   public Boolean isScalarIndex(int i) {
     Xobject subscr = subscripts.getArg(i);
-    return (subscr.Opcode() == Xcode.F_ARRAY_INDEX);
+    if (subscr.Opcode() != Xcode.F_ARRAY_INDEX)
+      return (subscr.getFrank(getBlock()) == 0);
+    return false;
+  }
+
+  public Boolean isVectorIndex(int i) {
+    Xobject subscr = subscripts.getArg(i);
+    if (subscr.Opcode() != Xcode.F_ARRAY_INDEX) {
+      switch (subscr.getFrank(getBlock())) {
+      case 0: return false;
+      case 1: return true;
+      default:
+        XMP.fatal("unexpected subscript expression: "+subscr);
+      }
+    }
+    return false;
   }
 
   public Boolean isTripletIndex(int i) {
@@ -475,7 +622,7 @@ public class XMPcoindexObj {
         start = coarray.getLbound(i);
       break;
     default:        // vector subscript is not supported
-      XMP.error("internal error: unexpected Xcode: "+subscr.Opcode());
+      XMP.fatal("internal error: unexpected Xcode: "+subscr.Opcode());
       start = null;
       break;
     }
@@ -496,7 +643,7 @@ public class XMPcoindexObj {
         end = coarray.getUbound(i);
       break;
     default:        // vector subscript is not supported
-      XMP.error("internal error: unexpected Xcode: "+subscr.Opcode());
+      XMP.fatal("internal error: unexpected Xcode: "+subscr.Opcode());
       end = null;
       break;
     }
@@ -516,7 +663,7 @@ public class XMPcoindexObj {
         stride = Xcons.IntConstant(1);
       break;
     default:        // vector subscript is not supported
-      XMP.error("internal error: unexpected Xcode: "+subscr.Opcode());
+      XMP.fatal("internal error: unexpected Xcode: "+subscr.Opcode());
       stride = null;
       break;
     }
@@ -580,7 +727,7 @@ public class XMPcoindexObj {
       break;
 
     default:        // vector subscript is not supported
-      XMP.error("internal error: maybe vector subscript. Xcode: "
+      XMP.fatal("internal error: maybe vector subscript. Xcode: "
                 + subscr.Opcode());
       size = null;
       break;
@@ -641,10 +788,28 @@ public class XMPcoindexObj {
     return obj;
   }
 
+  public String getName() {
+    return name;
+  }
+
+  public Xtype getType() {
+    return coarray.getType();
+  }
+
+  public Ident getIdent() {
+    return coarray.getIdent();
+  }
+
   public XMPcoarray getCoarray() {
     return coarray;
   }
 
+  // my mold object corresponding to the coindex object
+  //
+  public Xobject getMoldObj() {
+    return obj.getArg(0).getArg(0);
+  }
+
   public XMPenv getEnv() {
     return coarray.getEnv();
   }
@@ -661,18 +826,6 @@ public class XMPcoindexObj {
     return getBlockList().getDecls();
   }
 
-  public String getName() {
-    return getName();
-  }
-
-  public Xtype getType() {
-    return coarray.getType();
-  }
-
-  public Ident getIdent() {
-    return coarray.getIdent();
-  }
-
   public String toString() {
     return "XMPcoindexObj(" + obj.toString() + ")";
   }
