@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni XcalableMP Compiler 3 $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.xmpF;
@@ -61,12 +44,16 @@ public class XMPinfo
 
   // for gmove
   Xobject gmoveLeft,gmoveRight;
+  Xobject gmoveOpt;
 
   // for template_fix
   XMPtemplate template;
   XobjList sizeList;
   XobjList distList;
 
+  // for task
+  boolean nocomm_flag;
+
   public XMPinfo(XMPpragma pragma, XMPinfo parent, Block b, XMPenv env) {
     this.pragma = pragma;
     this.parent = parent;
@@ -158,6 +145,12 @@ public class XMPinfo
   
   public Xobject getGmoveRight() { return gmoveRight; }
 
+  public void setGmoveOpt(Xobject _gmoveOpt){
+    gmoveOpt = _gmoveOpt;
+  }
+
+  public Xobject getGmoveOpt() { return gmoveOpt; }
+
   public void setTemplateFix(XMPtemplate t, XobjList sList, XobjList dList){
     template = t;
     sizeList = sList;
@@ -168,4 +161,9 @@ public class XMPinfo
   public XobjList getSizeList() { return sizeList; }
   public XobjList getDistList() { return distList; }
 
+  public void setNocomm(Xobject nocomm){
+    nocomm_flag = (nocomm.getInt() == 1);
+  }
+
+  public boolean isNocomm() { return nocomm_flag; }
 }
