@@ -1,24 +1,7 @@
 /* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
+ * $TSUKUBA_Release: Omni XMP Compiler $
  * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
+ *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
  *  $
  */
 package exc.xmpF;
@@ -535,55 +518,78 @@ public class XMPrewriteExpr
 
       if (!a.isFullShadow(dim_i)){
 
-	// check this expression is ver+offset
 	Xobject e = i.getArg(0);
-	// we need normalize?
-	Xobject v = null;
-	Xobject offset = null;
-	if(e.isVariable()){
-	  v = e;
-	} else {
-	  switch(e.Opcode()){
-	  case PLUS_EXPR:
-	    if(e.left().isVariable()){
-	      v = e.left();
-	      offset = e.right();
-	    } else if(e.right().isVariable()){
-	      v = e.right();
-	      offset = e.left();
-	    }
-	    break;
-	  case MINUS_EXPR:
-	    if(e.left().isVariable()){
-	      v = e.left();
-	      offset = Xcons.unaryOp(Xcode.UNARY_MINUS_EXPR,e.right());
-	    }
-	    break;
+	Xobject x = null;
+
+	if (e.isVariable()){
+	  x = convertLocalIndex(e, dim_i, a, bb, block);
+	  if (localIndexOffset != null){
+	    x = Xcons.binaryOp(Xcode.PLUS_EXPR, x, localIndexOffset);
 	  }
 	}
-
-	if (v != null){
-	  v = convertLocalIndex(v, dim_i, a, bb, block);
-	  if (v != null){
-	      if (localIndexOffset != null){
-		  if (offset != null)
-		      offset = Xcons.binaryOp(Xcode.PLUS_EXPR,
-					      localIndexOffset, offset);
-		  else 
-		      offset = localIndexOffset;
-	      }
-	      if (offset != null)
-		  v = Xcons.binaryOp(Xcode.PLUS_EXPR, v, offset);
-
-	      Xobject x = a.convertOffset(dim_i);
-	      if (x != null)
-		  v = Xcons.binaryOp(Xcode.MINUS_EXPR, v, x);
-	      //v = Xcons.binaryOp(Xcode.MINUS_EXPR, v, a.convertOffset(dim_i));
-	      i.setArg(0, v);
-	      return i;
-	  }
+	else {
+	  x = e.copy();
+	  int cnt = convertLocalIndexInExpression(x, dim_i, a, bb, block, 0);
+	  if (cnt != 1) x = null;
 	}
 
+	if (x != null){
+	  Xobject y = a.convertOffset(dim_i);
+	  if (y != null)
+	    x = Xcons.binaryOp(Xcode.MINUS_EXPR, x, y);
+	  i.setArg(0, x);
+	  return i;
+	}
+
+	// // check this expression is ver+offset
+	// Xobject e = i.getArg(0);
+	// // we need normalize?
+	// Xobject v = null;
+	// Xobject offset = null;
+	// if(e.isVariable()){
+	//   v = e;
+	// } else {
+	//   switch(e.Opcode()){
+	//   case PLUS_EXPR:
+	//     if(e.left().isVariable()){
+	//       v = e.left();
+	//       offset = e.right();
+	//     } else if(e.right().isVariable()){
+	//       v = e.right();
+	//       offset = e.left();
+	//     }
+	//     break;
+	//   case MINUS_EXPR:
+	//     if(e.left().isVariable()){
+	//       v = e.left();
+	//       offset = Xcons.unaryOp(Xcode.UNARY_MINUS_EXPR,e.right());
+	//     }
+	//     break;
+	//   }
+	// }
+
+	// if (v != null){
+	//   v = convertLocalIndex(v, dim_i, a, bb, block);
+	//   if (v != null){
+	//       if (localIndexOffset != null){
+	// 	  if (offset != null)
+	// 	      offset = Xcons.binaryOp(Xcode.PLUS_EXPR,
+	// 				      localIndexOffset, offset);
+	// 	  else 
+	// 	      offset = localIndexOffset;
+	//       }
+	//       if (offset != null)
+	// 	  v = Xcons.binaryOp(Xcode.PLUS_EXPR, v, offset);
+
+	//       Xobject x = a.convertOffset(dim_i);
+	//       if (x != null)
+	// 	  v = Xcons.binaryOp(Xcode.MINUS_EXPR, v, x);
+	//       //v = Xcons.binaryOp(Xcode.MINUS_EXPR, v, a.convertOffset(dim_i));
+	//       i.setArg(0, v);
+	//       return i;
+	//   }
+	// }
+
       }
 
       Xobject x = null;
@@ -627,6 +633,55 @@ public class XMPrewriteExpr
     }
   }
 
+  private int convertLocalIndexInExpression(Xobject x, int dim_i, XMParray a,
+					    BasicBlock bb, Block block, int cnt){
+
+    Xobject v;
+
+    switch (x.Opcode()){
+
+    case PLUS_EXPR:
+
+      if (x.right().isVariable()){
+	v = convertLocalIndex(x.right(), dim_i, a, bb, block);
+	if (v != null){
+	  if (localIndexOffset != null)
+	    v = Xcons.binaryOp(Xcode.PLUS_EXPR, v, localIndexOffset);
+	  x.setRight(v);
+	  cnt++;
+	}
+      }
+      else {
+	cnt = convertLocalIndexInExpression(x.right(), dim_i, a, bb, block, cnt);
+      }
+
+      // fall through
+
+    case MINUS_EXPR:
+
+      // how to deal with the case for a(1 - (2 - i)) ???
+
+      if (x.left().isVariable()){
+	v = convertLocalIndex(x.left(), dim_i, a, bb, block);
+	if (v != null){
+	  if (localIndexOffset != null)
+	    v = Xcons.binaryOp(Xcode.PLUS_EXPR, v, localIndexOffset);
+	  x.setLeft(v);
+	  cnt++;
+	}
+      }
+      else {
+	cnt = convertLocalIndexInExpression(x.left(), dim_i, a, bb, block, cnt);
+      }
+
+      break;
+
+    }
+    
+    return cnt;
+
+  }
+
   private void insertSizeArray(Statement st, FunctionBlock fb){
 
     Xobject x = st.getExpr().getArg(0);
@@ -667,10 +722,11 @@ public class XMPrewriteExpr
       }
     }
 
-    if (param_list == null) return;
+    if (param_list == null || arg_list == null) return;
 
     int k = 0;
-    for (int i = 0; i < param_list.Nargs(); i++){
+    int nargs = Math.min(arg_list.Nargs(), param_list.Nargs());
+    for (int i = 0; i < nargs; i++){
 
       if (!param_list.getArg(i).Type().isFassumedShape()) continue;
 	
