@@ -1,26 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #ifndef _EXC_PLATFORM_H
 #define _EXC_PLATFORM_H
 
@@ -113,7 +90,7 @@ extern int      gettimeofday _ANSI_ARGS_((struct timeval *tp, struct timezone *t
 #   include <stdarg.h>
 #   define EXC_VARARGS(type, name) (type name, ...)
 #   define EXC_VARARGS_DEF(type, name) (type name, ...)
-#   define EXC_VARARGS_START(type, name, list) (va_start(list, name), name)
+#   define EXC_VARARGS_START(type, name, list) (va_start(list, name))
 #else
 #   include <varargs.h>
 #   ifdef __cplusplus
