@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include "acc_internal.h"
+#include <signal.h>
 
 void *_ACC_alloc(size_t size) {
   void *addr;
@@ -18,6 +19,7 @@ void _ACC_free(void *p) {
 
 void _ACC_fatal(const char *msg) {
   fprintf(stderr, "OpenACC runtime error: %s\n", msg);
+  raise(SIGABRT);
   exit(1);
 }
 
