@@ -1,27 +1,3 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
-
 #ifndef _XMP_CONSTANT
 #define _XMP_CONSTANT
 
@@ -34,6 +10,8 @@
 #define _XMP_N_MPI_TAG_GMOVE			10
 #define _XMP_N_MPI_TAG_REFLECT_LO		11
 #define _XMP_N_MPI_TAG_REFLECT_HI		12
+#define _XMP_N_MPI_TAG_POSTREQ			13
+#define _XMP_N_MPI_TAG_SYNCREQ			14
 
 // used by task
 #define _XMP_N_NODES_REF			20
@@ -79,12 +57,19 @@
 #define _XMP_N_REDUCE_EQV                       314
 #define _XMP_N_REDUCE_NEQV                      315
 #define _XMP_N_REDUCE_MINUS                     316
+#define _XMP_N_REDUCE_MAXLOC                    317
+#define _XMP_N_REDUCE_MINLOC                    318
 
 // defined in exc.xcalablemp.XMPshadow
 #define _XMP_N_SHADOW_NONE			400
 #define _XMP_N_SHADOW_NORMAL			401
 #define _XMP_N_SHADOW_FULL			402
 
+// defined in exc.xcalablemp.XMPcollective
+#define _XMP_N_GMOVE_NORMAL			400
+#define _XMP_N_GMOVE_IN				401
+#define _XMP_N_GMOVE_OUT			402
+
 // defined in exc.object.BasicType + 500
 #define _XMP_N_TYPE_BOOL			502
 #define _XMP_N_TYPE_CHAR			503
@@ -100,9 +85,13 @@
 #define _XMP_N_TYPE_FLOAT			513
 #define _XMP_N_TYPE_DOUBLE			514
 #define _XMP_N_TYPE_LONG_DOUBLE			515
+
+#ifdef __STD_IEC_559_COMPLEX__
 #define _XMP_N_TYPE_FLOAT_IMAGINARY		516
 #define _XMP_N_TYPE_DOUBLE_IMAGINARY		517
 #define _XMP_N_TYPE_LONG_DOUBLE_IMAGINARY	518
+#endif
+
 #define _XMP_N_TYPE_FLOAT_COMPLEX		519
 #define _XMP_N_TYPE_DOUBLE_COMPLEX		520
 #define _XMP_N_TYPE_LONG_DOUBLE_COMPLEX		521
@@ -123,4 +112,14 @@
 #define _XMP_GLOBAL_IO_ATOMIC			801
 #define _XMP_GLOBAL_IO_COLLECTIVE		802
 
+#define _XMP_ENTIRE_NODES			2000
+#define _XMP_EXECUTING_NODES			2001
+#define _XMP_PRIMARY_NODES			2002
+#define _XMP_EQUIVALENCE_NODES			2003
+
+// defined in exc.xmpF.XMPtransPragma
+#define XMP_N_GMOVE_ALL				0
+#define XMP_N_GMOVE_INDEX			1
+#define XMP_N_GMOVE_RANGE			2
+
 #endif // _XMP_CONSTANT
