@@ -1,29 +1,7 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.1 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
 #ifndef _XMP_DATA_STRUCT
 #define _XMP_DATA_STRUCT
 #include <mpi.h>
+#include <stdint.h>
 #include <stdbool.h>
 #include "xmp_constant.h"
 #if defined(OMNI_TARGET_CPU_KCOMPUTER) && defined(K_RDMA_REFLECT)
@@ -59,16 +37,17 @@ typedef struct _XMP_nodes_type {
   int is_member;
   int dim;
   int comm_size;
+  int attr;
 
   // enable when is_member is true
   int comm_rank;
   _XMP_comm_t *comm;
-  // -----------------------------
+  _XMP_comm_t *subcomm;  // enable when attr is _XMP_ENTIRE_NODES
+  int use_subcomm;       // enable when attr is _XMP_ENTIRE_NODES
 
   struct _XMP_nodes_type *inherit_nodes;
   // enable when inherit_nodes is not NULL
   _XMP_nodes_inherit_info_t *inherit_info;
-  // -------------------------------------
   _XMP_nodes_info_t info[1];
 } _XMP_nodes_t;
 
@@ -128,7 +107,7 @@ typedef struct _XMP_template_type {
 typedef struct _XMP_reflect_sched_type {
 
   int lo_width, hi_width;
-  _Bool is_periodic;
+  int is_periodic;
 
   MPI_Datatype datatype_lo;
   MPI_Datatype datatype_hi;
@@ -160,27 +139,64 @@ typedef struct _XMP_reflect_sched_type {
 #endif
 } _XMP_reflect_sched_t;
 
+// schedule of asynchronous reflect
+typedef struct _XMP_async_reflect_type {
+
+  int lwidth[_XMP_N_MAX_DIM], uwidth[_XMP_N_MAX_DIM];
+  _Bool is_periodic[_XMP_N_MAX_DIM];
+
+  MPI_Datatype *datatype;
+  MPI_Request *reqs;
+  int nreqs;
+
+} _XMP_async_reflect_t;
+
 // aligned array descriptor
 typedef struct _XMP_array_info_type {
   _Bool is_shadow_comm_member;
   _Bool is_regular_chunk;
   int align_manner;
 
-  int ser_lower;
-  int ser_upper;
-  int ser_size;
+  int ser_lower;  // Lower bound of a global array, and output value of xmp_array_lbound().
+  int ser_upper;  // Upper bound of a global array, and output value of xmp_array_lbound().
+  int ser_size;   // Size of a global array.
 
   // enable when is_allocated is true
-  int par_lower;
-  int par_upper;
-  int par_stride;
-  int par_size;
+  int par_lower;  // Lower bound of a global array on a node.
+  int par_upper;  // Upper bound of a global array on a node.
+  int par_stride; // Stride of a global array on a node.
+  int par_size;   // Size of a global array on a node.
 
-  int local_lower;
-  int local_upper;
-  int local_stride;
-  int alloc_size;
+  int local_lower;  // Lower bound of a local array
+  int local_upper;  // Upper bound of a local array
+  int local_stride; // Stride of a local array
+  int alloc_size;   // Number of elements of a local array
 
+  // ex1)
+  // #pragma xmp template t(0:19)
+  // #pragma xmp nodes p(4)
+  // #pragma xmp distribute t(block) onto p
+  // int a[20];
+  // #pragma xmp align a[i] with t(i)
+  //
+  // ser_lower   -> 0, ser_upper   -> 19, ser_size -> 20
+  // par_lower   -> 5*(xmp_node_num()-1),
+  // par_upper   -> 5*xmp_node_num()-1, par_size -> 5
+  // local_lower -> 0, local_upper -> 4, alloc_size -> 5
+
+  // ex2)
+  // #pragma xmp template t(0:19)
+  // #pragma xmp nodes p(4)
+  // #pragma xmp distribute t(block) onto p
+  // int a[20];
+  // #pragma xmp align a[i] with t(i)
+  // #pragma xmp shadow a[1]
+  //
+  // Values of ser_lower, ser_upper, ser_size, par_lower, par_upper, par_size are
+  // the same of ex1).
+  //
+  // local_lower -> 1, local_upper -> 5, alloc_size -> 7
+  
   int *temp0;
   int temp0_v;
 
@@ -214,6 +230,8 @@ typedef struct _XMP_array_type {
   int dim;
   int type;
   size_t type_size;
+  MPI_Datatype mpi_type;
+  int order;
 
   // enable when is_allocated is true
   void *array_addr_p;
@@ -231,6 +249,8 @@ typedef struct _XMP_array_type {
   unsigned long long total_elmts;
   // --------------------------------
 
+  _XMP_async_reflect_t *async_reflect;
+
   // FIXME do not use these members
   // enable when is_align_comm_member is true
   _XMP_comm_t *align_comm;
@@ -241,6 +261,10 @@ typedef struct _XMP_array_type {
   _Bool is_shrunk_template;
   _XMP_nodes_t *array_nodes;
 
+#ifdef _XMP_MPI3_ONESIDED
+  struct xmp_coarray *coarray;
+#endif
+
   _XMP_template_t *align_template;
   _XMP_array_info_t info[1];
 } _XMP_array_t;
@@ -261,6 +285,10 @@ typedef struct xmp_coarray{
   char **addr;      // Pointer to each node.
                     // e.g.) xmp_coarray.addr[2] is a pointer of an object on node 2.
 
+#ifdef _XMP_FJRDMA
+  uint64_t laddr;  // On the FJRDMA machines, xmp_coarray.addr[_XMP_world_rank] is not local address of a coarray,
+                   // Thus, "laddr" is defined the local address.
+#endif
   char *real_addr; // Pointer to local node.
                    // Note that xmp_coarray.addr[my_rank] may not be a pointer of an object.
 
@@ -270,32 +298,40 @@ typedef struct xmp_coarray{
   int coarray_dims; // Number of dimensions of coarray.
                     // e.g.) "int a[10][20]:[4][2][*]" is 2.
 
-  int *coarray_elmts; // Number of elements of each dimension of a coarray.
-                      // e.g.) If "int a[10][20]:[*]", coarray_elmts[0] is 10, coarray_elmts[1] is 20.
-                      //       If a coarray is scalar, coarray_elmts[0] is 1.
+  long *coarray_elmts; // Number of elements of each dimension of a coarray.
+                       // e.g.) When "int a[10][20]:[*]", coarray_elmts[0] is 10, coarray_elmts[1] is 20.
+                       //       If a coarray is scalar, coarray_elmts[0] is 1.
 
-  int *distance_of_coarray_elmts; // Distance between each dimension of coarray. A unit of the distance is Byte.
-                                  // e.g.) If "int a[10][20][30]:[*]", distance_of_coarray_elmts[0] is 2400 (20*30*sizeof(int)),
-                                  //       distance_of_coarray_elmts[1] is 120 (30*sizeof(int)),
-                                  //       distance_of_coarray_elmts[0] is 4 (sizeof(int)).
+  long *distance_of_coarray_elmts; // Distance between each dimension of coarray. A unit of the distance is Byte.
+                                   // e.g.) When "int a[10][20][30]:[*]", distance_of_coarray_elmts[0] is 2400 (20*30*sizeof(int)),
+                                   //       distance_of_coarray_elmts[1] is 120 (30*sizeof(int)),
+                                   //       distance_of_coarray_elmts[0] is 4 (sizeof(int)).
 
   int image_dims; // Number of dimensions of image set.
-                  // e.g.) If "int a[10][20]:[4][2][*]" is 3.
+                  // e.g.) When "int a[10][20]:[4][2][*]" is 3.
 
   int *distance_of_image_elmts; // Distance between each dimension of image set.
-                                // e.g.) If "int a[10][20]:[4][2][*]", distance_of_image_elmts[0] is 1,
+                                // e.g.) When "int a[10][20]:[4][2][*]", distance_of_image_elmts[0] is 1,
                                 //       distance_of_image_elmts[1] is 4, distance_of_image_elmts[2] is 8.
+#ifdef _XMP_MPI3_ONESIDED
+  MPI_Win win;
+  //#ifdef _XMP_XACC
+  char **addr_dev;
+  char *real_addr_dev;
+  MPI_Win win_acc;
+  //#endif
+#endif
 }_XMP_coarray_t;
 
 typedef struct _XMP_array_section{
-  long long start;
-  long long length;
-  long long stride;
-  long long elmts;    // Number of elements in each dimension
-  long long distance; // Distance between each dimension of an array.
-                      // e.g.) If "int a[10][20][30]", _XMP_array_section_t[0].distance is 20*30*sizeof(int),
-                      //       _XMP_array_section_t[1].distance is 30*sizeof(int), 
-                      //       _XMP_array_section_t[0].distance is sizeof(int),
+  long start;
+  long length;
+  long stride;
+  long elmts;    // Number of elements in each dimension
+  long distance; // Distance between each dimension of an array.
+                 // e.g.) When "int a[10][20][30]", _XMP_array_section_t[0].distance is 20*30*sizeof(int),
+                 //       _XMP_array_section_t[1].distance is 30*sizeof(int), 
+                 //       _XMP_array_section_t[0].distance is sizeof(int),
 } _XMP_array_section_t;
 
 typedef struct _XMP_gmv_desc_type
@@ -314,24 +350,76 @@ typedef struct _XMP_gmv_desc_type
   int *ub;
   int *st;
 
+#if defined(_XMP_XACC)
+  void *a_dev;
+#endif
 } _XMP_gmv_desc_t;
 
+// Regular Section Descriptor (RSD)
+// (l:u:s)
+typedef struct _XMP_rsd_type {
+  int l;
+  int u;
+  int s;
+} _XMP_rsd_t;
+
+// Basic Section Descriptor (BSD)
+// (l:u:b:c)
+// b = block width
+// c = cyclic length
+typedef struct _XMP_bsd_type {
+  int l;
+  int u;
+  int b;
+  int c;
+} _XMP_bsd_t;
+
+// Common Section Descriptor (CSD)
+// ((l_1,...,l_n):(u_1,...,u_n):s)
+// proposed in Gwan-Hwan Hwang , Jenq Kuen Lee, Communication set generations with CSD calculus and
+// expression-rewriting framework, Parallel Computing, v.25 n.9, p.1105-1130, Sept. 1999 
+typedef struct _XMP_csd_type {
+  int *l;
+  int *u;
+  int n;
+  int s;
+} _XMP_csd_t;
+
+// Communication Set
+// ((l_1:u_1),(l_2:u_2),...)
+typedef struct _XMP_comm_set_type {
+  int l;
+  int u;
+  struct _XMP_comm_set_type *next;
+} _XMP_comm_set_t;
+
 //
 // for asynchronous comms.
 //
 
+typedef struct _XMP_async_gmove {
+  int mode;
+  void *sendbuf;
+  void *recvbuf;
+  int recvbuf_size;
+  _XMP_array_t *a;
+  _XMP_comm_set_t *(*comm_set)[_XMP_N_MAX_DIM];
+} _XMP_async_gmove_t;
+
 typedef struct _XMP_async_comm {
-  int async_id;
-  int nreqs;
+  int   async_id;
+  int   nreqs;
+  int   nnodes;
+  _Bool is_used;
   MPI_Request *reqs;
+  _XMP_nodes_t **node;
+  _XMP_async_gmove_t *gmove;
   struct _XMP_async_comm *next;
 } _XMP_async_comm_t;
 
 #define _XMP_ASYNC_COMM_SIZE 511
-
-#define _XMP_MAX_ASYNC_REQS (4 * _XMP_N_MAX_DIM * 10)
-
-
+#define _XMP_MAX_ASYNC_REQS  (4 * _XMP_N_MAX_DIM * 10)
+#define _XMP_MAX_ASYNC_NODES (20)
 
 typedef struct _XMP_gpu_array_type {
   int gtol;
