@@ -11,6 +11,10 @@ void _XMP_post_wait_initialize()
   _xmp_gasnet_post_wait_initialize();
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post_wait_initialize();
+#elif _XMP_TCA
+  _xmp_tca_post_wait_initialize();
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post_wait_initialize();
 #else
   _XMP_fatal("Cannot use post function");
 #endif
@@ -24,6 +28,10 @@ void _XMP_post_1(const _XMP_nodes_t *node_desc, const int num1, const int tag)
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -35,6 +43,10 @@ void _XMP_post_2(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -48,6 +60,10 @@ void _XMP_post_3(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -62,6 +78,10 @@ void _XMP_post_4(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -79,6 +99,10 @@ void _XMP_post_5(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -98,6 +122,10 @@ void _XMP_post_6(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -119,6 +147,10 @@ void _XMP_post_7(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_post(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_post(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_post(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_post(node_num, tag);
 #endif
 }
 
@@ -128,6 +160,10 @@ void _XMP_wait_noargs()
   _xmp_gasnet_wait_noargs();
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_noargs();
+#elif _XMP_TCA
+  _xmp_tca_wait_noargs();
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_noargs();
 #endif
 }
 
@@ -138,6 +174,10 @@ void _XMP_wait_1(const _XMP_nodes_t *node_desc, const int num1, const int tag)
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -148,6 +188,10 @@ void _XMP_wait_node_1(const _XMP_nodes_t *node_desc, const int num1)
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -159,6 +203,10 @@ void _XMP_wait_2(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -170,6 +218,10 @@ void _XMP_wait_node_2(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -183,6 +235,10 @@ void _XMP_wait_3(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -196,6 +252,10 @@ void _XMP_wait_node_3(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -210,6 +270,10 @@ void _XMP_wait_4(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -224,6 +288,10 @@ void _XMP_wait_node_4(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -241,6 +309,10 @@ void _XMP_wait_5(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -257,6 +329,10 @@ void _XMP_wait_node_5(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -276,6 +352,10 @@ void _XMP_wait_6(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -295,6 +375,10 @@ void _XMP_wait_node_6(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
 
@@ -316,6 +400,10 @@ void _XMP_wait_7(const _XMP_nodes_t *node_desc, const int num1, const int num2,
   _xmp_gasnet_wait(node_num, tag);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait(node_num, tag);
+#elif _XMP_TCA
+  _xmp_tca_wait(node_num, tag);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait(node_num, tag);
 #endif
 }
 
@@ -337,5 +425,9 @@ void _XMP_wait_node_7(const _XMP_nodes_t *node_desc, const int num1, const int n
   _xmp_gasnet_wait_node(node_num);
 #elif _XMP_FJRDMA
   _xmp_fjrdma_wait_node(node_num);
+#elif _XMP_TCA
+  _xmp_tca_wait_node(node_num);
+#elif _XMP_MPI3_ONESIDED
+  _xmp_mpi_wait_node(node_num);
 #endif
 }
