@@ -12,6 +12,11 @@ static int _XMP_runtime_working = _XMP_N_INT_FALSE;
 int _XMPC_running = 1;
 int _XMPF_running = 0;
 
+void (*_xmp_pack_array)(void *buffer, void *src, int array_type, size_t array_type_size,
+			int array_dim, int *l, int *u, int *s, unsigned long long *d) = _XMPC_pack_array;
+void (*_xmp_unpack_array)(void *dst, void *buffer, int array_type, size_t array_type_size,
+			  int array_dim, int *l, int *u, int *s, unsigned long long *d) = _XMPC_unpack_array;
+
 void _XMP_init(int argc, char** argv)
 {
   if (!_XMP_runtime_working) {
@@ -19,12 +24,13 @@ void _XMP_init(int argc, char** argv)
     MPI_Comm_rank(MPI_COMM_WORLD, &_XMP_world_rank);
     MPI_Comm_size(MPI_COMM_WORLD, &_XMP_world_size);
 
-#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA)
+#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA) || defined(_XMP_TCA) || defined(_XMP_MPI3_ONESIDED)
     _XMP_onesided_initialize(argc, argv);
 #endif
-#ifdef _XMP_TCA
-    _XMP_init_tca();
+#ifdef _XMP_MPI3
+    _XMP_initialize_async_comm_tab();
 #endif
+    xmp_reduce_initialize();
   }
   _XMP_init_world(NULL, NULL);
   _XMP_runtime_working = _XMP_N_INT_TRUE;
@@ -34,7 +40,7 @@ void _XMP_init(int argc, char** argv)
 void _XMP_finalize(int return_val)
 {
   if (_XMP_runtime_working) {
-#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA)
+#if defined(_XMP_GASNET) || defined(_XMP_FJRDMA) || defined(_XMP_TCA) || defined(_XMP_MPI3_ONESIDED)
     _XMP_onesided_finalize(return_val);
 #endif
     _XMP_finalize_world();
