@@ -71,6 +71,8 @@ void _XMP_G2L(long long int global_idx,int *local_idx,
 /* From xmpf_misc.c */
 void xmpf_dbg_printf(char *fmt, ...);
 size_t _XMP_get_datatype_size(int datatype);
+void xmpf_finalize_all__(void);
+void xmpf_finalize_each__(void);
 
 
 /* From xmpf_gcomm.c */
@@ -82,6 +84,11 @@ extern int _XMP_world_size;
 extern int _XMP_world_rank;
 
 /* From xmp_align.c */
+void xmpf_array_alloc__(_XMP_array_t **a_desc, int *n_dim, int *type, _XMP_template_t **t_desc);
+void xmpf_array_dealloc__(_XMP_array_t **a_desc);
+void xmpf_align_info__(_XMP_array_t **a_desc, int *a_idx,
+		       int *lower, int *upper, int *t_idx, int *off);
+void xmpf_array_set_local_array__(_XMP_array_t **a_desc, void *array_addr, int *is_coarray);
 void _XMP_finalize_array_desc(_XMP_array_t *array);
 void _XMP_align_array_NOT_ALIGNED(_XMP_array_t *array, int array_index);
 void _XMP_align_array_DUPLICATION(_XMP_array_t *array, int array_index, int template_index,
@@ -167,25 +174,40 @@ void _XMP_sendrecv_ARRAY(int type, int type_size, MPI_Datatype *mpi_datatype,
                          int *dst_lower, int *dst_upper, int *dst_stride, unsigned long long *dst_dim_acc,
                          _XMP_array_t *src_array, int *src_array_nodes_ref,
                          int *src_lower, int *src_upper, int *src_stride, unsigned long long *src_dim_acc);
+void _XMP_gmove_BCAST_GSCALAR(void *dst_addr, void *src_addr, _XMP_array_t *array, int ref_index[]);
+void _XMP_gmove_SENDRECV_GSCALAR(void *dst_addr, void *src_addr,
+				 _XMP_array_t *dst_array, _XMP_array_t *src_array,
+				 int dst_ref_index[], int src_ref_index[]);
 
-extern void _XMP_gmove_array_array_common(_XMP_gmv_desc_t *gmv_desc_leftp, _XMP_gmv_desc_t *gmv_desc_rightp, int *dst_l, int *dst_u, int *dst_s, unsigned long long  *dst_d, int *src_l, int *src_u, int *src_s, unsigned long long *src_d);
+extern void _XMP_gmove_array_array_common(_XMP_gmv_desc_t *gmv_desc_leftp, _XMP_gmv_desc_t *gmv_desc_rightp, int *dst_l, int *dst_u, int *dst_s, unsigned long long  *dst_d, int *src_l, int *src_u, int *src_s, unsigned long long *src_d, int mode);
+extern void _XMP_gmove_inout_scalar(void *scalar, _XMP_gmv_desc_t *gmv_desc, int rdma_type);
 
 /* From xmp_runtime.c */
 void _XMP_init(int argc, char** argv);
 void _XMP_finalize(int);
 
 /* From xmp_reflect.c */
-void _XMP_set_reflect__(_XMP_array_t *a_desc, int dim, int lwidth, int uwidth, int is_periodic);
-void _XMP_reflect__(_XMP_array_t *a_desc);
-void _XMP_wait_async__(int async_id);
-void _XMP_reflect_async__(_XMP_array_t *a_desc, int async_id);
+extern void _XMP_set_reflect__(_XMP_array_t *a_desc, int dim, int lwidth, int uwidth, int is_periodic);
+extern void _XMP_reflect__(_XMP_array_t *a_desc);
+extern void _XMP_wait_async__(int async_id);
+extern void _XMP_reflect_async__(_XMP_array_t *a_desc, int async_id);
+extern _XMP_async_comm_t* _XMP_get_current_async();
+extern _XMP_async_comm_t* _XMP_get_async(int);
+extern void xmpc_init_async(int);
+extern void xmpc_start_async();
+
+/* From xmpf_pack.c */
+void _XMPF_pack_array(void *buffer, void *src, int array_type, size_t array_type_size,
+		      int array_dim, int *l, int *u, int *s, unsigned long long *d);
+void _XMPF_unpack_array(void *dst, void *buffer, int array_type, size_t array_type_size,
+			int array_dim, int *l, int *u, int *s, unsigned long long *d);
 
 /* From xmp_coarray.c */
 extern void _XMP_gasnet_not_continuous_put();
 extern void _XMP_gasnet_continuous_put();
 extern void _XMP_gasnet_not_continuous_get();
 extern void _XMP_gasnet_continuous_get();
-extern void _XMP_coarray_malloc_info_1(const int, const size_t);
+extern void _XMP_coarray_malloc_info_1(const long, const size_t);
 //extern void _XMP_coarray_malloc_info_2(const int, const int, const size_t);
 //extern void _XMP_coarray_malloc_info_3(const int, const int, const int, const size_t);
 //extern void _XMP_coarray_malloc_info_4(const int, const int, const int, const int, const size_t);
@@ -203,8 +225,9 @@ extern void _XMP_coarray_malloc_image_info_1();
 
 //extern void _XMP_coarray_malloc_do_f(void **, void *);
 extern void _XMP_coarray_malloc_do(void **, void *);
+extern void _XMP_coarray_regmem_do(void **, void *);
 
-extern void _XMP_coarray_rdma_coarray_set_1(const int, const int, const int);
+extern void _XMP_coarray_rdma_coarray_set_1(const long, const long, const long);
 //extern void _XMP_coarray_rdma_coarray_set_2(const int, const int, const int, const int, const int, const int);
 //extern void _XMP_coarray_rdma_coarray_set_3(const int, const int, const int, const int, const int, const int,
 //					    const int, const int, const int);
@@ -221,7 +244,7 @@ extern void _XMP_coarray_rdma_coarray_set_1(const int, const int, const int);
 //                                            const int, const int, const int, const int, const int, const int,
 //					    const int, const int, const int);
 
-extern void _XMP_coarray_rdma_array_set_1(const int, const int, const int, const int, const int);
+extern void _XMP_coarray_rdma_array_set_1(const long, const long, const long, const long, const long);
 //extern void _XMP_coarray_rdma_array_set_2(const int, const int, const int, const int, const int,
 //					  const int, const int, const int, const int, const int);
 //extern void _XMP_coarray_rdma_array_set_3(const int, const int, const int, const int, const int,
@@ -259,121 +282,17 @@ extern void _XMP_coarray_rdma_image_set_1(const int);
 //extern void _XMP_coarray_rdma_image_set_7(const int, const int, const int, const int, const int, const int, const int);
 
 //extern void _XMP_coarray_rdma_do_f(const int*, const void*, const void*, const void*);
-extern void _XMP_coarray_rdma_do(const int, const void*, const void*, const void *);
-//extern void _XMP_coarray_sync_all();
-//extern void _XMP_coarray_sync_memory();
+extern void _XMP_coarray_rdma_do(const int, void*, void*, void *);
 //extern size_t get_offset(const void *, const int);
-//extern void _XMP_coarray_shortcut_put(const int, const void*, const void*, const size_t, const size_t, const size_t);
-//extern void _XMP_coarray_shortcut_put_f(const int*, const void*, const void*, const size_t*, const size_t*, const size_t*);
-//extern void _XMP_coarray_shortcut_get(const int, const void*, const void*, const size_t, const size_t, const size_t);
-//extern void _XMP_coarray_shortcut_get_f(const int*, const void*, const void*, const size_t*, const size_t*, const size_t*);
+
+/* libxmp/include/xmp_func_decl.h */
+extern void _XMP_coarray_shortcut_put(const int, void*, const void*, const long, const long, const long, const long);
+//extern void _XMP_coarray_shortcut_put_f(const int*, void*, const void*, const long*, const long*, const long*, const long*);
+extern void _XMP_coarray_shortcut_get(const int, void*, const void*, const long, const long, const long, const long);
+//extern void _XMP_coarray_shortcut_get_f(const int*, void*, const void*, const long*, const long*, const long*, const long*);
 
 
 /******************************************\
     COARRAY
 \******************************************/
-#define BOOL   int
-#define TRUE   1
-#define FALSE  0
-
-#ifdef _XMP_FJRDMA
-#  define BOUNDARY_BYTE ((size_t)4)
-#else
-#  define BOUNDARY_BYTE ((size_t)1)
-#endif
-
-#define ROUND_UP(n,p)         (((((size_t)(n))-1)/(p)+1)*(p))
-#define ROUND_UP_BOUNDARY(n)  ROUND_UP((n),BOUNDARY_BYTE)
-
-#define MALLOC_UNIT  ((size_t)4)
-#define ROUND_UP_UNIT(n)      ROUND_UP((n),MALLOC_UNIT)
-
-/*-- parameters --*/
-#define DESCR_ID_MAX   250
-#define SMALL_WORK_SIZE_KB  10
-extern int _XMP_boundaryByte;     // communication boundary (bytes)
-
-/*-- codes --*/
-#define COARRAY_GET_CODE  700
-#define COARRAY_PUT_CODE  701
-
-/* xmpf_coarray.c */
-extern void _XMPF_coarray_init(void); 
-
-extern int _XMPF_coarrayMsg;          // default: debug message off
-extern void xmpf_coarray_msg_(int *sw);
-
-extern char *_XMPF_errmsg;   // to answer ERRMSG argument in Fortran
-extern void xmpf_copy_errmsg_(char *errmsg, int *msglen);
-
-extern int _XMPF_nowInTask(void);   // for restriction check
-extern void _XMPF_checkIfInTask(char *msgopt);   // restriction check
-extern void _XMPF_coarrayDebugPrint(char *format, ...);
-extern void _XMPF_coarrayFatal(char *format, ...);
-
-
-/* xmpf_coarray_alloc.c */
-extern void xmpf_coarray_malloc_(void **descPtr, char **crayPtr,
-                                 int *count, int *element, void **tag);
-extern void xmpf_coarray_free_(void **descPtr);
-
-extern void xmpf_coarray_malloc_pool_(void);
-extern void xmpf_coarray_share_pool_(void **descPtr, char **crayPtr,
-                                     int *count, int *element,
-                                     char *name, int *namelen);
-extern void xmpf_coarray_count_size_(int *count, int *element);
-
-extern void xmpf_coarray_prolog_(void **tag, char *name, int *namelen);
-extern void xmpf_coarray_epilog_(void **tag);
-
-extern void xmpf_coarray_get_descptr_(void **descPtr, char *baseAddr, void **tag);
-extern void xmpf_coarray_set_coshape_(void **descPtr, int *corank, ...);
-extern void xmpf_coarray_set_varname_(void **descPtr, char *name, int *namelen);
-
-extern int xmpf_coarray_get_image_index_(void **descPtr, int *corank, ...);
-
-extern int xmpf_coarray_allocated_bytes_(void);
-extern int xmpf_coarray_garbage_bytes_(void);
-
-extern void *_XMPF_get_coarrayDesc(void *descPtr);
-extern size_t _XMPF_get_coarrayOffset(void *descPtr, char *baseAddr);
-
-
-/* xmpf_coarray_lib.c */
-extern void _XMPF_set_this_image(void);
-extern int num_images_(void);
-extern int this_image_(void);
-//extern int xmpf_num_nodes_(void);
-//extern int xmpf_node_num_(void);
-
-extern void xmpf_sync_all_nostat_(void);
-extern void xmpf_sync_all_stat_(int *stat, char *msg, int *msglen);
-
-extern void xmpf_sync_memory_nostat_(void);
-extern void xmpf_sync_memory_stat_(int *stat, char *msg, int *msglen);
-
-extern void xmpf_sync_image_nostat_(int *image);
-extern void xmpf_sync_image_stat_(int *image,
-                                  int *stat, char *msg, int *msglen);
-extern void xmpf_sync_images_nostat_(int *images, int *size);
-extern void xmpf_sync_images_stat_(int *images, int *size,
-                                   int *stat, char *msg, int *msglen);
-extern void xmpf_sync_allimages_nostat_(void);
-extern void xmpf_sync_allimages_stat_(int *stat, char *msg, int *msglen);
-
-/* xmpf_coarray_put.c */
-extern void xmpf_coarray_put_scalar_(void **descPtr, char **baseAddr, int *element,
-                                     int *coindex, char *rhs, int *condition);
-extern void xmpf_coarray_put_array_(void **descPtr, char **baseAddr, int *element,
-                                    int *coindex, char *rhs, int *condition,
-                                    int *rank, ...);
-extern void xmpf_coarray_put_spread_(void **descPtr, char **baseAddr, int *element,
-                                     int *coindex, char *rhs, int *condition,
-                                     int *rank, ...);
-
-/* xmpf_coarray_get.c */
-extern void xmpf_coarray_get_scalar_(void **descPtr, char **baseAddr, int *element,
-                                     int *coindex, char *result);
-extern void xmpf_coarray_get_array_(void **descPtr, char **baseAddr, int *element,
-                                    int *coindex, char *result, int *rank, ...);
-
+#include "xmpf_internal_coarray.h"
