@@ -1,13 +1,10 @@
 #include "xmpf_internal.h"
 
-_Bool xmpf_test_task_on__(_XMP_object_ref_t **r_desc);
+//_Bool xmpf_test_task_on__(_XMP_object_ref_t **r_desc);
+void xmpf_create_task_nodes__(_XMP_nodes_t **n, _XMP_object_ref_t **r_desc);
+_Bool xmpf_test_task_on_nodes__(_XMP_nodes_t **n);
 void xmpf_end_task__(void);
-
-#ifdef _XMP_MPI3
-extern _Bool is_async;
-extern int _async_id;
-#endif
-
+void xmpf_nodes_dealloc__(_XMP_nodes_t **n_desc);
 //#define DBG 1
 
 //
@@ -30,7 +27,6 @@ void xmpf_reduction__(void *data_addr, int *count, int *datatype, int *op,
   _XMP_nodes_t *nodes;
 
   if (*r_desc){
-
     if (_XMP_is_entire(*r_desc)){
       if ((*r_desc)->ref_kind == XMP_OBJ_REF_NODES){
 	nodes = (*r_desc)->n_desc;
@@ -46,13 +42,23 @@ void xmpf_reduction__(void *data_addr, int *count, int *datatype, int *op,
       }
     }
     else {
-      if (xmpf_test_task_on__(r_desc)){
-	nodes = _XMP_get_execution_nodes();
-	if (*num_locs == 0) _XMP_reduce_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op);
-	else _XMPF_reduce_FLMM_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op, *num_locs,
-					    xmpf_reduction_loc_vars, xmpf_reduction_loc_types);
-	xmpf_end_task__();
+      /* if (xmpf_test_task_on__(r_desc)){ */
+      /* 	nodes = _XMP_get_execution_nodes(); */
+      /* 	if (*num_locs == 0) _XMP_reduce_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op); */
+      /* 	else _XMPF_reduce_FLMM_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op, *num_locs, */
+      /* 					    xmpf_reduction_loc_vars, xmpf_reduction_loc_types); */
+      /* 	xmpf_end_task__(); */
+      /* } */
+      _XMP_nodes_t *n;
+      xmpf_create_task_nodes__(&n, r_desc);
+      if (xmpf_test_task_on_nodes__(&n)){
+      	nodes = _XMP_get_execution_nodes();
+      	if (*num_locs == 0) _XMP_reduce_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op);
+      	else _XMPF_reduce_FLMM_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op, *num_locs,
+      					    xmpf_reduction_loc_vars, xmpf_reduction_loc_types);
+      	xmpf_end_task__();
       }
+      xmpf_nodes_dealloc__(&n);
     }
 
   }
@@ -62,43 +68,34 @@ void xmpf_reduction__(void *data_addr, int *count, int *datatype, int *op,
     else _XMPF_reduce_FLMM_NODES_ENTIRE(nodes, data_addr, *count, *datatype, *op, *num_locs,
 					xmpf_reduction_loc_vars, xmpf_reduction_loc_types);
   }
-
 }
 
-
 //
 // bcast
 //
-
 void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 			  _XMP_object_ref_t *from_desc, _XMP_object_ref_t *on_desc){
 
   _XMP_ASSERT(!on_desc || on_desc->ref_kind == XMP_OBJ_REF_NODES);
 
   int size = (int)_XMP_get_datatype_size(datatype);
-
   int root = 0;
 
   _XMP_RETURN_IF_SINGLE;
 
   // calc source nodes number
   if (from_desc){
-
-    if (from_desc->ref_kind != XMP_OBJ_REF_NODES){
+    if (from_desc->ref_kind != XMP_OBJ_REF_NODES)
       _XMP_fatal("Type of the From and ON clauses must be the same.");
-    }
 
     _XMP_nodes_t *from = from_desc->n_desc;
 
-    if (!from->is_member) {
+    if (!from->is_member) 
       _XMP_fatal("broadcast failed, cannot find the source node");
-    }
 
     if (on_desc){
-
-      if (on_desc->n_desc != from){
+      if (on_desc->n_desc != from)
 	_XMP_fatal("Node arrays in the FROM and ON clauses must be the same.");
-      }
 
       int acc_nodes_size = 1;
 
@@ -131,7 +128,6 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 	}
 	}
       }
-      
     }
     else {
       int acc_nodes_size = 1;
@@ -140,10 +136,8 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 	acc_nodes_size *= from->info[i].size;
       }
     }
-
   }
   else { /* no from clause, only check */
-
     if (on_desc){
       for (int i = 0; i < on_desc->n_desc->dim; i++){
 	switch (on_desc->subscript_type[i]){
@@ -165,19 +159,16 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 	  break;
       }
     }
-
   }
 
   _XMP_nodes_t *on;
-
-  if (on_desc){
-
-    if (_XMP_is_entire(on_desc)){
+  if(on_desc){
+    if(_XMP_is_entire(on_desc)){
       on = on_desc->n_desc;
 
 #ifdef _XMP_MPI3
-      if (is_async){
-	_XMP_async_comm_t *async = _XMP_get_or_create_async(_async_id);
+      if(xmp_is_async()){
+	_XMP_async_comm_t *async = _XMP_get_current_async();
 	MPI_Ibcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm),
 		   &async->reqs[async->nreqs]);
 	async->nreqs++;
@@ -186,13 +177,15 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 #endif
 	MPI_Bcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm));
     }
-    else {
-      if (xmpf_test_task_on__(&on_desc)){
+    else{
+      _XMP_nodes_t *n;
+      xmpf_create_task_nodes__(&n, &on_desc);
+      if(xmpf_test_task_on_nodes__(&n)){
 	on = _XMP_get_execution_nodes();
 
 #ifdef _XMP_MPI3
-	if (is_async){
-	  _XMP_async_comm_t *async = _XMP_get_or_create_async(_async_id);
+	if(xmp_is_async()){
+	  _XMP_async_comm_t *async = _XMP_get_current_async();
 	  MPI_Ibcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm),
 		     &async->reqs[async->nreqs]);
 	  async->nreqs++;
@@ -200,17 +193,23 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 	else
 #endif
 	  MPI_Bcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm));
-	xmpf_end_task__();
+      	xmpf_end_task__();
       }
+      
+#ifdef _XMP_MPI3
+      if(xmp_is_async())
+	_XMP_nodes_dealloc_after_wait_async(n);
+      else
+#endif
+	xmpf_nodes_dealloc__(&n);
     }
-
   }
-  else {
+  else{ // no on desc
     on = from_desc ? from_desc->n_desc : _XMP_get_execution_nodes();
 
 #ifdef _XMP_MPI3
-    if (is_async){
-      _XMP_async_comm_t *async = _XMP_get_or_create_async(_async_id);
+    if(xmp_is_async()){
+      _XMP_async_comm_t *async = _XMP_get_current_async();
       MPI_Ibcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm),
 		 &async->reqs[async->nreqs]);
       async->nreqs++;
@@ -219,10 +218,8 @@ void _XMPF_bcast_on_nodes(void *data_addr, int count, int datatype,
 #endif
       MPI_Bcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on->comm));
   }
-
 }
 
-
 void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
 			     _XMP_object_ref_t *from_desc, _XMP_object_ref_t *on_desc){
 
@@ -240,25 +237,19 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
 
   // calc source nodes number
   if (from_desc){
-
-    if (from_desc->ref_kind != XMP_OBJ_REF_TEMPL){
+    if (from_desc->ref_kind != XMP_OBJ_REF_TEMPL)
       _XMP_fatal("Type of the From and ON clauses must be the same.");
-    }
 
     from = from_desc->t_desc;
 
-    if (!from->is_owner) {
+    if (!from->is_owner)
       _XMP_fatal("broadcast failed, cannot find the source node");
-    }
 
-    if (on != from){
+    if (on != from)
       _XMP_fatal("Templates in the FROM and ON clauses must be the same.");
-    }
 
     for (int i = 0; i < from->dim; i++){
-
       int from_idx = -1;
-
       switch (on_desc->subscript_type[i]){
       case SUBSCRIPT_SCALAR:
 	if (from_desc->REF_INDEX[i] != on_desc->REF_INDEX[i]){
@@ -285,22 +276,17 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
 	if (from_idx < on_lb || from_idx > on_ub || (from_idx - on_lb) % on_st != 0){
 	  _XMP_fatal("A subscript of the from-template is out of the on-template bound");
 	}
-
 	break;
       }
       }
 
       int j = from->chunk[i].onto_nodes_index;
-      if (j != _XMP_N_NO_ONTO_NODES){
-	// 0-origin
+      if (j != _XMP_N_NO_ONTO_NODES){ // 0-origin
 	from_idx_in_nodes[j] = _XMP_calc_template_owner_SCALAR(from, j, (long long)from_idx);
       }
-
     }
-      
   }
   else { /* no from clause, only check */
-
     for (int i = 0; i < on->dim; i++){
       switch (on_desc->subscript_type[i]){
       case SUBSCRIPT_SCALAR:
@@ -320,7 +306,6 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
       }
       break;
     }
-
   }
 
   int root = 0;
@@ -338,8 +323,8 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
     }
 
 #ifdef _XMP_MPI3
-    if (is_async){
-      _XMP_async_comm_t *async = _XMP_get_or_create_async(_async_id);
+    if (xmp_is_async()){
+      _XMP_async_comm_t *async = _XMP_get_current_async();
       MPI_Ibcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on_nodes->comm), async->reqs);
       async->nreqs = 1;
     }
@@ -348,7 +333,9 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
       MPI_Bcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on_nodes->comm));
   }
   else {
-    if (xmpf_test_task_on__(&on_desc)){
+    _XMP_nodes_t *n;
+    xmpf_create_task_nodes__(&n, &on_desc);
+    if (xmpf_test_task_on_nodes__(&n)){
       on_nodes = _XMP_get_execution_nodes();
       
       if (from_desc){
@@ -364,8 +351,8 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
       }
 
 #ifdef _XMP_MPI3
-      if (is_async){
-	_XMP_async_comm_t *async = _XMP_get_or_create_async(_async_id);
+      if (xmp_is_async()){
+	_XMP_async_comm_t *async = _XMP_get_current_async();
 	MPI_Ibcast(data_addr, count*size, MPI_BYTE, root, *((MPI_Comm *)on_nodes->comm), async->reqs);
 	async->nreqs = 1;
       }
@@ -375,8 +362,13 @@ void _XMPF_bcast_on_template(void *data_addr, int count, int datatype,
 
       xmpf_end_task__();
     }
+#ifdef _XMP_MPI3
+    if(xmp_is_async())
+      _XMP_nodes_dealloc_after_wait_async(n);
+    else
+#endif
+      xmpf_nodes_dealloc__(&n);
   }
-
 }
 
 
@@ -406,15 +398,20 @@ void xmpf_barrier__(_XMP_object_ref_t **desc)
       }
     }
     else {
-      if (xmpf_test_task_on__(desc)){
-	_XMP_barrier_EXEC();
-	xmpf_end_task__();
+      /* if (xmpf_test_task_on__(desc)){ */
+      /* 	_XMP_barrier_EXEC(); */
+      /* 	xmpf_end_task__(); */
+      /* } */
+      _XMP_nodes_t *n;
+      xmpf_create_task_nodes__(&n, desc);
+      if (xmpf_test_task_on_nodes__(&n)){
+      	_XMP_barrier_EXEC();
+      	xmpf_end_task__();
       }
+      xmpf_nodes_dealloc__(&n);
     }
-
   }
   else {
     _XMP_barrier_EXEC();
   }
-
 }
