@@ -1,10 +1,12 @@
+#define NAMELEN 25
 #define NMAX 4
 #include <stdio.h>
 #include <stdlib.h>
 #include "xmp.h"
 
-extern int chk_int(int ierr);
+extern int chk_int(char name[], int ierr);
 
+char name[NAMELEN]="337";
 int n=NMAX;
 int a[n][n],b[n][n];
 #pragma xmp nodes p(2,2)
@@ -15,8 +17,8 @@ int a[n][n],b[n][n];
 
 int main(){
 
-  int i0,i1,ierr,irank;
-  irank=xmp_node_num();
+  int i0,i1,ierr;//,irank;
+  //  irank=xmp_node_num();
 
 #pragma xmp loop (i0,i1) on tx(i1,i0)
   for(i0=0;i0<n;i0++){
@@ -44,5 +46,7 @@ int main(){
   }
 
 #pragma xmp reduction (MAX:ierr)
-  chk_int(ierr);
+  chk_int(name, ierr);
+
+  return 0;
 }
