@@ -4,9 +4,13 @@ program test_matmul_Fjt
   call test_matmul_002()
   call test_matmul_003()
   call test_matmul_004()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_matmul_005()
+#endif
   call test_matmul_006()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_matmul_007()
+#endif
   call test_matmul_008()
   call test_matmul_009()
   call test_matmul_010()
@@ -14,25 +18,33 @@ program test_matmul_Fjt
   call test_matmul_012()
   call test_matmul_013()
   call test_matmul_014()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_matmul_015()
+#endif
   call test_matmul_016()
   call test_matmul_017()
   call test_matmul_018()
   call test_matmul_019()
   call test_matmul_020()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_matmul_021()
+#endif
   call test_matmul_022()
   call test_matmul_023()
   call test_matmul_024()
   call test_matmul_025()
   call test_matmul_026()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_matmul_027()
+#endif
   call test_matmul_028()
   call test_matmul_029()
   call test_matmul_030()
   call test_matmul_031()
   call test_matmul_032()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_matmul_033()
+#endif
   call test_matmul_101()
   call test_matmul_103()
   call test_matmul_104()
