@@ -1,8 +1,10 @@
 program test_transpose_Fjt
 
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_transpose_001()
 !  call test_transpose_002()
 !  call test_transpose_003()
+#endif
   call test_transpose_004()
   call test_transpose_005()
   call test_transpose_006()
@@ -10,49 +12,67 @@ program test_transpose_Fjt
   call test_transpose_008()
   call test_transpose_009()
   call test_transpose_010()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_transpose_011()
 !  call test_transpose_012()
+#endif
   call test_transpose_013()
   call test_transpose_014()
   call test_transpose_015()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_transpose_016()
+#endif
   call test_transpose_017()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_transpose_018()
 !  call test_transpose_019()
 !  call test_transpose_020()
+#endif
   call test_transpose_021()
   call test_transpose_022()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_transpose_023()
+#endif
   call test_transpose_030()
   call test_transpose_031()
   call test_transpose_032()
   call test_transpose_033()
   call test_transpose_034()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_transpose_035()
+#endif
   call test_transpose_036()
   call test_transpose_037()
   call test_transpose_038()
   call test_transpose_039()
   call test_transpose_040()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_transpose_041()
+#endif
   call test_transpose_042()
   call test_transpose_043()
   call test_transpose_044()
   call test_transpose_045()
   call test_transpose_046()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_transpose_047()
+#endif
   call test_transpose_048()
   call test_transpose_049()
   call test_transpose_050()
   call test_transpose_051()
   call test_transpose_052()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
   call test_transpose_053()
+#endif
   call test_transpose_101()
   call test_transpose_102()
   call test_transpose_104()
   call test_transpose_105()
   call test_transpose_106()
+#if ((XMP_MPI_VERSION >= 3) || (XMP_MPI_VERSION == 2 && XMP_MPI_SUBVERSION >= 2))
 !  call test_transpose_107()
+#endif
   call test_transpose_108()
 
 end program
@@ -1216,7 +1236,7 @@ subroutine test_transpose_037()
   integer,parameter:: M=23, N=31
   integer*4 a(M,N), b(N,M)
   integer error
-  integer m1(2)=(/13,18/)
+  integer m1(4)=(/5,8,8,10/)
 !$xmp nodes q(16)
 !$xmp nodes pa(2,4)=q(1:8)
 !$xmp nodes pb(2,2,2)=q(1:8)
@@ -1255,7 +1275,7 @@ subroutine test_transpose_038()
   integer,parameter:: M=23, N=31
   integer*8 a(M,N), b(N,M)
   integer error
-  integer m1(2)=(/13,23/)
+  integer m1(4)=(/5,8,10,13/)
 !$xmp nodes q(16)
 !$xmp nodes pa(2,4)=q(1:8)
 !$xmp nodes pb(8)=q(1:8)
@@ -1612,7 +1632,7 @@ subroutine test_transpose_047()
   integer,parameter:: M=23, N=31
   complex*16 a(M,N), b(N,M)
   integer error
-  integer m1(2)=(/1,22/)
+  integer m1(4)=(/1,2,3,17/)
 !$xmp nodes q(16)
 !$xmp nodes pa(2,4)=q(1:8)
 !$xmp nodes pb(2,2,2)=q(1:8)
@@ -1855,7 +1875,7 @@ subroutine test_transpose_053()
   integer,parameter:: M=23, N=31
   complex*16 a(M,N), b(N,M)
   integer error
-  integer m2(2)=(/16,21/)
+  integer m2(2)=(/11,16/)
   integer m3(2)=(/21,10/)
 !$xmp nodes q(16)
 !$xmp nodes pa(8)=q(1:8)
